#############################################################################
# menus.tcl Copyright 2004, 2005, 2006, 2007 r0lZ
# Build main menu and PGC Selector popup menus.
#
# This file is part of PgcEdit.
#
# PgcEdit is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# PgcEdit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#############################################################################

namespace eval ::menus {

proc build_main_menu {} {
	menu .mainmenu

	.mainmenu add cascade -label "File" -menu .mainmenu.file
	menu .mainmenu.file
	.mainmenu.file add command -label "Open DVD" -command {::main::Read_New_DVD} -accelerator "Ctrl+O"
	bind . <Control-o> ::main::Read_New_DVD
	.mainmenu.file add command -label "Reopen DVD" -command {::main::ReRead_DVD true true} -accelerator "Ctrl+R"
	bind . <Control-r> {::main::ReRead_DVD true true}
	.mainmenu.file add cascade -label "Recent DVDs" -menu .mainmenu.file.mru
	menu .mainmenu.file.mru
	build_mru_menu
	.mainmenu.file add command -label "Find BOVs" -command {if [info exists ::dvddir] {::menued::get_buttons_all_title_domains $::dvddir false} else bell}
	.mainmenu.file add separator
	.mainmenu.file add command -label "Save DVD" -command {::main::Write_DVD false} -accelerator "Ctrl+S"
	bind . <Control-s> {::main::Write_DVD false}
	.mainmenu.file add command -label "Save IFOs as" -command {::main::Write_DVD true}
	.mainmenu.file add separator
	.mainmenu.file add command -label "New DVD" -command {::utils::new_DVD}
	.mainmenu.file add command -label "Import VTST Titles" -command {::utils::import_VTST}
	.mainmenu.file add command -label "Replace VTST Titles" -command {::utils::replace_VTST}
	.mainmenu.file add separator

	.mainmenu.file add cascade -label "Backups" -menu .mainmenu.file.ba
	menu .mainmenu.file.ba
	.mainmenu.file.ba add command -label "Create backup"  -command {::main::Create_Backup false false} -accelerator "Ctrl+B"
	bind . <Control-b> {::main::Create_Backup false false}
	.mainmenu.file.ba add command -label "Restore backup" -command ::main::Restore_Backup -accelerator "Ctrl+Alt+B"
	bind . <Control-Alt-b> {::main::Restore_Backup}
	.mainmenu.file.ba add separator
	.mainmenu.file.ba add command -label "New incremental backup"  -command ::main::Incremental_Backup -accelerator "Ctrl+N"
	bind . <Control-n> {::main::Incremental_Backup}
	.mainmenu.file.ba add command -label "Undo last Incremental backup"  -command ::main::Undo_Incremental_Backup -accelerator "Ctrl+Alt+N"
	bind . <Control-Alt-n> {::main::Undo_Incremental_Backup}
	.mainmenu.file.ba add separator
	.mainmenu.file.ba add command -label "Delete backup(s)"  -command ::main::Delete_Backup

	.mainmenu.file add separator
	.mainmenu.file add cascade -label "PGC Commands" -menu .mainmenu.file.pc
	menu .mainmenu.file.pc
	.mainmenu.file.pc add command -label "Import PGC commands" -command ::main::import_pgc_cmds -accelerator "Ctrl+I"
	bind . <Control-i> ::main::import_pgc_cmds
	.mainmenu.file.pc add command -label "Export PGC commands" -command ::main::export_pgc_cmds -accelerator "Ctrl+E"
	bind . <Control-e> ::main::export_pgc_cmds
	.mainmenu.file.pc add separator
	.mainmenu.file.pc add command -label "Import all PGC commands" -command ::main::import_all_pgc_cmds
	.mainmenu.file.pc add command -label "Export all PGC commands" -command ::main::export_all_pgc_cmds

	.mainmenu.file add cascade -label "Menu buttons & BOVs" -menu .mainmenu.file.mb
	menu .mainmenu.file.mb
	.mainmenu.file.mb add command -label "Import all Menu Buttons and BOVs" -command ::menubuttons::load_buts
	.mainmenu.file.mb add command -label "Export all Menu Buttons and BOVs" -command ::menubuttons::save_buts

	.mainmenu.file add separator
	.mainmenu.file add cascade -label "Comments and GPRM labels" -menu .mainmenu.file.cl
	menu .mainmenu.file.cl
	.mainmenu.file.cl add command -label "Save comments and labels now" -command {::utils::export_pgc_comments true}
	.mainmenu.file.cl add command -label "Import comments and labels" -command {::utils::import_pgc_comments} -accelerator "Control+Alt+C"
	bind . <Control-Alt-c> ::utils::import_pgc_comments
	.mainmenu.file.cl add command -label "Export comments and labels" -command {::utils::export_pgc_comments}
	.mainmenu.file.cl add separator
	.mainmenu.file.cl add command -label "GPRM Labels editor" -command {::utils::set_gprm_labels}
	.mainmenu.file.cl add separator
	.mainmenu.file.cl add command -label "Clear all comments and labels" -command {::utils::clear_all_pgc_comments}
	.mainmenu.file add cascade -label "Scratchbook" -menu .mainmenu.file.sb
	menu .mainmenu.file.sb
	.mainmenu.file.sb add command -label "Open scratchbook" -command ::main::open_scratchbook
	.mainmenu.file.sb add command -label "Save scratchbook" -command ::main::save_scratchbook
	.mainmenu.file add separator
	.mainmenu.file add command -label "Check DVD+/-R Size" -command {::writedvd::DVD_size}
	if {$::tcl_platform(platform) == "windows"} {
		.mainmenu.file add command -label "Burn DVD / Create ISO" -command {::burn::burn}
		.mainmenu.file add command -label "Explore DVD Folder" -command {::macros::explore_dvd} -accelerator "Ctrl+F"
		bind . <Control-f> ::macros::explore_dvd
	} elseif {$::tcl_platform(platform) != "macintosh"} {
		.mainmenu.file add command -label "Create ISO" -command {::burn::burn}
	}
	if {$::config(debug) > 0 && $::tcl_platform(platform) == "windows"} {
		.mainmenu.file add separator
		.mainmenu.file add command -label "Console" -command {console show}
	}
	.mainmenu.file add separator
	.mainmenu.file add command -label "Quit" -command {cleanexit 0}
	.mainmenu.file add command -label "Kill (emergency exit)" -command exit

	.mainmenu add cascade -label "Edit" -menu .actionmenu
	.actionmenu add separator
	set itemnum [expr {[.actionmenu index end]+1}]
	.actionmenu add command -label "Jump to target" -accelerator "Ctrl+j" \
			-command "set ::main::last_menu_selected $itemnum; ::main::action_Jump_to_target"
	incr itemnum
	.actionmenu add command -label "Undo cmds changes" -accelerator "Ctrl+z" \
			-command "set ::main::last_menu_selected $itemnum; ::main::action_Undo_pgc"
	incr itemnum
	.actionmenu add command -label "Copy as text" \
			-command "set ::main::last_menu_selected $itemnum; ::main::copy_as_text"
	incr itemnum
	.actionmenu add command -label "Append to scratchbook" -accelerator "Ctrl+Insert" \
			-command "set ::main::last_menu_selected $itemnum; ::main::send_to_clipboard"
	incr itemnum
	.actionmenu add separator
	incr itemnum
	.actionmenu add command -label "Delete all commands" \
			-command "set ::main::last_menu_selected $itemnum; ::main::action_delete_all_commands"


	.mainmenu add cascade -label "Trace" -menu .mainmenu.trace
	menu .mainmenu.trace
	.mainmenu.trace add checkbutton -variable ::tr::tracemode -label "Trace mode" -command ::tr::dvd_trace -accelerator "Ctrl+T"
	bind . <Control-t> ::tr::dvd_trace
	.mainmenu.trace add separator
	.mainmenu.trace add command -label "Toggle PGC pre/post breakpoint" -command {::tr::toggle_breakpoint prepost} -accelerator "Ctrl+Shift+T"
	.mainmenu.trace add command -label "Toggle PGC playback breakpoint" -command {::tr::toggle_breakpoint playback} -accelerator "Ctrl+Alt+T"
	bind . <Control-T> {::tr::toggle_breakpoint prepost}
	bind . <Control-Alt-t> {::tr::toggle_breakpoint playback}
	.mainmenu.trace add command -label "Clear all PGC breakpoints" -command ::tr::clear_all_breakpoints
	.mainmenu.trace add separator
	.mainmenu.trace add command -label "Virtual Player Setup" -command ::tr::player_setup
	.mainmenu.trace add separator
	.mainmenu.trace add checkbutton -label "Hide toolbars in Trace mode ?" \
			-variable ::tr::hide_gui  -onvalue true -offvalue false


	.mainmenu add cascade -label "Preview" -menu .mainmenu.preview
	menu .mainmenu.preview
	if {$::config(use_preview)} {
		.mainmenu.preview add command -label "Preview PGC" -command ::preview::preview_current_pgc_2 -accelerator "` (~), Control+P"
		bind . <quoteleft> ::preview::preview_current_pgc_2
		bind . <Control-p> ::preview::preview_current_pgc_2
		.mainmenu.preview add command -label "Preview full domain VOB(s)" -command ::preview::preview_current_domain
		.mainmenu.preview add separator
		.mainmenu.preview add checkbutton -label "Automatically start Preview playback?" \
				-variable ::config(preview_autoplay)
		.mainmenu.preview add checkbutton -label "Real time playback?" \
				-variable ::config(preview_realtime)
		.mainmenu.preview add separator
	}
	.mainmenu.preview add radiobutton -label "Full size" \
			-variable ::config(preview_screen_divisor) -value 1.0
	.mainmenu.preview add radiobutton -label "Medium size" \
			-variable ::config(preview_screen_divisor) -value 1.333333
	.mainmenu.preview add radiobutton -label "Small size" \
			-variable ::config(preview_screen_divisor) -value 2.0
	if {$::tcl_platform(platform) == "windows" && $::config(use_preview)} {
		.mainmenu.preview add separator
		.mainmenu.preview add checkbutton -label "Disable DirectDraw Overlays?" \
				-variable ::config(preview_no_overlay)
	}
	if {$::config(preview_available)} {
		.mainmenu.preview add separator
		.mainmenu.preview add checkbutton -label "Use Preview?  (require restart)" \
				-variable ::config(use_preview) -onvalue true -offvalue false
	} else {
		.mainmenu.preview add separator
		.mainmenu.preview add command -label "Install Preview" -command {
			set txt "To be able to use the Preview, you must install"
			set txt "$txt\n\"wine\" (for Linux) or \"darwine\" (for Mac OSX)."
			set txt "$txt\nThen, return to this menu, select the new \"Use"
			set txt "$txt\nPreview\" option, then quit and restart PgcEdit."
			set txt "$txt\n(Note: Still not tested under Mac OSX!)"
			set txt "$txt\n\nwine for Linux: http://www.winehq.com"
			set txt "$txt\ndarwine for Mac: http://darwine.opendarwin.org"
			set txt "$txt\nBoth winehq and darwine are free.  Enjoy!"
			tk_messageBox -title "PgcEdit Preview" -icon info -message $txt -parent .
		}
	}

	set menuname .mainmenu.dvd
	.mainmenu add cascade -label "DVD" -menu $menuname
	menu $menuname
	$menuname add command -label "Region Code" -command {::utils::set_region_code}
	$menuname add command -label "Filter Prohibited User Operations" -command {::utils::filter_puos}
	$menuname add command -label "Enable Navigation Buttons" -command {::macros::nav_buttons}
	$menuname add command -label "Remove Parental Management" -command {::macros::remove_parental_managment}
	$menuname add separator
	$menuname add command -label "Blank out all Title PGCs in DVD" -command {::utils::blank_out_all_DVD_titles}
	$menuname add command -label "Remap Titlesets" -command {::utils::remap_titlesets}
	$menuname add separator
	$menuname add command -label "Delete Uncalled TitleSets  (Fast!)" -command {::utils::delete_uncalled_vts}
	$menuname add command -label "Delete Uncalled PGCs" -command {::utils::delete_all_uncalled}
	$menuname add command -label "Delete last VTS in DVD" -command {::utils::delete_last_vts}
	$menuname add separator
	$menuname add command -label "Fix number of streams in VMGM/VTSI_MAT tables" -command {::streams::verify_num_streams}
	$menuname add command -label "Rebuild all time maps of DVD" -command ::tmap::rebuild_all_tmaps
	$menuname add separator
	$menuname add command -label "Set Provider ID" -command ::utils::set_provider_ID
	$menuname add command -label "Set DVD-TEXT General Name" -command ::utils::set_General_Name_GUI
	$menuname add command -label "Remove DVD-TEXT data" -command ::utils::remove_dvd_text_data


	set menuname .mainmenu.domain
	.mainmenu add cascade -label "Domain" -menu $menuname
	menu $menuname
	$menuname add command -label "Domain Streams Attributes" -command {::streams::set_streams_attribs .} -accelerator "Control+A"
	bind . <Control-a> {::streams::set_streams_attribs .}
	$menuname add separator
	$menuname add command -label "Blank out all PGCs in domain" -command {::utils::blank_out_PGCS_GUI} -accelerator "Control+Delete"
	bind . <Control-Delete> {::utils::blank_out_PGCS_GUI}
	$menuname add command -label "Delete All Commands in domain" -command "::utils::delete_all_domain_commands"
	$menuname add command -label "Remap PGCs of domain" -command {::utils::remap_pgcs_gui}
	$menuname add command -label "Delete last PGC in domain" -command {::utils::delete_last_pgc}

	set menuname .mainmenu.pgc
	.mainmenu add cascade -label "PGC" -menu $menuname
	menu $menuname
	$menuname add command -label "Kill PGC Playback" -command {::macros::kill_pgc_playback} -accelerator "Ctrl+K"
	bind . <Control-k> ::macros::kill_pgc_playback
	$menuname add command -label "Enable all operations" -command {::macros::enable_all_operations}
	$menuname add separator
	$menuname add command -label "Reset Used GPRMs to 0" -command {::macros::reset_used_gprms}
	$menuname add separator
	$menuname add command -label "New dummy PGC" -command {::utils::new_pgc false false}
	$menuname add command -label "Clone current PGC" -command {::utils::clone_pgc}
	$menuname add separator
	$menuname add command -label "Jump to PGC upon DVD insert" -command {::macros::jump2pgc}
	$menuname add separator
	$menuname add command -label "PGC comment" -command {::utils::pgc_comment} -accelerator "Control+Shift+C"
	bind . <Control-C> ::utils::pgc_comment

	set menuname .mainmenu.menu
	.mainmenu add cascade -label "Menu" -menu $menuname
	menu $menuname
	$menuname add command -label "Show/Edit Menu Buttons or BOVs" \
			-command {::menubuttons::display_current_menu_buttons info} -accelerator "Ctrl+M"
	bind . <Control-m> {::menubuttons::display_current_menu_buttons info}
	$menuname add command -label "Select a non-hidden button/BOV by default" \
			-command ::menued::fix_sethl_btn_pre_cmds

	$menuname add separator
	$menuname add command -label "New Menu" -command {::utils::new_menu false}
	$menuname add command -label "Import Menu" -command {::utils::import_menu}
	$menuname add command -label "Remove Menu" -command {::utils::remove_menu}
	$menuname add separator
	$menuname add command -label "Set Menu Type" -command {::utils::set_menu_type_GUI}
#	$menuname add command -label "Swap or Copy Menu PGC" -command {::utils::swap_or_copy_pgc_from}
	$menuname add separator
	$menuname add command -label "Swap menu LUs" -command {::utils::swap_lus}
	$menuname add command -label "Remove LU" -command {::utils::delete_lu}
	$menuname add command -label "Copy LU cmds to other LUs" -command {::utils::copy_lu_to_lus}

	set menuname .mainmenu.title
	.mainmenu add cascade -label "Title" -menu $menuname
	menu $menuname
	$menuname add command -label "Title Play Map table" -command ::srptinfo::srptinfo
	$menuname add command -label "Remap Title numbers"  -command ::srptinfo::remap_titles_numbers_gui
	$menuname add command -label "Re-order TTNs in VTST domain"  -command ::utils::reorder_ttns_gui
	$menuname add command -label "PTT (Chapter) Editor"  -command ::ptted::ptted
	$menuname add separator
	$menuname add command -label "Rebuild time map of current PGC" -command ::tmap::rebuild_tmap_cur_pgc
	$menuname add command -label "Rebuild time maps of current VTS" -command ::tmap::rebuild_tmap_cur_vts
	$menuname add separator
	$menuname add command -label "New 'Play All' Title" -command {::utils::create_playall_pgc}
	$menuname add separator
	$menuname add command -label "Import First-Play (startup) Clip" -command {::macros::import_fp_clip}
	$menuname add command -label "Import Title's Intro Clip" -command {::macros::import_title_intro_clip}
	$menuname add command -label "Import Title's Closing Clip" -command {::macros::import_title_closing_clip}

	set menuname .mainmenu.info
	.mainmenu add cascade -label "Info" -menu $menuname
	menu $menuname
	$menuname add command -label "PGC" -command {::macros::show_current_pgc_as_text}
	$menuname add separator
	$menuname add command -label "Title Play Map table" -command ::srptinfo::srptinfo_dump
	$menuname add separator
	$menuname add command -label "DVD commands" -command ::main::dump_DVD
	$menuname add command -label "PGCs params and Cell Tables" -command ::pgced::dump_all_pgcs
	$menuname add command -label "Display PGCs list as text" -command {::main::showlabels_as_text}
	$menuname add command -label "Menu Buttons/BOVs commands" -command {::menubuttons::show_menu_buttons_info}
	$menuname add separator
	$menuname add command -label "Go to calling command" -command {::utils::pgc_crossrefs} -accelerator "Ctrl+G"
	bind . <Control-g> ::utils::pgc_crossrefs
	$menuname add command -label "Calls cross references" -command {::utils::show_crossrefs} -accelerator "Ctrl+H"
	bind . <Control-h> ::utils::show_crossrefs
	$menuname add command -label "Jumps from VMG to current VTS" -command {::utils::vts_refs} -accelerator "Ctrl+Q"
	bind . <Control-q> ::utils::vts_refs
	$menuname add command -label "Jumps from VMG to all VTSs" -command {::utils::vts_refs true}
	$menuname add command -label "Jumps from current VTS to VMG" -command {::utils::vmg_refs} -accelerator "Ctrl+Shift-Q"
	bind . <Control-Q> ::utils::vmg_refs
	$menuname add command -label "Jumps from all VTSs to VMG" -command {::utils::vmg_refs true}
	$menuname add command -label "Find jumps to nowhere" -command {::utils::pgc_crossrefs illegal}
	$menuname add command -label "Find jumps to void PGCs" -command {::utils::pgc_crossrefs void}
	$menuname add command -label "Find Uncalled PGCs" -command {::utils::find_all_uncalled}
	$menuname add separator
	$menuname add command -label "Streams" -command ::streams::get_streams_info
	$menuname add separator
	#$menuname add command -label "Find Unused GPRMs" -command {::macros::find_unused_gprms false}
	$menuname add command -label "GPRMs" -command {::utils::set_gprm_labels -1 . true}

	.mainmenu add cascade -label "Tools" -menu .mainmenu.tools
	menu .mainmenu.tools
	::toolscfg::configure_tools_menu

	.mainmenu add cascade -label "View" -menu .mainmenu.view
	menu .mainmenu.view
	.mainmenu.view add checkbutton -label "Toolbar" \
			-variable ::toolbar::toolbar_opened -onvalue true -offvalue false \
			-command {::toolbar::init_toolbar $::toolbar::toolbar_opened}
	.mainmenu.view add checkbutton -label "PGC Selector" \
			-variable ::main::selector_opened \
			-command {::main::selector toggle ::main::labelslist ::main::cmdslist}
	.mainmenu.view add cascade -label "PGC Selector side" -menu .mainmenu.view.selectorside
	menu .mainmenu.view.selectorside
	.mainmenu.view.selectorside add radiobutton -label "Left" \
			-variable ::config(selector_side) -value "left" \
			-command {pack  configure .mf.f2.selector -side left}
	.mainmenu.view.selectorside add radiobutton -label "Right" \
			-variable ::config(selector_side) -value "right" \
			-command {pack  configure .mf.f2.selector -side right}
	.mainmenu.view.selectorside add separator
	.mainmenu.view.selectorside add radiobutton -label "Top" \
			-variable ::config(selector_side) -value "top" \
			-command {pack  configure .mf.f2.selector -side top}
	.mainmenu.view.selectorside add radiobutton -label "Bottom" \
			-variable ::config(selector_side) -value "bottom" \
			-command {pack  configure .mf.f2.selector -side bottom}
	.mainmenu.view.selectorside add command -label "Increase height" \
			-command {
				if {$::config(selector_height)<50 && [winfo exists .mf.f2.selector.a.lb]} {
					incr ::config(selector_height)
					.mf.f2.selector.a.lb configure -height $::config(selector_height)
				} else bell
			}
	.mainmenu.view.selectorside add command -label "Decrease height" \
			-command {
				if {$::config(selector_height)>3 && [winfo exists .mf.f2.selector.a.lb]} {
					incr ::config(selector_height) -1
					.mf.f2.selector.a.lb configure -height $::config(selector_height)
				} else bell
			}
	.mainmenu.view.selectorside add command -label "Default height (8 lines)" \
			-command {if {[winfo exists .mf.f2.selector.a.lb]} {set ::config(selector_height) 8; .mf.f2.selector.a.lb configure -height 8} else bell}
	.mainmenu.view add checkbutton -label "Edit Buttons" \
			-variable ::main::buttons_opened \
			-command {::main::action_buttons toggle}
	.mainmenu.view add separator
	.mainmenu.view add command -label "Tcl Calculator/Converter" \
			-command {::calc::calculator .calc}
	.mainmenu.view add separator
	.mainmenu.view add command -label "Show Log" -command {openlog}
	.mainmenu.view add command -label "Clear Log" -command {set ::log ""}

	.mainmenu add cascade -label "Options" -menu .mainmenu.options
	menu .mainmenu.options

	if {$::tcl_platform(platform) == "windows"} {
		set menuname .mainmenu.options.install
		.mainmenu.options add cascade -label "Install" -menu $menuname
		.mainmenu.options add separator
		menu $menuname
		if {[info exists ::freewrap::progname]} {
			$menuname add checkbutton \
					-label "Shortcut icon to PgcEdit in Start Menu?" \
					-variable ::config(shortcut_startmenu) \
					-command {::winshellext::create_shortcut_to_PgcEdit startmenu $::config(shortcut_startmenu)}
			$menuname add checkbutton \
					-label "Shortcut icon to PgcEdit on Desktop?" \
					-variable ::config(shortcut_desktop) \
					-command {::winshellext::create_shortcut_to_PgcEdit desktop $::config(shortcut_desktop)}
			$menuname add checkbutton \
					-label "Shortcut icon to PgcEdit in Send To context menu?" \
					-variable ::config(shortcut_sendto) \
					-command {::winshellext::create_shortcut_to_PgcEdit sendto $::config(shortcut_sendto)}
			$menuname add command \
					-label "Create another shortcut icon to PgcEdit" \
					-command {::winshellext::create_shortcut_to_PgcEdit anydir true}
			$menuname add separator
			$menuname add checkbutton \
					-label "Associate PgcEdit with IFO files?" \
					-variable ::config(assoicated_with_IFO) \
					-command {::winshellext::associate_PgcEdit "IFO"}
			$menuname add checkbutton \
					-label "Associate PgcEdit with BUP files?" \
					-variable ::config(assoicated_with_BUP) \
					-command {::winshellext::associate_PgcEdit "BUP"}
			$menuname add separator
		}
		$menuname add command -label "Install a plugin" -command {::plugins::install_plugin}
		if {[info exists ::freewrap::progname]} {
			$menuname add separator
			$menuname add command -label "Uninstall PgcEdit" -command {::winshellext::uninstallPgcEdit}
		}
	}

	set menuname .mainmenu.options.startup
	.mainmenu.options add cascade -label "Startup" -menu $menuname
	menu $menuname
	$menuname add checkbutton -label "Check for new version at startup?" \
			-variable ::config(check_version_at_startup) -onvalue true -offvalue false
	$menuname add checkbutton -label "Launch Open DVD dialog at startup?" \
			-variable ::config(auto_open_new_dvd) -onvalue true -offvalue false
	$menuname add checkbutton -label "Start in Trace mode (when a DVD is opened at startup)?" \
			-variable ::config(open_trace_mode) -onvalue true -offvalue false
	$menuname add checkbutton -label "Show toolbar at startup?" \
			-variable ::config(toolbar) -onvalue true -offvalue false
	$menuname add checkbutton -label "Show PGC Selector at startup?" \
			-variable ::config(selector) -onvalue true -offvalue false
	$menuname add checkbutton -label "Show Edit Buttons at startup?" \
			-variable ::config(buttons) -onvalue true -offvalue false
	$menuname add separator
	$menuname add command -label "Save current scratchbook as default" \
			-command {saveFile $::config(default_scratchbook_filename) binary $::scratchbook_buffer}


	set menuname .mainmenu.options.io
	.mainmenu.options add cascade -label "Input/Output" -menu $menuname
	menu $menuname
	$menuname add checkbutton -label "Create backup when a DVD is opened for the first time?" \
			-variable ::config(auto_create_backup) -onvalue true -offvalue false
	$menuname add checkbutton -label "Store PgcEdit_Backup folder in parent of VIDEO_TS folder?" \
			-variable ::config(backup_in_root) -onvalue true -offvalue false -command ::main::move_backup
	$menuname add checkbutton -label "Automatically load and save user's comments and labels in backup folder?" \
			-variable ::config(auto_backup_comments) -onvalue true -offvalue false -command \
			{
				if {$::config(auto_backup_comments) && ! $::config(auto_create_backup)} {
					set txt "This option works only if the backup folder exists!"
					set txt "$txt\nFor best results, you should enable the Create Backup option."
					tk_messageBox -title "PgcEdit" -icon warning -message $txt -parent .
				}
			}
	$menuname add checkbutton -label "Automatically launch BOV finder when a DVD is opened?" \
			-variable ::config(auto_launch_menubuttons_parser) -onvalue true -offvalue false
	$menuname add checkbutton -label "Automatically launch \"Fix number of streams in VMGM/VTSI_MAT\"?" \
			-variable ::config(auto_fix_streams) -onvalue true -offvalue false

	$menuname add separator
	$menuname add checkbutton -label "When saving, leave at least 32K of space between IFO and BUP?" \
			-variable ::config(save_BUP_with_gaps) -onvalue true -offvalue false -command {
				if {[info exists ::dvddir]} {::main::dvd_modified true false}
				if {$::config(save_BUP_with_gaps)} {
					set txt "This option will modify the VTS pointers so that there"
					set txt "${txt}\nare at least 32Kbytes (16 sectors) of space between"
					set txt "${txt}\nthe end of the IFO and the start of the VOB."
					set txt "${txt}\nThis is recommended to help protect the DVD against"
					set txt "${txt}\ndefects and scratches.\n"
					set txt "${txt}\nHowever, some burning programs are not able to add"
					set txt "${txt}\nthe gaps between the files, and will burn a DVD"
					set txt "${txt}\nunplayable on a standalone player!"
					set txt "${txt}\nOther burning softwares (notably Nero 6) will remove"
					set txt "${txt}\nthe gaps, and fix the pointers accordingly.\n"
					set txt "${txt}\nThe Burn DVD / Create ISO function integrated in"
					set txt "${txt}\nPgcEdit will, of course, take care of the gaps."
					set txt "${txt}\nImgTool Classic (which, like PgcEdit uses mkisofs) and"
					set txt "${txt}\nStomp's RecordNow MAX are also good choices.\n"
					set txt "${txt}\nCAUTION: If you use another burner software, do a"
					set txt "${txt}\ntest on a RW media and check it on your standalone!"
					tk_messageBox -title "PgcEdit" -icon warning -message $txt -parent . -skip "32kgaps_warning"
				}
			}
	$menuname add separator
	$menuname add command -label "Burn / Create ISO setup" -command {::burn::setup .}
	$menuname add command -label "Select web browser..." -command {
		if {$::config(web_browser) == ""} {
			set initialdir  [file normalize "/"]
			set initialfile ""
		} else {
			set initialdir  [file normalize [file dirname $::config(web_browser)]]
			set initialfile [file tail $::config(web_browser)]
		}
		if {$::tcl_platform(platform) == "windows"} {
			set tmp [tk_getOpenFile -filetypes {{{Program files} {*.EXE}}} -initialdir $initialdir -initialfile $initialfile -parent . -title {PgcEdit: Please select your web browser}]
		} else {
			set tmp [tk_getOpenFile -initialdir $initialdir -initialfile $initialfile -parent . -title {PgcEdit: Please select your web browser}]
		}
		if {$tmp == ""} return
		set ::config(web_browser) $tmp
	}


	set menuname .mainmenu.options.gui
	.mainmenu.options add cascade -label "User interface" -menu $menuname
	menu $menuname
	$menuname add checkbutton -label "Use file browser instead of folder browser for Open/Save folder operations?" \
			-variable ::config(use_file_browser) -onvalue true -offvalue false
	$menuname add checkbutton -label "Auto-adjust Command Editor window to contents?" \
			-variable ::config(adjust_win_size) -onvalue true -offvalue false
	$menuname add checkbutton -label "Allow Menu Tearoffs?  (require restart)" \
			-variable ::config(tearoff_menus) -onvalue true -offvalue false
	$menuname add checkbutton -label "Popup menu when right mouse button released?  (require restart)" \
			-variable ::config(popups_button) -onvalue "<ButtonRelease-3>" -offvalue "<3>"
	$menuname add separator
	$menuname add command -label "Global font  (require restart)" \
			-command {set f [::fontreq::fontreq .fr $::config(global_font) false]; \
					if {$f!={}} {set ::config(global_font) $f}}
	$menuname add command -label "Fixed-width font" \
			-command {set f [::fontreq::fontreq .fr $::config(fixed_width_font) true]; \
					if {$f!={}} {set ::config(fixed_width_font) $f; \
					$::widget(main_listbox) configure -font $::config(fixed_width_font); \
					catch {$::tr::wm_main_window.fr.a.txt configure -font $::config(fixed_width_font)}
					}}
	$menuname add checkbutton -label "Use custom colors for GUI ? (require restart)" \
			-variable ::config(use_custom_color) -onvalue true -offvalue false \
			-command {
				if {$::config(use_custom_color)} {
					set tmp [tk_chooseColor -title "Select default background color" -parent . -initialcolor $::config(custom_color)]
					if {$tmp == ""} {
						set ::config(use_custom_color) false
					} else {
						set ::config(custom_color) $tmp
						tk_setPalette $::config(custom_color)
					}
				} else {
					tk_setPalette $::config(default_color)
				}
			}
	$menuname add checkbutton -label "White listbox background ? (require restart)" \
			-variable ::config(white_listbox_background) -onvalue true -offvalue false
	$menuname add separator
	$menuname add checkbutton -label "Display hidden dialog boxes during current session ?" \
			-variable ::config(show_hidden_dialogs) -onvalue true -offvalue false
	$menuname add command -label "Restore all hidden dialog boxes ?" -command {set ::config(skip_dialogs) {}}
	$menuname add checkbutton -label "Open yes/no/cancel dialog boxes under the mouse pointer ?" \
			-variable ::config(standard_dialogs_under_mouse) -onvalue true -offvalue false
	$menuname add checkbutton -label "Play sounds with yes/no/cancel dialog boxes ?  (for dual core CPU bug)" \
			-variable ::config(play_dialog_sounds) -onvalue true -offvalue false

	set menuname .mainmenu.options.cmd
	.mainmenu.options add cascade -label "Command Editor" -menu $menuname
	menu $menuname
	$menuname add checkbutton -label "Hi-lite Errors and GOTO Targets? (Doesn't detect all errors!)" \
			-variable ::config(highlight_gotos) -onvalue true -offvalue false \
			-command {::main::init_refresh_listbox ; ::main::refresh_listbox}
	$menuname add checkbutton -label "Auto renumber GOTOs when commands added/removed?" \
			-variable ::config(renumber_gotos) -onvalue true -offvalue false \
			-command ::main::renumber_gotos_warning
	$menuname add checkbutton -label "Include \[hex codes\] in main listbox ?" \
			-variable ::config(show_hex_codes) -onvalue true -offvalue false -command {
				if {$::config(show_hex_codes)} {set v ::editor_cmds_list} else {set v ::editor_cmds_list_no_hex}
				$::widget(main_listbox) configure -listvariable $v
			}
	$menuname add separator
	$menuname add checkbutton -label "Include \[hex codes\] in Info / commands dumps ?" \
			-variable ::config(dump_hex_codes) -onvalue true -offvalue false
	$menuname add separator
	$menuname add checkbutton -label "Allow PgcEdit to add automatic comments in the PGC labels ?" \
			-variable ::config(allow_auto_comments) -onvalue true -offvalue false
	$menuname add separator
	$menuname add command -label "Set current search string as default" \
			-command {set ::config(searchstring) $::main::searchstring}

	set menuname .mainmenu.options.func
	.mainmenu.options add cascade -label "Functions" -menu $menuname
	menu $menuname
	set menuname .mainmenu.options.func.killpb
	.mainmenu.options.func add cascade -label "Kill PGC Playback" -menu $menuname
	menu $menuname
	$menuname add checkbutton -label "Always remove or blank the PGC cells ?" \
			-variable ::config(kill_pb_remove_cells) -onvalue true -offvalue false
	set menuname .mainmenu.options.func.dvdtext
	.mainmenu.options.func add cascade -label "DVD-TEXT General Name" -menu $menuname
	menu $menuname
	$menuname add command -label "Maximum length of the General Name field" \
			-command {::utils::InputGeneralNameMaxLen}


	set menuname .mainmenu.options
	$menuname add separator
	$menuname add checkbutton -label "Save settings on exit?" \
			-variable ::config(save_setups_on_exit) -onvalue true -offvalue false
	$menuname add command -label "Save settings now" -command ::save_user_prefs
	$menuname add command -label "Restore defaults  (require restart)" \
			-command {catch {file delete [file join $::config(config_directory) "pgcedit.cfg"]}; \
					set ::config(restore_defaults) true}



	.mainmenu add cascade -label "Help" -menu .mainmenu.help
	menu .mainmenu.help
	.mainmenu.help add command -label "Documentation" -command ::main::display_doc
	.mainmenu.help add command -label "Version history" -command {::main::showtextfile \
			.hist "PgcEdit Versions History" "$::config(apppath)/HISTORY.txt"}
	.mainmenu.help add command -label "GPL License" -command {::main::showtextfile .license \
			"GNU General Public License" "$::config(apppath)/COPYING.txt"}
	.mainmenu.help add separator
	.mainmenu.help add command -label "ISO639 language codes" -command {::main::showtextfile .langcodes \
			"ISO639 language codes" "$::config(apppath)/lib/data/lang_codes.txt"}
	.mainmenu.help add command -label "Regular Expressions syntax" -command \
			{::main::display_html "http://www.tcl.tk/man/tcl8.4/TclCmd/re_syntax.htm#M5"}
	.mainmenu.help add command -label "Regular Expressions table" -command \
			{::main::display_html "http://www.wellho.net/regex/tcl.html"}
	.mainmenu.help add separator
	.mainmenu.help add command -label "Homepage" \
			-command {::main::display_html "http://www.videohelp.com/~r0lZ/pgcedit/index.html"}
	.mainmenu.help add separator
	.mainmenu.help add command -label "Check for new version" -command ::main::check_new_version
	.mainmenu.help add command -label "About" -command ::main::about

	. configure -menu .mainmenu
}

proc build_popup_menus {} {
	set menuname .selector_popup_menu_fp
	if {[winfo exists $menuname]} {destroy $menuname}
	menu $menuname
	if {$::config(tearoff_menus)} {set itemnum 1} else {set itemnum 0}
	set ::main::last_fp_menu_selected $itemnum
	$menuname add command -label "Edit PGC" -accelerator "Return" \
			-command "set ::main::last_fp_menu_selected $itemnum; ::main::action_Edit_PGC"
	incr itemnum
	$menuname add command -label "Info: PGC" \
			-command "set ::main::last_fp_menu_selected $itemnum; ::macros::show_current_pgc_as_text"
	incr itemnum
	$menuname add command -label "Go to calling command"  -accelerator "Control+G" \
			-command "set ::main::last_fp_menu_selected $itemnum; ::utils::pgc_crossrefs"
	incr itemnum
	$menuname add separator
	incr itemnum
	$menuname add command -label "New Menu" \
			-command "set ::main::last_fp_menu_selected $itemnum; ::utils::new_menu false"
	incr itemnum
	$menuname add command -label "Import Menu" \
			-command "set ::main::last_fp_menu_selected $itemnum; ::utils::import_menu"
	incr itemnum
	$menuname add separator
	incr itemnum
	$menuname add command -label "Import First-Play Clip" \
			-command "set ::main::last_fp_menu_selected $itemnum; ::macros::import_fp_clip"
	incr itemnum
	$menuname add separator
	incr itemnum
	$menuname add command -label "Toggle PGC pre/post breakpoint" -accelerator "Control+Shift+T" \
			-command "set ::main::last_fp_menu_selected $itemnum; ::tr::toggle_breakpoint prepost"
	incr itemnum
	$menuname add command -label "Toggle PGC playback breakpoint" -accelerator "Control+Alt+T" \
			-command "set ::main::last_fp_menu_selected $itemnum; ::tr::toggle_breakpoint playback"
	incr itemnum
	$menuname add separator
	incr itemnum
	$menuname add command -label "Delete all commands in PGC" \
			-command "set ::main::last_fp_menu_selected $itemnum; ::main::action_delete_all_commands"
	incr itemnum
	$menuname add separator
	incr itemnum
	$menuname add command -label "PGC Comment" \
			-command "set ::main::last_fp_menu_selected $itemnum; ::utils::pgc_comment" -accelerator "Control+Shift+C"


	set menuname .selector_popup_menu_menus
	if {[winfo exists $menuname]} {destroy $menuname}
	menu $menuname
	if {$::config(tearoff_menus)} {set itemnum 1} else {set itemnum 0}
	set ::main::last_menus_menu_selected $itemnum

	if {$::config(use_preview)} {
		$menuname add command -label "Preview PGC" -accelerator "` (~), Control+P" -command \
				"set ::main::last_menus_menu_selected $itemnum; ::preview::preview_current_pgc_2"
		incr itemnum
		$menuname add separator
		incr itemnum
	}
	$menuname add command -label "Edit PGC" -accelerator "Return" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::main::action_Edit_PGC"
	incr itemnum
	$menuname add command -label "Info: PGC" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::macros::show_current_pgc_as_text"
	incr itemnum
	$menuname add command -label "Go to calling command"  -accelerator "Control+G" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::utils::pgc_crossrefs"
	incr itemnum
	$menuname add separator
	incr itemnum
	$menuname add command -label "Domain Streams Attributes" -accelerator "Control+A" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::streams::set_streams_attribs ."
	incr itemnum
	$menuname add separator
	incr itemnum
	$menuname add command -label "Blank out all Menu PGCs" -accelerator "Control+Delete" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::utils::blank_out_PGCS_GUI"
	incr itemnum
	$menuname add command -label "New Menu" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::utils::new_menu false"
	incr itemnum
	$menuname add command -label "Import Menu" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::utils::import_menu"
	incr itemnum
	$menuname add command -label "Remove Menu" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::utils::remove_menu"
	incr itemnum
	$menuname add command -label "Remove LU" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::utils::delete_lu"
	incr itemnum
	$menuname add command -label "Set Menu Type" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::utils::set_menu_type_GUI"
	incr itemnum
	$menuname add command -label "Menu Buttons" -accelerator "Control+M" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::menubuttons::display_current_menu_buttons info"
	incr itemnum
	$menuname add command -label "Swap menu LUs" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::utils::swap_lus"
	incr itemnum
	$menuname add command -label "Copy LU cmds to other LUs" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::utils::copy_lu_to_lus"
	incr itemnum
	$menuname add command -label "New dummy PGC" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::utils::new_pgc false false"
	incr itemnum
	$menuname add command -label "Clone PGC" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::utils::clone_pgc"
	incr itemnum
	$menuname add separator
	incr itemnum
	$menuname add command -label "Kill PGC Playback" -accelerator "Control+K" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::macros::kill_pgc_playback"
	incr itemnum
	$menuname add command -label "Enable all operations" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::macros::enable_all_operations"
	incr itemnum
	$menuname add separator
	incr itemnum
	$menuname add command -label "Toggle PGC pre/post breakpoint" -accelerator "Control+Shift+T" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::tr::toggle_breakpoint prepost"
	incr itemnum
	$menuname add command -label "Toggle PGC playback breakpoint" -accelerator "Control+Alt+T" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::tr::toggle_breakpoint playback"
	incr itemnum
	$menuname add separator
	incr itemnum
	$menuname add command -label "Delete all commands in PGC" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::main::action_delete_all_commands"
	incr itemnum
	$menuname add command -label "Delete all commands in Domain!" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::utils::delete_all_domain_commands"
	incr itemnum
	$menuname add separator
	incr itemnum
	$menuname add command -label "PGC Comment" \
			-command "set ::main::last_menus_menu_selected $itemnum; ::utils::pgc_comment" -accelerator "Control+Shift+C"


	set menuname .selector_popup_menu_titles
	if {[winfo exists $menuname]} {destroy $menuname}
	menu $menuname
	if {$::config(tearoff_menus)} {set itemnum 1} else {set itemnum 0}
	set ::main::last_titles_menu_selected $itemnum

	if {$::config(use_preview)} {
		$menuname add command -label "Preview PGC" -accelerator "` (~), Control+P" -command \
				"set ::main::last_titles_menu_selected $itemnum; ::preview::preview_current_pgc_2"
		incr itemnum
		$menuname add separator
		incr itemnum
	}
	$menuname add command -label "Edit PGC" -accelerator "Return" \
			-command "set ::main::last_titles_menu_selected $itemnum; ::main::action_Edit_PGC"
	incr itemnum
	$menuname add command -label "Info: PGC" \
			-command "set ::main::last_titles_menu_selected $itemnum; ::macros::show_current_pgc_as_text"
	incr itemnum
	$menuname add command -label "Go to calling command"  -accelerator "Control+G" \
			-command "set ::main::last_titles_menu_selected $itemnum; ::utils::pgc_crossrefs"
	incr itemnum
	$menuname add separator
	incr itemnum
	$menuname add command -label "PTT (Chapter) Editor"  \
			-command "set ::main::last_titles_menu_selected $itemnum; ::ptted::ptted"
	incr itemnum
	$menuname add separator
	incr itemnum
	$menuname add command -label "Domain Streams Attributes" -accelerator "Control+A" \
			-command "set ::main::last_titles_menu_selected $itemnum; ::streams::set_streams_attribs ."
	incr itemnum
	$menuname add separator
	incr itemnum
	$menuname add command -label "Blank out all VTST Titles" -accelerator "Control+Delete" \
			-command "set ::main::last_titles_menu_selected $itemnum; ::utils::blank_out_PGCS_GUI"
	incr itemnum
	$menuname add command -label "New Menu" \
			-command "set ::main::last_titles_menu_selected $itemnum; ::utils::new_menu false"
	incr itemnum
	$menuname add command -label "Import Menu" \
			-command "set ::main::last_titles_menu_selected $itemnum; ::utils::import_menu"
	incr itemnum
	$menuname add command -label "BOVs (Buttons Over Video)" -accelerator "Control+M" \
			-command "set ::main::last_titles_menu_selected $itemnum; ::menubuttons::display_current_menu_buttons info"
	incr itemnum
	$menuname add command -label "New dummy PGC" \
			-command "set ::main::last_titles_menu_selected $itemnum; ::utils::new_pgc false false"
	incr itemnum
	$menuname add command -label "Clone PGC" \
			-command "set ::main::last_titles_menu_selected $itemnum; ::utils::clone_pgc"
	incr itemnum
	$menuname add separator
	incr itemnum
	$menuname add command -label "Kill PGC Playback" -accelerator "Control+K" \
			-command "set ::main::last_titles_menu_selected $itemnum; ::macros::kill_pgc_playback"
	incr itemnum
	$menuname add command -label "Enable all operations" \
			-command "set ::main::last_titles_menu_selected $itemnum; ::macros::enable_all_operations"
	incr itemnum
	$menuname add separator
	incr itemnum
	$menuname add command -label "Import Title's Intro Clip" \
			-command "set ::main::last_titles_menu_selected $itemnum; ::macros::import_title_intro_clip"
	incr itemnum
	$menuname add command -label "Import Title's Closing Clip" \
			-command "set ::main::last_titles_menu_selected $itemnum; ::macros::import_title_closing_clip"
	incr itemnum
	$menuname add command -label "New 'Play All' Title" \
			-command "set ::main::last_titles_menu_selected $itemnum; ::utils::create_playall_pgc"
	incr itemnum
	$menuname add separator
	incr itemnum
	$menuname add command -label "Toggle PGC pre/post breakpoint" -accelerator "Control+Shift+T" \
			-command "set ::main::last_titles_menu_selected $itemnum; ::tr::toggle_breakpoint prepost"
	incr itemnum
	$menuname add command -label "Toggle PGC playback breakpoint" -accelerator "Control+Alt+T" \
			-command "set ::main::last_titles_menu_selected $itemnum; ::tr::toggle_breakpoint playback"
	incr itemnum
	$menuname add separator
	incr itemnum
	$menuname add command -label "Delete all commands in PGC" \
			-command "set ::main::last_titles_menu_selected $itemnum; ::main::action_delete_all_commands"
	incr itemnum
	$menuname add command -label "Delete all commands in Domain!" \
			-command "set ::main::last_titles_menu_selected $itemnum; ::utils::delete_all_domain_commands"
	incr itemnum
	$menuname add separator
	incr itemnum
	$menuname add command -label "PGC Comment" \
			-command "set ::main::last_titles_menu_selected $itemnum; ::utils::pgc_comment" -accelerator "Control+Shift+C"


	set menuname .selector_popup_menu_notpgc
	if {[winfo exists $menuname]} {destroy $menuname}
	menu $menuname
	if {$::config(tearoff_menus)} {set itemnum 1} else {set itemnum 0}
	set ::main::last_notpgc_menu_selected $itemnum
	$menuname add command -label "Clear 'PGC played' highlights" \
			-command "set ::main::last_notpgc_menu_selected $itemnum; ::main::init_tree_menu_no_move"
	incr itemnum
	$menuname add command -label "Clear all comments and labels" \
			-command "set ::main::last_notpgc_menu_selected $itemnum; ::utils::clear_all_pgc_comments"
	incr itemnum
	$menuname add separator
	incr itemnum
	$menuname add command -label "Display PGCs list as text" \
			-command "set ::main::last_notpgc_menu_selected $itemnum; ::main::showlabels_as_text"
	incr itemnum
	$menuname add command -label "Hide PGC Selector pane" \
			-command "set ::main::last_notpgc_menu_selected $itemnum; ::main::selector toggle ::main::labelslist ::main::cmdslist"
	incr itemnum
	$menuname add separator
	incr itemnum
	$menuname add command -label "Delete all commands in DVD!" \
			-command "set ::main::last_notpgc_menu_selected $itemnum; ::utils::delete_all_DVD_commands"
}

proc build_mru_menu {} {
	set mnu .mainmenu.file.mru
	set cfgfilename [file join $::config(config_directory) "mru.cfg"]
	if {[file exists $cfgfilename]} {
		set mrulist [split [string trim [readFile $cfgfilename auto] \n] \n]
		if {[llength $mrulist] > 0} {
			destroy $mnu
			menu $mnu
			foreach i $mrulist {
				if {[file exists $i]} {
					set label [file nativename $i]
					if {[file tail $label] == "VIDEO_TS"} {
						set label [file dirname $label]
					}
					$mnu add command -label $label -command "::main::Read_New_DVD \{$i\}"
				}
			}
			$mnu add separator
			if {[llength $mrulist] < 2} {
				set state disabled
			} else {
				set state normal
			}
			$mnu add command -label "Sort by alphabetical order" -command "::menus::sort_mru_menu" -state $state
			$mnu add command -label "Delete all entries" -command "::menus::delete_mru_menu"
			return
		}
	}
	destroy $mnu
	menu $mnu
	$mnu add command -label "- No recent DVD yet -" -command bell -state disabled
}

proc save_mru_menu {} {
	if {![info exists ::dvddir]} return
	set dvdname $::dvddir
	set dvdname [file normalize $dvdname]

	set cfgfilename [file join $::config(config_directory) "mru.cfg"]
	set mrulist {}
	if {[file exists $cfgfilename]} {
		set mrulist [split [string trim [readFile $cfgfilename auto] \n] \n]
		set idx [lsearch $mrulist $dvdname]
		if {$idx > -1} {
			set mrulist [lreplace $mrulist $idx $idx]
		}
		set mrulist [linsert $mrulist 0 $dvdname]
	} else {
		lappend mrulist $dvdname
	}

	set txt {}
	set numelements 0
	foreach i $mrulist {
		if {[file exists $i]} {
			incr numelements
			if {$numelements > 25} break
			set txt $txt$i\n
		}
	}
	saveFile $cfgfilename auto [string trim $txt \n]
}

proc sort_mru_menu {} {
	set cfgfilename [file join $::config(config_directory) "mru.cfg"]
	set mrulist {}
	if {[file exists $cfgfilename]} {
		set mrulist [lsort [split [string trim [readFile $cfgfilename auto] \n] \n]]
	}
	set txt {}
	foreach i $mrulist {
		if {[file exists $i]} {
			set txt $txt$i\n
		}
	}
	saveFile $cfgfilename auto [string trim $txt \n]
	build_mru_menu
}

proc delete_mru_menu {} {
	file delete [file join $::config(config_directory) "mru.cfg"]
	build_mru_menu
}

} ;# end of namespace ::menus


namespace eval ::plugins {

proc install_plugin {} {
	catch {exec -- explorer.exe [file nativename $::config(plugindir)] &}
	set txt "To install a plugin, extract all plugin files from the plugin archive, and"
	set txt "$txt\ncopy them in the \"plugins\" folder window that should be opened now."
	set txt "$txt\nWhen done, restart PgcEdit."
	set txt "$txt\nA new submenu should be available in the PgcEdit's Plugins menu."
	set txt "$txt\n\nNote: If the plugin is made of several files, you should normally copy"
	set txt "$txt\nall files in the \"plugins\" folder.  Please refer to the plugin install file or"
	set txt "$txt\ngeneral documentation for details."
	set txt "$txt\n\nClose PgcEdit now?"
	if {[tk_messageBox  -title "Install a plugin" -icon question -message $txt -type yesno \
			-default yes] == "yes"} {
		cleanexit 0
	}
}

} ;# end of namespace ::plugins
