#############################################################################
# menued.tcl Copyright 2004, 2005, 2006, 2007 r0lZ
# Menu VOB buttons infos and editor.
#
# This file is part of PgcEdit.
#
# PgcEdit is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# PgcEdit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#############################################################################

# TODO
#
# KNOWN BUGS:
# - Strange bug (Tcl/Tk bug?) when importing several whole menus AND several color
#   schemes in the SAME PGC (but in different cells) without quitting the menu editor
#   and viewer.  PgcEdit may be abruptly closed, without error messages!
# - If you use the cursor keys to edit the Highlight end time field, the adjacent
#   buttons are selected as well.
# - Message "DVD Saved OK" do not appear anymore if buttons are saved.
# - Do not take into account the case of several new button definitions in the same cell!
#
# ENHANCEMENTS:
# - When entering a cell w/o buttons, offer to create a new button (OK) OR
#   to import a saved menu.
# - automatically calculate all position values in All mode
# - Autoroute function to calculate the links automatically
# - Use keys 1-9 to select buttons (also in viewer) - prob with numerical entry widget
# - Add tooltype on group buttons with group type descriptions
# - After Hide Button, automatically select next button (Warning: not in Delete mode!)
# - Check if it is safe to use "Delete Button"
# - Show All links flags (for up, down, right, left)
#
# MAYBE:
# - View color scheme 0 (not editable) (?)
# - Force action timing: replace -1 by checkbox
# - Allow to add a new group to existant buttons if the video flags are changed (?)
# - Avoid redraw of background image whenever possible.
# - Add Starting Button Number (Usage?)
# - Add new params in the button dumps
# - Function to change the subpic color and attributes.  (Big work!)
#
# TO BE VERIFIED:
# - Attention at highlight status > 2 !
# - Verify flag DVD Modified
# - After a Reread DVD or Open new DVD, all old variables should be cleared
# - Last VOBU end time must be >= force action & highlight end time
# - Test under Linux & OSX
#
# PROBS:
# - No error messages.  Why?


namespace eval ::menued {

proc launch_menued {type vts lu pgc cell vobid cellid butnum group} {
	if {![winfo exists .menubuttons]} {
		bell
		return 0
	}

	if {! [confirm_edit_several_sets $type $vts $vobid $cellid .menubuttons]} {
		return 0
	}

	set first_button_created false
	if {![info exists ::buttons::curbutsinfo($vobid,$cellid,numbuts)] || \
			$::buttons::curbutsinfo($vobid,$cellid,numbuts) == 0 ||
			![info exists ::buttons::curbutsinfo($vobid,$cellid,numgroups)] || \
			$::buttons::curbutsinfo($vobid,$cellid,numgroups) == 0} {
		# retrieve video stream attributes field
		if {$type == "menu"} {
			if {$vts == 0} {
				set mat_table_name VMGM_MAT
			} else {
				set mat_table_name VTSI_MAT
			}
			set mat_table_offset 0x100
		} else {
			set mat_table_name VTSI_MAT
			set mat_table_offset 0x200
		}
		set mat_table $::raw_sectors($vts,$mat_table_name)
		set vattr [get_int mat_table $mat_table_offset 2]

		if {![create_first_button $type $vts $vobid $cellid $vattr]} {
			bell
			return 0
		}
		set butnum 1
		set group 1
		set first_button_created true
	}
	init_menued_variables $type $vts $cell $vobid $cellid $group $butnum
	menued
	validate_command $::menued::command .menued.f2.cmd

	# select button group 'All' if buttons are identical
	if {$::menued::buts_are_identical} {
		set ::menued::gui_group All
		::menued::group_changed All
	}

	set ::menued::menued_done 0
	tkwait variable ::menued::menued_done
	if {$::menued::menued_done == 1} {
		if {[confirm_edit_several_sets $type $vts $vobid $cellid .menued]} {
			if {$::menued::gui_group == "All"} {
				for {set g 1} {$g <= $::menued::numgroups} {incr g} {
					button_or_group_changed $g $::menued::old_button
				}
				set ::menued::all_pos_changed false
			} else {
				button_or_group_changed $::menued::old_gui_group $::menued::old_button
			}
			remember_menued_variables $type $vts $vobid $cellid
			if {$first_button_created} {
				set numsubpics [get_int mat_table [expr {$mat_table_offset + 0x54}] 2]
				if {$numsubpics == 0} {
					set_int mat_table [expr {$mat_table_offset + 0x54}] 1 2
					set ::raw_sectors($vts,$mat_table_name) $mat_table
				}
				if {$type == "menu"} {
					set header $::menus($vts,$lu,$pgc,header)
				} else {
					set header $::pgcs($vts,$pgc,header)
				}
				set sub1 [get_int header 0x1C 1]
				if {[expr {$sub1 & 0x80}] == 0} {
					set_int header 0x1C [expr {$sub1 | 0x80}] 1
					if {$type == "menu"} {
						set ::menus($vts,$lu,$pgc,header) $header
					} else {
						set ::pgcs($vts,$pgc,header) $header
					}
				}
			}
			wm title .menubuttons "Menu buttons  *"
		}
	} else {
		if {$first_button_created} {
			set ::menued::numbuts 0
			set ::menued::button 0
			set ::buttons::curbutsinfo($vobid,$cellid,numgroups) 0
			set ::buttons::curbutsinfo($vobid,$cellid,numbuts) 0
		}
		if {$::menued::button > $::buttons::curbutsinfo($vobid,$cellid,numbuts)} {
			set ::buttons::default_button $::buttons::curbutsinfo($vobid,$cellid,numbuts)
		}
	}
	grab set .menubuttons
	focus .menubuttons
	# refresh viewer display
	if {$::menued::numbuts==0} {
		set ::buttons::default_button 0
	} else {
		set ::buttons::default_button $::menued::button
	}
	catch {eval [.menubuttons.fb.g$::menued::true_group cget -command]}
	set ::config(menueditor_win_geometry) [wm geometry .menued]
	destroy .menued

	return $::menued::menued_done
}

proc init_menued_variables {type vts cell vobid cellid group butnum} {
	set ::menued::cell $cell
	set ::menued::numgroups $::buttons::curbutsinfo($vobid,$cellid,numgroups)
	set ::menued::numbuts $::buttons::curbutsinfo($vobid,$cellid,numbuts)
	set ::menued::true_group $group
	set ::menued::gui_group $group
	set ::menued::old_gui_group $group
	set ::menued::all_pos_changed false
	set ::menued::button $butnum
	set ::menued::old_button $butnum
	set ::menued::hlstarttime $::buttons::curbutsinfo($vobid,$cellid,hlstarttime)
	set ::menued::hlendtime   $::buttons::curbutsinfo($vobid,$cellid,hlendtime)

	set endtime $::buttons::curbutsinfo($vobid,$cellid,selendtime)
	if {$endtime == [expr {(0xFFFFFFFF)}]} {set endtime -1}
	if {$endtime != -1} {
		set endtime [expr {(($endtime - $::menued::hlstarttime) / 90) / 1000.0}]
		if {$endtime < 0} {set endtime 0}
	}
	set ::menued::selendtime $endtime
	set ::menued::orig_selendtime $endtime

	if {$::menued::hlendtime != 0xFFFFFFFF} {
		set ::menued::maxselendtimesecs [expr (($::buttons::curbutsinfo($vobid,$cellid,hlendtime) \
				- $::buttons::curbutsinfo($vobid,$cellid,hlstarttime) + 1) / 90) / 1000.0]
	} else {
	    set ::menued::maxselendtimesecs -1
	}


	set ::menued::numselectablebuts $::buttons::curbutsinfo($vobid,$cellid,numselectablebuts)
	set ::menued::force_select_but  $::buttons::curbutsinfo($vobid,$cellid,forceselectbutn)
	set ::menued::force_action_but  $::buttons::curbutsinfo($vobid,$cellid,forceactionbutn)
	for {set g 1} {$g <= $::menued::numgroups} {incr g} {
		set ::menued::grouptype($g) $::buttons::curbutsinfo($vobid,$cellid,$g,grouptype)
		for {set b 1} {$b <= $::menued::numbuts} {incr b} {
			set posx [lindex $::buttons::curbutsinfo($vobid,$cellid,$g,$b,coords) 0]
			set posy [lindex $::buttons::curbutsinfo($vobid,$cellid,$g,$b,coords) 1]
			set ::menued::but(posx,$g,$b) $posx
			set ::menued::but(posy,$g,$b) $posy
			set ::menued::but(posw,$g,$b) [expr {[lindex $::buttons::curbutsinfo($vobid,$cellid,$g,$b,coords) 3] - $posx}]
			set ::menued::but(posh,$g,$b) [expr {[lindex $::buttons::curbutsinfo($vobid,$cellid,$g,$b,coords) 4] - $posy}]
			set ::menued::but(adj_but_up,$g,$b)    [lindex $::buttons::curbutsinfo($vobid,$cellid,$g,$b,udlr) 0]
			set ::menued::but(adj_but_down,$g,$b)  [lindex $::buttons::curbutsinfo($vobid,$cellid,$g,$b,udlr) 1]
			set ::menued::but(adj_but_left,$g,$b)  [lindex $::buttons::curbutsinfo($vobid,$cellid,$g,$b,udlr) 2]
			set ::menued::but(adj_but_right,$g,$b) [lindex $::buttons::curbutsinfo($vobid,$cellid,$g,$b,udlr) 3]
			set ::menued::but(color_scheme,$g,$b)   $::buttons::curbutsinfo($vobid,$cellid,$g,$b,colorscheme)
			set ::menued::but(auto_activate,$g,$b) $::buttons::curbutsinfo($vobid,$cellid,$g,$b,autoaction)
			set hex [string toupper $::buttons::curbutsinfo($vobid,$cellid,$g,$b,command)]
			set ::menued::but(command,$g,$b) "\[$hex\]\n[::disasm::disasm $hex]"
		}
	}
	if {$::menued::numgroups > 1} {
		set identical true
		foreach i {adj_but_up adj_but_down adj_but_left adj_but_right \
				color_scheme auto_activate command} {
			for {set b 1} {$b <= $::menued::numbuts} {incr b} {
				set v $::menued::but($i,1,$b)
				for {set g 2} {$g <= $::menued::numgroups} {incr g} {
					if {$::menued::but($i,$g,$b) != $v} {
						set identical false
						break
					}
				}
				if {! $identical} break
			}
			if {! $identical} break
		}
		set ::menued::buts_are_identical $identical
	} else {
		set ::menued::buts_are_identical false
	}
	# load the color table from disk if needed
	if {[info exists ::buttons::butsinfo($type,$vts,$vobid,$cellid,colortable)]} {
		set ::menued::colortable $::buttons::butsinfo($type,$vts,$vobid,$cellid,colortable)
	} else {
		set ::menued::colortable ""
		set poslist [get_nav_pack_position $type $vts $::buttons::butsinfo($type,$vts,$vobid,$cellid,navpack) $::dvddir]
		set vobfile [lindex $poslist 0]
		set sector  [lindex $poslist 1]
		if {$vobfile!=""} {
			set fh [openbinfile $vobfile r]
			if {$fh!=""} {
				unset -nocomplain ::menued::load_nav_pack_error_skip
				set navpack [load_nav_pack $vobfile $fh $sector]
				if {$navpack!=""} {
					set ::menued::colortable [string range $navpack 0xA3 0xBA]
					set ::buttons::butsinfo($type,$vts,$vobid,$cellid,colortable) $::menued::colortable
					unset navpack
				}
				close $fh
			}
		}
	}
	set ::menued::color_schemes_edited false
	if {$butnum > 0} {
		init_button $group $butnum
	}
}

proc remember_menued_variables {type vts vobid cellid} {
	if {$::menued::orig_selendtime != $::menued::selendtime} {
		set endtime $::menued::selendtime
		if {$endtime != -1} {
			set starttime $::menued::hlstarttime
			if {$starttime == -1} {set starttime 0}
			set endtime [expr {round($endtime * 90000) + $starttime}]
		}
#		set ::buttons::curbutsinfo($vobid,$cellid,hlendtime) $endtime
#		set ::buttons::butsinfo($type,$vts,$vobid,$cellid,hlendtime) $endtime
		set ::buttons::curbutsinfo($vobid,$cellid,selendtime) $endtime
		set ::buttons::butsinfo($type,$vts,$vobid,$cellid,selendtime) $endtime
 	} elseif {$::menued::hlstarttime == -1} {
 		set endtime $::menued::selendtime
 		if {$endtime != -1} {
 			set endtime [expr {($endtime * 90000)}]
 		}
# 		set ::buttons::curbutsinfo($vobid,$cellid,hlendtime) $endtime
# 		set ::buttons::butsinfo($type,$vts,$vobid,$cellid,hlendtime) $endtime
 		set ::buttons::curbutsinfo($vobid,$cellid,selendtime) $endtime
 		set ::buttons::butsinfo($type,$vts,$vobid,$cellid,selendtime) $endtime
	}
	set ::buttons::curbutsinfo($vobid,$cellid,hlstarttime) $::menued::hlstarttime
	set ::buttons::butsinfo($type,$vts,$vobid,$cellid,hlstarttime) $::menued::hlstarttime

	set ::buttons::curbutsinfo($vobid,$cellid,forceselectbutn) $::menued::force_select_but
	set ::buttons::butsinfo($type,$vts,$vobid,$cellid,forceselectbutn) $::menued::force_select_but
	set ::buttons::curbutsinfo($vobid,$cellid,numselectablebuts) $::menued::numselectablebuts
	set ::buttons::butsinfo($type,$vts,$vobid,$cellid,numselectablebuts) $::menued::numselectablebuts
	set ::buttons::curbutsinfo($vobid,$cellid,forceactionbutn) $::menued::force_action_but
	set ::buttons::butsinfo($type,$vts,$vobid,$cellid,forceactionbutn) $::menued::force_action_but
	set ::buttons::curbutsinfo($vobid,$cellid,numgroups) $::menued::numgroups
	set ::buttons::butsinfo($type,$vts,$vobid,$cellid,numgroups) $::menued::numgroups
	if {$::buttons::butsinfo($type,$vts,$vobid,$cellid,numbuts) != $::menued::numbuts} {
		set refresh_pgc_selector true
	} else {
		set refresh_pgc_selector false
	}
	set ::buttons::curbutsinfo($vobid,$cellid,numbuts) $::menued::numbuts
	set ::buttons::butsinfo($type,$vts,$vobid,$cellid,numbuts) $::menued::numbuts
	for {set g 1} {$g <= $::menued::numgroups} {incr g} {
		if {![info exists ::buttons::curbutsinfo($vobid,$cellid,grouptype)]} {
			set ::buttons::curbutsinfo($vobid,$cellid,$g,grouptype) $::menued::grouptype($g)
			set ::buttons::butsinfo($type,$vts,$vobid,$cellid,$g,grouptype) $::menued::grouptype($g)
		}
		for {set b 1} {$b <= $::menued::numbuts} {incr b} {
			set pos {}
			lappend pos $::menued::but(posx,$g,$b)
			lappend pos $::menued::but(posy,$g,$b)
			lappend pos "-"
			lappend pos [expr {$::menued::but(posx,$g,$b) + $::menued::but(posw,$g,$b)}]
			lappend pos [expr {$::menued::but(posy,$g,$b) + $::menued::but(posh,$g,$b)}]
			set ::buttons::curbutsinfo($vobid,$cellid,$g,$b,coords) $pos
			set ::buttons::butsinfo($type,$vts,$vobid,$cellid,$g,$b,coords) $pos
			set udlr [list \
					$::menued::but(adj_but_up,$g,$b)   $::menued::but(adj_but_down,$g,$b) \
					$::menued::but(adj_but_left,$g,$b) $::menued::but(adj_but_right,$g,$b)]
			set ::buttons::curbutsinfo($vobid,$cellid,$g,$b,udlr) $udlr
			set ::buttons::butsinfo($type,$vts,$vobid,$cellid,$g,$b,udlr) $udlr

			set ::buttons::curbutsinfo($vobid,$cellid,$g,$b,colorscheme) $::menued::but(color_scheme,$g,$b)
			set ::buttons::butsinfo($type,$vts,$vobid,$cellid,$g,$b,colorscheme) $::menued::but(color_scheme,$g,$b)
			set ::buttons::curbutsinfo($vobid,$cellid,$g,$b,autoaction) $::menued::but(auto_activate,$g,$b)
			set ::buttons::butsinfo($type,$vts,$vobid,$cellid,$g,$b,autoaction) $::menued::but(auto_activate,$g,$b)

			set cmd [string range $::menued::but(command,$g,$b) 1 23]
			set ::buttons::curbutsinfo($vobid,$cellid,$g,$b,command) $cmd
			set ::buttons::butsinfo($type,$vts,$vobid,$cellid,$g,$b,command) $cmd
		}
	}
	# keep color table if needed
	set colortable $::menued::colortable
	if {$::menued::color_schemes_edited} {
		set idx 0
		for {set scheme 1} {$scheme <= 3} {incr scheme} {
			foreach c {sel_color sel_opacity act_color act_opacity} {
				set b [expr {($::menued::color($scheme,${c}_0)*0x10) \
						+ $::menued::color($scheme,${c}_1)}]
				set_int colortable $idx $b 1
				incr idx

				set b [expr {($::menued::color($scheme,${c}_2)*0x10) \
						+ $::menued::color($scheme,${c}_3)}]
				set_int colortable $idx $b 1
				incr idx
			}
		}
	}
	set ::buttons::butsinfo($type,$vts,$vobid,$cellid,colortable) $colortable
	set ::menued::color_schemes_edited false
	# flag this cell to be saved later on disc
	set ::buttons::butsinfo($type,edited) 1
	set ::buttons::butsinfo($type,$vts,edited) 1
	set ::buttons::butsinfo($type,$vts,$vobid,edited) 1
	set ::buttons::butsinfo($type,$vts,$vobid,$cellid,edited) 1
	::main::dvd_modified true true
	# refresh PGC Selector GUI
	if {$refresh_pgc_selector} {
		if {$type == "menu"} {
			add_menu_buttons_marker $vts
		} else {
			add_title_buttons_marker $vts
		}
		::main::init_tree_menu_no_move
	}
}

proc create_first_button {type vts vobid cellid vattr} {
	if {![info exists ::buttons::butsinfo($type,$vts,$vobid,$cellid,navpack)]} {
		bell
		return 0
	}

	set txt "This cell has no buttons.\n\nDo you want to create a new button?"
	if {[tk_messageBox -title "Buttons editor" -icon question \
		-message $txt -parent .menubuttons -type yesno -default yes] == "no"} {
		return 0
	} else {
		set numgroups 1
		if {[expr {$vattr & 0x0C00}] == 0} {
			# 4:3
			set grouptype(1) 0
		} else {
			# 16:9
			set grouptype(1) 1
			set txt "normal"
			if {[expr {$vattr & 0x0100}] == 0} {
				# automatic letterbox
				incr numgroups
				set grouptype($numgroups) 2
				set txt "$txt + letterbox"
			}
			if {[expr {$vattr & 0x0200}] == 0} {
				# automatic Pan&Scan
				incr numgroups
				set grouptype($numgroups) 4
				set txt "$txt + pan&scan"
			}
			if {$numgroups > 1} {
				set txt "Do you want to create $numgroups groups (for $txt)?"
				set txt "$txt\n\nAnswer NO to create only one undefined group."
				set txt "$txt\n(You will not be able to accurately position the buttons.)"
				if {[tk_messageBox -title "Buttons editor" -icon question \
						-message $txt -parent .menubuttons -type yesno -default yes] == "no"} {
					set numgroups 1
					set grouptype(1) 0
				}
			}
		}
		set ::buttons::curbutsinfo($vobid,$cellid,hlstarttime) -1 ;# [get_int navpack 0x8F 4]
		set ::buttons::curbutsinfo($vobid,$cellid,hlendtime)   -1
		set ::buttons::curbutsinfo($vobid,$cellid,selendtime)  -1
		set ::buttons::curbutsinfo($vobid,$cellid,numselectablebuts) 1
		set ::buttons::curbutsinfo($vobid,$cellid,forceselectbutn) 0
		set ::buttons::curbutsinfo($vobid,$cellid,forceactionbutn) 0
		set ::buttons::curbutsinfo($vobid,$cellid,numgroups) $numgroups
		set ::buttons::curbutsinfo($vobid,$cellid,numbuts) 1
		#set ::buttons::butsinfo(pgc,$vts,errorcode) 0
		#set ::buttons::butsinfo(pgc,$vts,errormsg) "OK"
		for {set g 1} {$g <= $numgroups} {incr g} {
			set ::buttons::curbutsinfo($vobid,$cellid,$g,grouptype) $grouptype($g)
			set ::buttons::curbutsinfo($vobid,$cellid,$g,1,coords) {0 0 - 100 50}
			set ::buttons::curbutsinfo($vobid,$cellid,$g,1,udlr) {1 1 1 1}
			set ::buttons::curbutsinfo($vobid,$cellid,$g,1,colorscheme) 0
			set ::buttons::curbutsinfo($vobid,$cellid,$g,1,autoaction) 0
			set ::buttons::curbutsinfo($vobid,$cellid,$g,1,command) {20 01 00 00 00 00 00 0D}
		}
	}
	return 1
}

proc init_colors {} {
	if {[info exists ::menued::color_schemes_edited] && $::menued::color_schemes_edited} {
		return true
	}
	if {$::menued::colortable == ""} {
		return false
	}
	set idx 0
	for {set scheme 1} {$scheme <= 3} {incr scheme} {
		foreach c {sel_color sel_opacity act_color act_opacity} {
			set b [get_int ::menued::colortable $idx 1]
			set ::menued::color($scheme,${c}_0) [expr {$b/0x10}]
			set ::menued::color($scheme,${c}_1) [expr {$b&0xF}]
			incr idx
			set b [get_int ::menued::colortable $idx 1]
			set ::menued::color($scheme,${c}_2) [expr {$b/0x10}]
			set ::menued::color($scheme,${c}_3) [expr {$b&0xF}]
			incr idx
		}
	}
	return true
}

proc group_changed {group} {
	if {$group == $::menued::old_gui_group} return
	if {$::menued::old_gui_group == "All"} {
		for {set g 1} {$g <= $::menued::numgroups} {incr g} {
			button_or_group_changed $g $::menued::old_button
		}
		set ::menued::all_pos_changed false
	} else {
		button_or_group_changed $::menued::old_gui_group $::menued::old_button
	}
	.menued.f2.sel.lgn  configure -foreground black
	.menued.f2.sel.rgn1 configure -foreground black -relief flat
	.menued.f2.sel.rgn2 configure -foreground black -relief flat
	.menued.f2.sel.rgn3 configure -foreground black -relief flat

	if {$group == "All"} {
		.menued.f2.sel.rgn$::menued::old_gui_group configure -relief ridge
		set ::menued::all_pos_changed false
		set ::menued::true_group $::menued::old_gui_group
		init_button $::menued::true_group $::menued::old_button
		if {$::menued::numgroups == 2} {
			if {$::menued::grouptype(1)==2 || $::menued::grouptype(2)==2} {
				# letterboxed: disable the y & height sliders only
				::ced::scale_state .menued.f2.pos.spx normal
				::ced::scale_state .menued.f2.pos.spw normal
				::ced::scale_state .menued.f2.pos.spy disabled
				::ced::scale_state .menued.f2.pos.sph disabled
			} elseif {$::menued::grouptype(1)==4 || $::menued::grouptype(2)==4} {
				# pan$scan: disable the x & width sliders only
				::ced::scale_state .menued.f2.pos.spx disabled
				::ced::scale_state .menued.f2.pos.spw disabled
				::ced::scale_state .menued.f2.pos.spy normal
				::ced::scale_state .menued.f2.pos.sph normal
			} else {
				::ced::scale_state .menued.f2.pos.spx disabled
				::ced::scale_state .menued.f2.pos.spw disabled
				::ced::scale_state .menued.f2.pos.spy disabled
				::ced::scale_state .menued.f2.pos.sph disabled
			}
		} else {
			::ced::scale_state .menued.f2.pos.spx disabled
			::ced::scale_state .menued.f2.pos.spw disabled
			::ced::scale_state .menued.f2.pos.spy disabled
			::ced::scale_state .menued.f2.pos.sph disabled
		}
	} else {
		if {$::menued::true_group != $group} {
			set ::buttons::groupnum $group
			set cmd [.menubuttons.fb.g$group cget -command]
			eval $cmd
			for {set but 1} {$but <= $::menued::numbuts} {incr but} {
				if {[winfo exists .menubuttons.video.f.b$but]} {
					place forget .menubuttons.video.f.b$but
				}
			}
		}
		set ::menued::true_group $group
		init_button $group $::menued::old_button
		::ced::scale_state .menued.f2.pos.spx normal
		::ced::scale_state .menued.f2.pos.spw normal
		::ced::scale_state .menued.f2.pos.spy normal
		::ced::scale_state .menued.f2.pos.sph normal

		if {$::menued::numgroups > 1} {
			.menued.f2.sel.lgn  configure -foreground red
			.menued.f2.sel.rgn$group configure -foreground red -relief ridge
		}
	}
	catch draw_all_buttons
	catch draw_button_outline
	set ::menued::old_gui_group $group
}

proc button_changed {but} {
	if {$but == $::menued::old_button} return
	if {$but > $::menued::numbuts} {
		bell
		return
	}
	if {$::menued::old_gui_group == "All"} {
		for {set g 1} {$g <= $::menued::numgroups} {incr g} {
			button_or_group_changed $g $::menued::old_button
		}
		set ::menued::all_pos_changed false
	} else {
		button_or_group_changed $::menued::true_group $::menued::old_button
	}
	init_button $::menued::true_group $but
	draw_all_buttons
	draw_button_outline

	# display current button in the viewer
	set ::buttons::default_button $but

	## Highlight target of current button command
	#if [winfo exists .menubuttons.video.f.b$but] {
	#	set cmd [.menubuttons.video.f.b$but cget -command]
	#	eval $cmd
	#	place forget .menubuttons.video.f.b$but
	#}

	set ::menued::old_button $but
}

### TODO: save correct values in All groups mode
proc button_or_group_changed {gr but} {
	if {[.menued.f2.pos.spx cget -state] == "normal" && $::menued::all_pos_changed} {
		set ::menued::but(posx,$gr,$but) $::menued::posx
		set ::menued::but(posw,$gr,$but) $::menued::posw
	}
	if {[.menued.f2.pos.spy cget -state] == "normal" && $::menued::all_pos_changed} {
		set ::menued::but(posy,$gr,$but) $::menued::posy
		set ::menued::but(posh,$gr,$but) $::menued::posh
	}
	set ::menued::but(adj_but_up,$gr,$but) $::menued::adj_but_up
	set ::menued::but(adj_but_down,$gr,$but) $::menued::adj_but_down
	set ::menued::but(adj_but_left,$gr,$but) $::menued::adj_but_left
	set ::menued::but(adj_but_right,$gr,$but) $::menued::adj_but_right
	set ::menued::but(color_scheme,$gr,$but) $::menued::color_scheme
	set ::menued::but(auto_activate,$gr,$but) $::menued::auto_activate
	set ::menued::but(command,$gr,$but) $::menued::command
	return
}

proc init_button {group but} {
	set ::menued::posx $::menued::but(posx,$group,$but)
	set ::menued::posy $::menued::but(posy,$group,$but)
	set ::menued::posw $::menued::but(posw,$group,$but)
	set ::menued::posh $::menued::but(posh,$group,$but)
	set ::menued::adj_but_up $::menued::but(adj_but_up,$group,$but)
	set ::menued::adj_but_down $::menued::but(adj_but_down,$group,$but)
	set ::menued::adj_but_left $::menued::but(adj_but_left,$group,$but)
	set ::menued::adj_but_right $::menued::but(adj_but_right,$group,$but)
	set ::menued::color_scheme $::menued::but(color_scheme,$group,$but)
	set ::menued::auto_activate $::menued::but(auto_activate,$group,$but)
	set ::menued::command $::menued::but(command,$group,$but)
	validate_command $::menued::command .menued.f2.cmd
}

proc undo_button_changes {} {
	init_button $::menued::true_group $::menued::button
	draw_button_outline
}

proc select_adjacent_button {dir} {
	set adj [subst \$::menued::$dir]
	if {$adj != $::menued::button && $adj >= 1 && $adj <= $::menued::numbuts} {
		set ::menued::button $adj
		button_changed $adj
	}
}

proc allign_button {cr} {
	set cur $::menued::button
	set ref [select_button .menued "Align button $cur on same $cr with button:" \
			"Caution: Works on all groups.  No Undo!"]
	if {$ref == 0} return
	if {$ref == $cur} {
		bell
		return
	}
	button_or_group_changed $::menued::true_group $cur
	if {$cr == "column"} {
		for {set g 1} {$g <= $::menued::numgroups} {incr g} {
			set ::menued::but(posx,$g,$cur) $::menued::but(posx,$g,$ref)
			set ::menued::but(posw,$g,$cur) $::menued::but(posw,$g,$ref)
		}
	} else {
		for {set g 1} {$g <= $::menued::numgroups} {incr g} {
			set ::menued::but(posy,$g,$cur) $::menued::but(posy,$g,$ref)
			set ::menued::but(posh,$g,$cur) $::menued::but(posh,$g,$ref)
		}
	}
	init_menued_gui .menued $::menued::numgroups $::menued::numbuts
	init_button $::menued::true_group $::menued::button
	draw_button_outline
}
proc select_button {parent label {label2 ""}} {
	set utiltitle "Button?"
	set w .select_button

	### GUI
	if {[winfo exists $w]} {destroy $w}
	toplevel $w -class Dialog -menu $w.mnu
	wm withdraw $w
	wm title $w $utiltitle
	if {$::config(tk) != "aqua"} {wm transient $w .}
	if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
	wm protocol $w WM_DELETE_WINDOW {set ::menued::select_button_done 0}
	bind $w <Escape> {set ::menued::select_button_done 0}
	bind $w <Return> {set ::menued::select_button_done 1}

	label $w.lbn -text $label
	pack  $w.lbn -padx 4 -pady 6
	scale $w.sbn -from 1 -to $::menued::numbuts -variable ::menued::select_button \
			-command {::menued::highlight_button pink}
	pack  $w.sbn -fill x -expand 1 -padx 4
	if {$label2 != ""} {
		label $w.lb2 -text $label2
		pack  $w.lb2 -padx 4 -pady 2
	}

	# OK/Cancel buttons
	frame  $w.fb
	pack   $w.fb -padx 2 -pady 4 -anchor e
	button $w.fb.ok -text "OK" -width 6 -command {set ::menued::select_button_done 1}
	pack   $w.fb.ok -side left -padx 2
	button $w.fb.cancel -text "Cancel" -width 6 -command {set ::menued::select_button_done 0}
	pack   $w.fb.cancel -side left -padx 2

	update
	set x [expr {[winfo pointerx $parent] - ([winfo reqwidth $w]/2)}]
	set y [expr {[winfo pointery $parent] - ([winfo reqheight $w]/2) -20}]
	if {$x < 10} {set x 10}
	if {$y < 20} {set y 20}
	wm geometry $w "+$x+$y"
	wm minsize $w [winfo reqwidth $w] [winfo reqheight $w]
	wm maxsize $w [winfo reqwidth $w] [winfo reqheight $w]
	focus $w
	grab set $w
	wm deiconify $w

	if {! [info exists ::menued::select_button]} {
		set ::menued::select_button $::menued::button
	}

	tkwait variable ::menued::select_button_done
	destroy $w
	grab set $parent
	if {$::menued::select_button_done == 1} {
		return $::menued::select_button
	} else {
		return 0
	}
}

proc export_whole_menu {{clip false}} {
	if {!$clip} {
		set def_filename [file rootname [::main::get_default_filename $::editor_cmds_from]]_c$::menued::cell.mnu
		set filename [tk_getSaveFile -filetypes {{"PgcEdit exported menu" {*.mnu}}} \
			-parent .menued -initialdir $::config(import_initialdir) \
			-initialfile $def_filename -title "PgcEdit: Export whole menu"]
		if {$filename == ""} return
		if {[string toupper [file extension $filename]] != ".MNU"} {
			set filename "${filename}.mnu"
		}
		set ::config(import_initialdir) [file dirname $filename]
	}
	# remember current values
	button_or_group_changed $::menued::true_group $::menued::button

	set txt "# PgcEdit's exported buttons file.\n"
	set txt "${txt}# Edit at your own risk!\n\n"

	set txt "${txt}# Number of buttons and groups:\n"
	set txt "${txt}set ::menued::numbuts $::menued::numbuts\n"
	set txt "${txt}set ::menued::numgroups $::menued::numgroups\n\n"

	set txt "${txt}# Group types (0=undefined, 1=normal, 2=letterbox, 4=pan&scan):\n"
	for {set g 1} {$g <= $::menued::numgroups} {incr g} {
		set txt "${txt}set ::menued::grouptype($g) $::menued::grouptype($g)\n"
	}

	for {set b 1} {$b <= $::menued::numbuts} {incr b} {
		set txt "${txt}\n# Button $b:\n"
		for {set g 1} {$g <= $::menued::numgroups} {incr g} {
			if {$::menued::numgroups > 1} {
				set txt "${txt}# Group $g\n"
			}
			set txt "${txt}set ::menued::but(posx,$g,$b) $::menued::but(posx,$g,$b)\n"
			set txt "${txt}set ::menued::but(posw,$g,$b) $::menued::but(posw,$g,$b)\n"
			set txt "${txt}set ::menued::but(posy,$g,$b) $::menued::but(posy,$g,$b)\n"
			set txt "${txt}set ::menued::but(posh,$g,$b) $::menued::but(posh,$g,$b)\n"
			set txt "${txt}set ::menued::but(adj_but_up,$g,$b)    $::menued::but(adj_but_up,$g,$b)\n"
			set txt "${txt}set ::menued::but(adj_but_left,$g,$b)  $::menued::but(adj_but_left,$g,$b)\n"
			set txt "${txt}set ::menued::but(adj_but_right,$g,$b) $::menued::but(adj_but_right,$g,$b)\n"
			set txt "${txt}set ::menued::but(adj_but_down,$g,$b)  $::menued::but(adj_but_down,$g,$b)\n"
			set txt "${txt}set ::menued::but(color_scheme,$g,$b)   $::menued::but(color_scheme,$g,$b)\n"
			set txt "${txt}set ::menued::but(auto_activate,$g,$b) $::menued::but(auto_activate,$g,$b)\n"
			set cmd [string range $::menued::but(command,$g,$b) 1 23]
			set txt "${txt}set ::menued::but(command,$g,$b) \{\[$cmd\]\}\n"
			set txt "${txt}# [::disasm::disasm $cmd]\n\n"
		}
	}

	set txt "${txt}\n# Global parameters:\n"
	set txt "${txt}set ::menued::force_select_but  $::menued::force_select_but\n"
	set txt "${txt}set ::menued::force_action_but  $::menued::force_action_but\n"
	if {$::menued::selendtime == [expr {(0xFFFFFFFF)}]} {
		set time -1
	} else {
		set time $::menued::selendtime
	}
	set txt "${txt}set ::menued::selendtime        $time\n"
	set txt "${txt}set ::menued::numselectablebuts $::menued::numselectablebuts"

	if {$clip} {
		clipboard clear
		clipboard append $txt
	} else {
		saveFile $filename auto $txt
	}
}
proc import_whole_menu {{clip false}} {
	set old_button_group $::menued::true_group
	set old_button $::menued::button
	if {$clip} {
		set txt [clipboard get]
		if {[string first "# PgcEdit's exported buttons file." $txt] < 0} {
			bell
			return
		}
		if {[catch {eval $txt} errstr]} {
			tk_messageBox -title "PgcEdit: Paste whole menu" -icon error -parent .menued -type ok \
					-message "Error pasting exported buttons.\n\n$errstr"
			return
		}
	} else {
		set def_filename [file rootname [::main::get_default_filename $::editor_cmds_from]]_c$::menued::cell.mnu
		set filename [tk_getOpenFile -filetypes {{"PgcEdit exported menu" {*.mnu}}} \
			-parent .menued -initialdir $::config(import_initialdir) \
			-initialfile $def_filename -title "PgcEdit: Import whole menu"]
		if {$filename == ""} return
		set ::config(import_initialdir) [file dirname $filename]
		if {! [file exists $filename]} {
			set filename $filename.mnu
		}
		if {[catch {source $filename} errstr]} {
			tk_messageBox -title "PgcEdit: Import whole menu" -icon error -parent .menued -type ok \
					-message "Error interpreting exported button file\n\"$filename\".\n\n$errstr"
			return
		}
	}

	if {$::menued::numgroups > 1} {
		set identical true
		foreach i {adj_but_up adj_but_down adj_but_left adj_but_right \
				color_scheme auto_activate command} {
			for {set b 1} {$b <= $::menued::numbuts} {incr b} {
				set v $::menued::but($i,1,$b)
				for {set g 2} {$g <= $::menued::numgroups} {incr g} {
					if {$::menued::but($i,$g,$b) != $v} {
						set identical false
						break
					}
				}
				if {! $identical} break
			}
			if {! $identical} break
		}
		set ::menued::buts_are_identical $identical
	} else {
		set ::menued::buts_are_identical false
	}

	for {set g 1} {$g <= $::menued::numgroups} {incr g} {
		for {set b 1} {$b <= $::menued::numbuts} {incr b} {
			set cmd [string range $::menued::but(command,$g,$b) 1 23]
			set ::menued::but(command,$g,$b) "\[$cmd\]\n[::disasm::disasm $cmd]"
		}
	}

	init_menued_gui .menued $::menued::numgroups $::menued::numbuts
	set old_grouptype $::menued::grouptype($::menued::true_group)

	if {$old_button_group > $::menued::numgroups} {
		set ::menued::true_group 1
	} else {
		set ::menued::true_group $old_button_group
	}
	.menued.f2.sel.lgn  configure -foreground black
	.menued.f2.sel.rgn1 configure -foreground black -relief flat
	.menued.f2.sel.rgn2 configure -foreground black -relief flat
	.menued.f2.sel.rgn3 configure -foreground black -relief flat
	.menued.f2.sel.rgn$::menued::true_group configure -relief ridge
	if {$::menued::buts_are_identical || $::menued::numgroups == 1} {
		set ::menued::gui_group All
		.menued.f2.sel.rgn4 configure -background $::config(systembuttonface) -relief flat
	} else {
		set ::menued::gui_group $::menued::true_group
		.menued.f2.sel.lgn  configure -foreground red
		.menued.f2.sel.rgn$::menued::true_group configure -foreground red
		.menued.f2.sel.rgn4 configure -background #FF8080 -relief ridge
	}
	set new_grouptype $::menued::grouptype($::menued::true_group)
	if {$::menued::button < $old_button} {
		set ::menued::button 1
		set ::menued::old_button 1
	}
	init_button $::menued::true_group $::menued::button
	draw_button_outline
	if {$new_grouptype != $old_grouptype} {
		set ::menued::true_group 0
		set ::menued::old_gui_group 0
		group_changed $::menued::gui_group
	}
}
proc export_GUI_values {{clip false} {onlypos false}} {
	if {!$clip} {
		set def_filename [file rootname [::main::get_default_filename $::editor_cmds_from]]_c${::menued::cell}_g${::menued::true_group}_b${::menued::button}.gbv
		set filename [tk_getSaveFile -filetypes {{"PgcEdit exported GUI button values" {*.gbv}}} \
			-parent .menued -initialdir $::config(import_initialdir) \
			-initialfile $def_filename -title "PgcEdit: Export current button"]
		if {$filename == ""} return
		if {[string toupper [file extension $filename]] != ".GBV"} {
			set filename "${filename}.gbv"
		}
		set ::config(import_initialdir) [file dirname $filename]
	}
	set txt "# PgcEdit's exported GUI button values file.\n"
	set txt "${txt}# Edit at your own risk!\n\n"

	set txt "${txt}set ::menued::posx $::menued::posx\n"
	set txt "${txt}set ::menued::posw $::menued::posw\n"
	set txt "${txt}set ::menued::posy $::menued::posy\n"
	set txt "${txt}set ::menued::posh $::menued::posh\n"
	if {!$onlypos} {
		set txt "${txt}set ::menued::adj_but_up    $::menued::adj_but_up\n"
		set txt "${txt}set ::menued::adj_but_left  $::menued::adj_but_left\n"
		set txt "${txt}set ::menued::adj_but_right $::menued::adj_but_right\n"
		set txt "${txt}set ::menued::adj_but_down  $::menued::adj_but_down\n"
		set txt "${txt}set ::menued::color_scheme   $::menued::color_scheme\n"
		set txt "${txt}set ::menued::auto_activate $::menued::auto_activate\n"
		set cmd [string range $::menued::command 1 23]
		set txt "${txt}set ::menued::command \{\[$cmd\]\}\n"
		set txt "${txt}# [::disasm::disasm $cmd]"
	}

	if {$clip} {
		clipboard clear
		clipboard append $txt
	} else {
		saveFile $filename auto $txt
	}
}
proc import_GUI_values {{clip false}} {
	set oldx $::menued::posx
	set oldw $::menued::posw
	set oldy $::menued::posy
	set oldh $::menued::posh
	if {$clip} {
		set txt [clipboard get]
		if {[string first "# PgcEdit's exported GUI button values file." $txt] < 0} {
			bell
			return
		}
		if {[catch {eval $txt} errstr]} {
			tk_messageBox -title "PgcEdit: Paste button" -icon error -parent .menued -type ok \
					-message "Error pasting current button values.\n\n$errstr"
			return
		}
	} else {
		set def_filename [file rootname [::main::get_default_filename $::editor_cmds_from]]_c${::menued::cell}_g${::menued::true_group}_b${::menued::button}.gbv
		set filename [tk_getOpenFile -filetypes {{"PgcEdit exported GUI button values" {*.gbv}}} \
			-parent .menued -initialdir $::config(import_initialdir) \
			-initialfile $def_filename -title "PgcEdit: Import current button"]
		if {$filename == ""} return
		set ::config(import_initialdir) [file dirname $filename]
		if {! [file exists $filename]} {
			set filename $filename.gbv
		}
		if {[catch {source $filename} errstr]} {
			tk_messageBox -title "PgcEdit: Import current button values" -icon error -parent .menued -type ok \
					-message "Error interpreting exported button file\n\"$filename\".\n\n$errstr"
			return
		}
	}
	set cmd [string range $::menued::command 1 23]
	set ::menued::command "\[$cmd\]\n[::disasm::disasm $cmd]"
	validate_command $::menued::command .menued.f2.cmd
	if {[.menued.f2.pos.spx cget -state] == "disabled"} {
		set ::menued::posx $oldx
		set ::menued::posw $oldw
	}
	if {[.menued.f2.pos.spy cget -state] == "disabled"} {
		set ::menued::posy $oldy
		set ::menued::posh $oldh
	}
	draw_button_outline
	set ::menued::all_pos_changed 1
}
proc export_color_schemes {{clip false}} {
	if {! [init_colors]} {
		bell
		return
	}
	if {!$clip} {
		set filename [tk_getSaveFile -filetypes {{"PgcEdit exported color schemes" {*.csm}}} \
			-parent .menued -initialdir $::config(import_initialdir) \
			-initialfile [file rootname [::main::get_default_filename $::editor_cmds_from]]_c${::menued::cell}.csm \
			-title "PgcEdit: Export color schemes"]
		if {$filename == ""} return
		if {[string toupper [file extension $filename]] != ".CSM"} {
			set filename "${filename}.csm"
		}
		set ::config(import_initialdir) [file dirname $filename]
	}
	set txt "# PgcEdit's exported color schemes.\n"
	set txt "${txt}# Edit at your own risk!\n\n"

	for {set scheme 1} {$scheme <= 3} {incr scheme} {
		set txt "${txt}# Color scheme $scheme:\n"
		foreach c {sel_color sel_opacity act_color act_opacity} {
			set txt "${txt}set ::menued::color($scheme,${c}_3) $::menued::color($scheme,${c}_3)\n"
			set txt "${txt}set ::menued::color($scheme,${c}_2) $::menued::color($scheme,${c}_2)\n"
			set txt "${txt}set ::menued::color($scheme,${c}_1) $::menued::color($scheme,${c}_1)\n"
			set txt "${txt}set ::menued::color($scheme,${c}_0) $::menued::color($scheme,${c}_0)\n\n"
		}
	}
	if {$clip} {
		clipboard clear
		clipboard append $txt
	} else {
		saveFile $filename auto $txt
	}
}
proc import_color_schemes {{clip false}} {
	init_colors
	for {set scheme 1} {$scheme <= 3} {incr scheme} {
		foreach c {sel_color sel_opacity act_color act_opacity} {
			set back_color($scheme,${c}_3) $::menued::color($scheme,${c}_3)
			set back_color($scheme,${c}_2) $::menued::color($scheme,${c}_2)
			set back_color($scheme,${c}_1) $::menued::color($scheme,${c}_1)
			set back_color($scheme,${c}_0) $::menued::color($scheme,${c}_0)
		}
	}
	if {$clip} {
		set txt [clipboard get]
		if {[string first "# PgcEdit's exported color schemes." $txt] < 0} {
			bell
			return
		}
		if {[catch {eval $txt} errstr]} {
			tk_messageBox -title "PgcEdit: Paste color schemes" -icon error -parent .menued -type ok \
					-message "Error pasting color shames.\n\n$errstr"
			return
		}
	} else {
		set filename [tk_getOpenFile \
			-filetypes {{"PgcEdit exported color schemes" {*.csm}}} \
			-parent .menued -initialdir $::config(import_initialdir) \
			-initialfile [file rootname [::main::get_default_filename $::editor_cmds_from]]_c${::menued::cell}.csm \
			-title "PgcEdit: Import color schemes"]
		if {$filename == ""} return
		set ::config(import_initialdir) [file dirname $filename]
		if {! [file exists $filename]} {
			set filename $filename.csm
		}
		if {[catch {source $filename} errstr]} {
			tk_messageBox -title "PgcEdit: Import color schemes" -icon error -parent .menued -type ok \
					-message "Error loading exported color schemes\n\"$filename\".\n\n$errstr"
			return
		}
	}
	set old_edited $::menued::color_schemes_edited
	set ::menued::color_schemes_edited true
	if {![colorschemesed]} {
		set ::menued::color_schemes_edited $old_edited
		for {set scheme 1} {$scheme <= 3} {incr scheme} {
			foreach c {sel_color sel_opacity act_color act_opacity} {
				set ::menued::color($scheme,${c}_3) $back_color($scheme,${c}_3)
				set ::menued::color($scheme,${c}_2) $back_color($scheme,${c}_2)
				set ::menued::color($scheme,${c}_1) $back_color($scheme,${c}_1)
				set ::menued::color($scheme,${c}_0) $back_color($scheme,${c}_0)
			}
		}
	}
}

proc renumber_button {direction} {
	set cur $::menued::button
	if {($cur == 1 && $direction == -1) || ($cur == $::menued::numbuts && $direction == 1)} {
		bell
		return
	}
	set swap [expr $cur + $direction]

	for {set g 1} {$g <= $::menued::numgroups} {incr g} {
		set tmp(posx) $::menued::but(posx,$g,$cur)
		set tmp(posw) $::menued::but(posw,$g,$cur)
		set tmp(posy) $::menued::but(posy,$g,$cur)
		set tmp(posh) $::menued::but(posh,$g,$cur)
		set tmp(adj_but_up)    $::menued::but(adj_but_up,$g,$cur)
		set tmp(adj_but_left)  $::menued::but(adj_but_left,$g,$cur)
		set tmp(adj_but_right) $::menued::but(adj_but_right,$g,$cur)
		set tmp(adj_but_down)  $::menued::but(adj_but_down,$g,$cur)
		set tmp(color_scheme)  $::menued::but(color_scheme,$g,$cur)
		set tmp(auto_activate) $::menued::but(auto_activate,$g,$cur)
		set tmp(command) $::menued::but(command,$g,$cur)

		set ::menued::but(posx,$g,$cur) $::menued::but(posx,$g,$swap)
		set ::menued::but(posw,$g,$cur) $::menued::but(posw,$g,$swap)
		set ::menued::but(posy,$g,$cur) $::menued::but(posy,$g,$swap)
		set ::menued::but(posh,$g,$cur) $::menued::but(posh,$g,$swap)
		set ::menued::but(adj_but_up,$g,$cur)    $::menued::but(adj_but_up,$g,$swap)
		set ::menued::but(adj_but_left,$g,$cur)  $::menued::but(adj_but_left,$g,$swap)
		set ::menued::but(adj_but_right,$g,$cur) $::menued::but(adj_but_right,$g,$swap)
		set ::menued::but(adj_but_down,$g,$cur)  $::menued::but(adj_but_down,$g,$swap)
		set ::menued::but(color_scheme,$g,$cur)  $::menued::but(color_scheme,$g,$swap)
		set ::menued::but(auto_activate,$g,$cur) $::menued::but(auto_activate,$g,$swap)
		set ::menued::but(command,$g,$cur) $::menued::but(command,$g,$swap)

		set ::menued::but(posx,$g,$swap) $tmp(posx)
		set ::menued::but(posw,$g,$swap) $tmp(posw)
		set ::menued::but(posy,$g,$swap) $tmp(posy)
		set ::menued::but(posh,$g,$swap) $tmp(posh)
		set ::menued::but(adj_but_up,$g,$swap)    $tmp(adj_but_up)
		set ::menued::but(adj_but_left,$g,$swap)  $tmp(adj_but_left)
		set ::menued::but(adj_but_right,$g,$swap) $tmp(adj_but_right)
		set ::menued::but(adj_but_down,$g,$swap)  $tmp(adj_but_down)
		set ::menued::but(color_scheme,$g,$swap)  $tmp(color_scheme)
		set ::menued::but(auto_activate,$g,$swap) $tmp(auto_activate)
		set ::menued::but(command,$g,$swap) $tmp(command)
	}

	for {set b 1} {$b <= $::menued::numbuts} {incr b} {
		for {set g 1} {$g <= $::menued::numgroups} {incr g} {
			if {$::menued::but(adj_but_up,$g,$b) == $cur} {
				set ::menued::but(adj_but_up,$g,$b) $swap
			} elseif {$::menued::but(adj_but_up,$g,$b) == $swap} {
				set ::menued::but(adj_but_up,$g,$b) $cur
			}
			if {$::menued::but(adj_but_left,$g,$b) == $cur} {
				set ::menued::but(adj_but_left,$g,$b) $swap
			} elseif {$::menued::but(adj_but_left,$g,$b) == $swap} {
				set ::menued::but(adj_but_left,$g,$b) $cur
			}
			if {$::menued::but(adj_but_right,$g,$b) == $cur} {
				set ::menued::but(adj_but_right,$g,$b) $swap
			} elseif {$::menued::but(adj_but_right,$g,$b) == $swap} {
				set ::menued::but(adj_but_right,$g,$b) $cur
			}
			if {$::menued::but(adj_but_down,$g,$b) == $cur} {
				set ::menued::but(adj_but_down,$g,$b) $swap
			} elseif {$::menued::but(adj_but_down,$g,$b) == $swap} {
				set ::menued::but(adj_but_down,$g,$b) $cur
			}
		}
	}

	init_menued_gui .menued $::menued::numgroups $::menued::numbuts
	set ::menued::button $swap
	set ::menued::old_button 0
	button_changed $swap
}

proc init_select_and_action_buttons {w numbuts} {
	catch {destroy $w.f1.fs.ssb.m}
	menu $w.f1.fs.ssb.m -tearoff false
	catch {destroy $w.f1.fa.f1.sab.m}
	menu $w.f1.fa.f1.sab.m -tearoff false
	$w.f1.fs.ssb.m add command -label "button in sprm(8)" -command {
					set ::menued::force_select_but 0
					set ::menued::force_select_but_text "button in sprm(8)"}
	$w.f1.fa.f1.sab.m add command -label "no button (disabled)" -command {
					set ::menued::force_action_but 0
					set ::menued::force_action_but_text "no button (disabled)"
					::menued::warning_action_but_in_still .menued}
	for {set b 1} {$b <= $numbuts} {incr b} {
		$w.f1.fs.ssb.m add command -label "button $b" -command \
			"set ::menued::force_select_but $b; set ::menued::force_select_but_text \"button $b\"; ::menued::highlight_target_button none $b"
		$w.f1.fa.f1.sab.m add command -label "button $b" -command \
			"set ::menued::force_action_but $b; set ::menued::force_action_but_text \"button $b\"; ::menued::highlight_target_button none $b; ::menued::warning_action_but_in_still .menued"
	}
	$w.f1.fa.f1.sab.m add command -label "the selected button" -command {
					set ::menued::force_action_but 63
					set ::menued::force_action_but_text "the selected button"
					::menued::warning_action_but_in_still .menued}

	set ::menued::force_select_but_text [$w.f1.fs.ssb.m entrycget $::menued::force_select_but -label]
	if {$::menued::force_action_but == 63} {
	    set ::menued::force_action_but_text "the selected button"
	} else {
	    set ::menued::force_action_but_text [$w.f1.fa.f1.sab.m entrycget $::menued::force_action_but -label]
	}
	return
}

proc init_menued_gui {w numgroups numbuts} {
	if {$numbuts <= 0} {set numbuts 1}
    init_select_and_action_buttons $w $numbuts

	if {$numbuts > 9} {
		$w.f1.lnsf.sns configure -to 9
	} else {
		$w.f1.lnsf.sns configure -to $numbuts
	}
	$w.f2.sel.sbn configure -to $numbuts
	$w.f2.ab.sup configure -to $numbuts
	$w.f2.ab.slf configure -to $numbuts
	$w.f2.ab.srg configure -to $numbuts
	$w.f2.ab.sdn configure -to $numbuts

	.menued.f2.sel.lgn  configure -foreground black
	.menued.f2.sel.rgn1 configure -foreground black -relief flat
	.menued.f2.sel.rgn2 configure -foreground black -relief flat
	.menued.f2.sel.rgn3 configure -foreground black -relief flat

	if {$numgroups < 3} {
		$w.f2.sel.rgn3 configure -state disabled
	} else {
		$w.f2.sel.rgn3 configure -state normal
	}
	if {$numgroups < 2} {
		$w.f2.sel.rgn2 configure -state disabled
	} else {
		$w.f2.sel.rgn2 configure -state normal
	}
	if {$numgroups > 1} {
		.menued.f2.sel.rgn$::menued::true_group configure -relief ridge
		if {$::menued::gui_group != "All"} {
			.menued.f2.sel.lgn  configure -foreground red
			.menued.f2.sel.rgn$::menued::true_group configure -foreground red
		}
		$w.f2.sel.rgn4 configure -state normal
		if {!$::menued::buts_are_identical} {
			$w.f2.sel.rgn4 configure -background #FF8080 -relief ridge
			::balloon::set_balloon $w.f2.sel.rgn4 "WARNING!\nThere are parameters values that are not\nidentical in the different groups.\nIf you select \"All\" groups, the current button\nparameters will be copied to all groups!"
		}
	} else {
		.menued.f2.sel.rgn1 configure -relief ridge
		$w.f2.sel.rgn4 configure -state disabled
	}
	catch draw_all_buttons
	catch draw_button_outline

	for {set but $numbuts} {$but > 0} {incr but -1} {
		if {[winfo exists .menubuttons.video.f.b$but]} {
			place forget .menubuttons.video.f.b$but
		}
	}
	if {$::buttons::curbutsinfo(ispal)} {
		$w.f2.pos.spy configure -to 574
		$w.f2.pos.sph configure -to 575
	} else {
		$w.f2.pos.spy configure -to 478
		$w.f2.pos.sph configure -to 479
	}
}

proc button_outline {variable value} {
	set ::menued::all_pos_changed true
	if {$variable == "x" && [expr {$::menued::posx + $::menued::posw}] > 719} {
		set ::menued::posw [expr {719 - $::menued::posx}]
		draw_button_outline
		return
	}
	if {$variable == "w" && [expr {$::menued::posx + $::menued::posw}] > 719} {
		set ::menued::posx [expr {719 - $::menued::posw}]
		draw_button_outline
		return
	}
	if {$::buttons::curbutsinfo(ispal)} {
		set maxheight 575
	} else {
		set maxheight 479
	}
	if {$variable == "y" && [expr {$::menued::posy + $::menued::posh}] > $maxheight} {
		set ::menued::posh [expr {$maxheight - $::menued::posy}]
		draw_button_outline
		return
	}
	if {$variable == "h" && [expr {$::menued::posy + $::menued::posh}] > $maxheight} {
		set ::menued::posy [expr {$maxheight - $::menued::posh}]
	}
	draw_button_outline
}

proc draw_button_outline {} {
	set c .menubuttons.video.f
	if {![info exists ::menued::last_outline]} {
		set ::menued::last_outline {0 0 0 0 0}
	}
	foreach i $::menued::last_outline {
		if {$i > 0} {
			$c delete $i
		}
	}
	set ::menued::last_outline {0 0 0 0 0}
	set x1 [expr {round($::menued::posx / $::config(preview_screen_divisor))+2}]
	set y1 [expr {round($::menued::posy / $::menubuttons::buttons_divisor_y)+2}]
	set x2 [expr {round(($::menued::posx + $::menued::posw) / $::config(preview_screen_divisor))+2}]
	set y2 [expr {round(($::menued::posy + $::menued::posh) / $::menubuttons::buttons_divisor_y)+2}]
	lset ::menued::last_outline 0 [$c create rectangle $x1 $y1 $x2 $y2 -outline $::config(menued_outline_color)]
	draw_link down
	draw_link up
	draw_link right
	draw_link left
}
proc draw_link {dir} {
	set but $::menued::button
	set group $::menued::true_group
	set dirname "adj_but_$dir"
	set target [subst \$::menued::$dirname]
	switch $dir {
		up {
			set x1 [expr {$::menued::posx + ($::menued::posw/2)-1}]
			set y1 $::menued::posy
			set item 1
			set color red
			set hoffset -1
			set voffset 0
		}
		down {
			set x1 [expr {$::menued::posx + ($::menued::posw/2)+1}]
			set y1 [expr {$::menued::posy + $::menued::posh}]
			set item 2
			set color green
			set hoffset 1
			set voffset 0
		}
		left {
			set x1 $::menued::posx
			set y1 [expr {$::menued::posy + ($::menued::posh/2)+1}]
			set item 3
			set color #6666FF	;# blue
			set hoffset 0
			set voffset 1
		}
		right {
			set x1 [expr {$::menued::posx + $::menued::posw}]
			set y1 [expr {$::menued::posy + ($::menued::posh/2)-1}]
			set item 4
			set color yellow
			set hoffset 0
			set voffset -1
		}
	}
	set x1 [expr {round($x1 / $::config(preview_screen_divisor))+2}]
	set y1 [expr {round($y1 / $::menubuttons::buttons_divisor_y)+2}]
	if {[catch {set x2 [expr {round(($::menued::but(posx,$group,$target)+($::menued::but(posw,$group,$target)/2)) \
		/$::config(preview_screen_divisor))+2+$hoffset}]}]}  return
	if {[catch {set y2 [expr {round(($::menued::but(posy,$group,$target)+($::menued::but(posh,$group,$target)/2)) \
		/$::menubuttons::buttons_divisor_y)+2+$voffset}]}]}  return
	set line [lindex $::menued::last_outline $item]
	if {$line != 0} {.menubuttons.video.f delete $line}
	if {$::config(menued_show_links) && $target != $but} {
		lset ::menued::last_outline $item \
			[.menubuttons.video.f create line $x1 $y1 $x2 $y2 -fill $color -arrow last -dash {1 1 1}]
	}
	return
}

proc draw_all_buttons {} {
	set c .menubuttons.video.f
	if {[info exists ::menued::last_buttons_outlines]} {
		foreach i $::menued::last_buttons_outlines {
			$c delete $i
		}
		set ::menued::last_buttons_outlines {}
	}

	if {$::config(menued_show_all_buttons)} {
		set g $::menued::true_group
		for {set b 1} {$b <= $::menued::numbuts} {incr b} {
			set x1 [expr {round($::menued::but(posx,$g,$b) / $::config(preview_screen_divisor))+2}]
			set y1 [expr {round($::menued::but(posy,$g,$b) / $::menubuttons::buttons_divisor_y)+2}]
			set x2 [expr {round(($::menued::but(posx,$g,$b) + $::menued::but(posw,$g,$b)) / $::config(preview_screen_divisor))+2}]
			set y2 [expr {round(($::menued::but(posy,$g,$b) + $::menued::but(posh,$g,$b)) / $::menubuttons::buttons_divisor_y)+2}]
			if {$b == $::menued::button} {
				set color $::config(menued_phantom_color)
				set dash .
			} else {
				set color $::config(menued_other_buttons_color)
				set dash ""
			}
			lappend ::menued::last_buttons_outlines [$c create rectangle $x1 $y1 $x2 $y2 -outline $color -dash $dash]
			lappend ::menued::last_buttons_outlines [$c create text [expr {$x1+2}] [expr {$y1+1}] -text $b -anchor nw -fill $color]
		}
	}
}

proc highlight_button {color but} {
	if {$but == 0} return
	if {![winfo ismapped .menued]} return
	if {$but == $::menued::button} {
		set posx $::menued::posx
		set posy $::menued::posy
		set posw $::menued::posw
		set posh $::menued::posh
	} else {
		set posx $::menued::but(posx,$::menued::true_group,$but)
		set posy $::menued::but(posy,$::menued::true_group,$but)
		set posw $::menued::but(posw,$::menued::true_group,$but)
		set posh $::menued::but(posh,$::menued::true_group,$but)
	}
	set x1 [expr {round($posx / $::config(preview_screen_divisor))+2}]
	set y1 [expr {round($posy / $::menubuttons::buttons_divisor_y)+2}]
	set x2 [expr {round(($posx + $posw) / $::config(preview_screen_divisor))+2}]
	set y2 [expr {round(($posy + $posh) / $::menubuttons::buttons_divisor_y)+2}]
	set outline [.menubuttons.video.f create rectangle $x1 $y1 $x2 $y2 -outline $color -dash . -width 3]
	after 500 ".menubuttons.video.f delete $outline"
}

proc highlight_target_button {dir but} {
	if {$but == 0} return
	if {![winfo ismapped .menued]} return

	switch $dir {
		up {set color red}
		down {set color green}
		left {set color #6666FF}
		right {set color yellow}
		default {set color cyan}
	}
	highlight_button $color $but
	if {$dir != "none"} {
		draw_link $dir
	}
}

proc edit_command {} {
	set ::hexcommand [string range $::menued::command 1 23]
	set ::ced_return [::ced::cmdeditor [lindex $::editor_cmds_from 0] \
			[lindex [split [lindex $::editor_cmds_from 1] ,] 0] button]
	grab set .menued
	focus .menued
	if {$::ced_return == 1} {
		set ::menued::command "\[$::hexcommand\]\n[::disasm::disasm $::hexcommand]"
		validate_command $::hexcommand .menued.f2.cmd
		return true
	}
	return false
}

proc new_button {} {
	set newbut [expr {$::menued::numbuts+1}]
	set numgroups $::menued::numgroups
	if {$numgroups == 0} {set numgroups 1}
	set maxbuts [expr {36 / $numgroups}]
	if {$newbut > $maxbuts} {bell; return 0}

	set ::menued::numbuts $newbut
	set w .menued
	init_select_and_action_buttons $w $::menued::numbuts

	if {$newbut > 9} {
		$w.f1.lnsf.sns configure -to 9
	} else {
		$w.f1.lnsf.sns configure -to $newbut
	}
	$w.f2.sel.sbn configure -to $newbut
	$w.f2.ab.sup configure -to $newbut
	$w.f2.ab.slf configure -to $newbut
	$w.f2.ab.srg configure -to $newbut
	$w.f2.ab.sdn configure -to $newbut

	set hex {20 01 00 00 00 00 00 0D}
	set cmd "\[$hex\]\n[::disasm::disasm $hex]"

	for {set g 1} {$g <= $numgroups} {incr g} {
		set ::menued::but(posx,$g,$newbut) 0
		set ::menued::but(posy,$g,$newbut) 0
		set ::menued::but(posw,$g,$newbut) 100
		set ::menued::but(posh,$g,$newbut) 50
		set ::menued::but(adj_but_up,$g,$newbut)    $newbut
		set ::menued::but(adj_but_down,$g,$newbut)  $newbut
		set ::menued::but(adj_but_left,$g,$newbut)  $newbut
		set ::menued::but(adj_but_right,$g,$newbut) $newbut
		set ::menued::but(color_scheme,$g,$newbut)   0
		set ::menued::but(auto_activate,$g,$newbut) 0
		set ::menued::but(command,$g,$newbut) $cmd
	}
	set ::menued::button $newbut
	if {$::menued::numselectablebuts == [expr {$newbut -1}]&& \
			$::menued::numselectablebuts < 9} {
		incr ::menued::numselectablebuts
	}
	button_changed $newbut
	return 1
}

proc delete_button {} {
	set del $::menued::button
	if {$del == 0} {bell; return 0}
	set numgroups $::menued::numgroups
	set numbuts $::menued::numbuts

	if {$numbuts == 1} {
		set txt "This will delete the last button of this cell."
		set txt "$txt\nIf you continue, the editor will be closed,"
		set txt "$txt\nand the changes accepted."
		set txt "$txt\n\nDelete last buttons end exit editor?"
		set r [tk_messageBox -title "Delete Button" -icon question \
			-message $txt -parent .menued -type yesno -default yes]
		if {$r == "no"} {
			return 0
		}
	}

	hide_button orig true
	for {set g 1} {$g <= $numgroups} {incr g} {
		for {set b 1} {$b <= $numbuts} {incr b} {
			if {$::menued::but(adj_but_up,$g,$b) > $del} {
				incr ::menued::but(adj_but_up,$g,$b) -1
			}
			if {$::menued::but(adj_but_down,$g,$b) > $del} {
				incr ::menued::but(adj_but_down,$g,$b) -1
			}
			if {$::menued::but(adj_but_right,$g,$b) > $del} {
				incr ::menued::but(adj_but_right,$g,$b) -1
			}
			if {$::menued::but(adj_but_left,$g,$b) > $del} {
				incr ::menued::but(adj_but_left,$g,$b) -1
			}
		}
		for {set b [expr {$del+1}]} {$b <= $numbuts} {incr b} {
			set target [expr {$b-1}]
			set ::menued::but(posx,$g,$target) $::menued::but(posx,$g,$b)
			set ::menued::but(posy,$g,$target) $::menued::but(posy,$g,$b)
			set ::menued::but(posw,$g,$target) $::menued::but(posw,$g,$b)
			set ::menued::but(posh,$g,$target) $::menued::but(posh,$g,$b)
			set ::menued::but(adj_but_up,$g,$target)    $::menued::but(adj_but_up,$g,$b)
			set ::menued::but(adj_but_down,$g,$target)  $::menued::but(adj_but_down,$g,$b)
			set ::menued::but(adj_but_left,$g,$target)  $::menued::but(adj_but_left,$g,$b)
			set ::menued::but(adj_but_right,$g,$target) $::menued::but(adj_but_right,$g,$b)
			set ::menued::but(color_scheme,$g,$target)   $::menued::but(color_scheme,$g,$b)
			set ::menued::but(auto_activate,$g,$target) $::menued::but(auto_activate,$g,$b)
			set ::menued::but(command,$g,$target)       $::menued::but(command,$g,$b)
		}
		set ::menued::but(posx,$g,$numbuts) 0
		set ::menued::but(posy,$g,$numbuts) 0
		set ::menued::but(posw,$g,$numbuts) 0
		set ::menued::but(posh,$g,$numbuts) 0
		set ::menued::but(adj_but_up,$g,$numbuts)    0
		set ::menued::but(adj_but_down,$g,$numbuts)  0
		set ::menued::but(adj_but_left,$g,$numbuts)  0
		set ::menued::but(adj_but_right,$g,$numbuts) 0
		set ::menued::but(color_scheme,$g,$numbuts)   0
		set ::menued::but(auto_activate,$g,$numbuts) 0
		set ::menued::but(command,$g,$numbuts)       "\[00 00 00 00 00 00 00 00\]\nNOP"
	}
	incr ::menued::numbuts -1
	if {$::menued::numbuts == 0} {
		set ::menued::menued_done 1
		return 1
	}
	init_menued_gui .menued $numgroups $::menued::numbuts
	set newbut $del
	if {$newbut > $::menued::numbuts} {
		incr newbut -1
	}
	if {$::menued::numselectablebuts > $::menued::numbuts} {
		set ::menued::numselectablebuts [expr {$::menued::numbuts}]
	}
	set ::menued::button $newbut
	set ::menued::old_button 0
	button_changed $newbut
	return 1
}

proc hide_button {mode allgroups} {
	set but   $::menued::button
	set group $::menued::true_group

	if {$allgroups || $::menued::gui_group == "All"} {
		set firstgroup 1
		set lastgroup $::menued::numgroups
	} else {
		set firstgroup $group
		set lastgroup  $group
	}
	for {set g $firstgroup} {$g <= $lastgroup} {incr g} {
		switch $mode {
			orig {
				set ::menued::but(auto_activate,$g,$but) 0
			}
			nop {
				set ::menued::but(command,$g,$but) "\[00 00 00 00 00 00 00 00\]\nNOP"
				set ::menued::but(auto_activate,$g,$but) 0
			}
			jump {
				set target [find_non_hidden_button $g $but]
				if {$target == -1} {
					set ::menued::but(command,$g,$but) "\[00 00 00 00 00 00 00 00\]\nNOP"
					set ::menued::but(auto_activate,$g,$but) 0
				} else {
					set hex "20 01 00 00 00 00 [format %02X [expr {$target * 4}]] 01"
					set ::menued::but(command,$g,$but) "\[$hex\]\n[::disasm::disasm $hex]"
					set ::menued::but(auto_activate,$g,$but) 1
				}
			}
		}
		set ::menued::but(posx,$g,$but) 1
		set ::menued::but(posy,$g,$but) 1
		set ::menued::but(posw,$g,$but) 1
		set ::menued::but(posh,$g,$but) 1
	}
	fix_hidden_buttons
	if {$::menued::numselectablebuts >= $but && !$allgroups} {
		set ::menued::numselectablebuts [expr {$but-1}]
	}

	# verify the forced buttons
	if {$::menued::force_select_but == $but || $::menued::force_action_but == $but} {
		set ::config(menued_advanced_controls) true
		advanced_controls
		set txt "Warning:"
		if {$::menued::force_select_but == $but} {
			set txt "$txt\nThe hidden button $but is forced as the default selected button."
		}
		if {$::menued::force_action_but == $but} {
			set txt "$txt\nThe hidden button $but is forced as the default activated button."
		}
		set txt "$txt\n\nPlease select another default button in the advanced controls"
		set txt "$txt\nat the bottom of the menu editor window."
		tk_messageBox -title "Hide Button" -icon warning -message $txt -parent .menued
	}

	# refresh GUI
	set ::menued::posx 1
	set ::menued::posy 1
	set ::menued::posw 1
	set ::menued::posh 1
	set ::menued::adj_but_up    $::menued::but(adj_but_up,$group,$but)
	set ::menued::adj_but_down  $::menued::but(adj_but_down,$group,$but)
	set ::menued::adj_but_left  $::menued::but(adj_but_left,$group,$but)
	set ::menued::adj_but_right $::menued::but(adj_but_right,$group,$but)
	set ::menued::auto_activate $::menued::but(auto_activate,$group,$but)
	set ::menued::command       $::menued::but(command,$group,$but)
	validate_command $::menued::command .menued.f2.cmd
	draw_button_outline
}

proc fix_hidden_buttons {} {
	for {set g 1} {$g <= $::menued::numgroups} {incr g} {
		for {set b 1} {$b <= $::menued::numbuts} {incr b} {
			if {$::menued::but(posx,$g,$b) == 1 && $::menued::but(posy,$g,$b) == 1 && \
					$::menued::but(posw,$g,$b) == 1 && $::menued::but(posh,$g,$b) == 1} {
				set ::menued::ishidden($g,$b) true
			} else {
				set ::menued::ishidden($g,$b) false
			}
		}
	}

	for {set g 1} {$g <= $::menued::numgroups} {incr g} {
		for {set b 1} {$b <= $::menued::numbuts} {incr b} {
			fix_hidden_button_dir $g $b adj_but_up
			fix_hidden_button_dir $g $b adj_but_down
			fix_hidden_button_dir $g $b adj_but_left
			fix_hidden_button_dir $g $b adj_but_right
			if {$::menued::ishidden($g,$b)} {
				if {[string range $::menued::but(command,$g,$b) 1 17] == "20 01 00 00 00 00" && \
						[string range $::menued::but(command,$g,$b) 22 23] == "01"} {
					set target [find_non_hidden_button $g $b]
					if {$target == -1} {
						set ::menued::but(command,$g,$b) "\[00 00 00 00 00 00 00 00\]\nNOP"
						set ::menued::but(auto_activate,$g,$b) 0
					} else {
						set hex "20 01 00 00 00 00 [format %02X [expr {$target * 4}]] 01"
						set ::menued::but(command,$g,$b) "\[$hex\]\n[::disasm::disasm $hex]"
						set ::menued::but(auto_activate,$g,$b) 1
					}
				}
			}
		}
	}
	unset -nocomplain ::menued::ishidden
}
proc fix_hidden_button_dir {g origbutton dir} {
	set but $origbutton
	while 1 {
		set adj $::menued::but($dir,$g,$but)
		if {$adj == $origbutton || $adj < 1 || $adj > $::menued::numbuts || \
					($adj == $but && $::menued::ishidden($g,$but))} {
			set ::menued::but($dir,$g,$origbutton) $origbutton
			if {$adj == $but && $::menued::ishidden($g,$but)} {
				foreach newdir {adj_but_down adj_but_left adj_but_up adj_but_right} {
					set newadj $::menued::but($newdir,$g,$but)
					if {$newadj != $origbutton && $newadj >= 1 && $newadj <= $::menued::numbuts && \
								!$::menued::ishidden($g,$newadj)} {
						set ::menued::but($dir,$g,$origbutton) $newadj
						break
					}
				}
			}
			return
		}
		if {! $::menued::ishidden($g,$adj)} {
			set ::menued::but($dir,$g,$origbutton) $adj
			return
		}
		set but $adj
	}
}

# find a suitable non hidden button
proc find_non_hidden_button {group but} {
	set target -1
	# try to use an adjacent button
	foreach dir {adj_but_right adj_but_down adj_but_left adj_but_up} {
		set b [subst \$::menued::$dir]
		if {$target == -1} {
			if {$b != $but && \
				($::menued::but(posx,$group,$b) != 1 || $::menued::but(posy,$group,$b) != 1 || \
				 $::menued::but(posw,$group,$b) != 1 || $::menued::but(posh,$group,$b) != 1)} {
				set target $b
				break
			}
		}
	}
	# if no adjacent buttons found, then try to use next non-hidden button
	if {$target == -1} {
		for {set b [expr {$but+1}]} {$b <= $::menued::numbuts} {incr b} {
			if {$::menued::but(posx,$group,$b) == 1 && $::menued::but(posy,$group,$b) == 1 && \
					$::menued::but(posw,$group,$b) == 1 && $::menued::but(posh,$group,$b) == 1} {
				continue
			}
			set target $b
			break
		}
	}
	# if no adjacent buttons found, then try to use prev non-hidden button
	if {$target == -1} {
		for {set b [expr {$but-1}]} {$b >= 1} {incr b -1} {
			if {$::menued::but(posx,$group,$b) == 1 && $::menued::but(posy,$group,$b) == 1 && \
					$::menued::but(posw,$group,$b) == 1 && $::menued::but(posh,$group,$b) == 1} {
				continue
			}
			set target $b
			break
		}
	}
	return $target
}

proc menued {} {
	set utiltitle "Menu Editor"
	set w .menued

	### GUI
	if {[winfo exists $w]} {destroy $w}
	toplevel $w -class Dialog -menu $w.mnu
	wm withdraw $w
	wm title $w $utiltitle
	if {$::config(tk) != "aqua"} {wm transient $w .}
	if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
	wm protocol $w WM_DELETE_WINDOW {set ::menued::menued_done 0}
	bind $w <Escape> {set ::menued::menued_done 0}
	bind $w <Return> {set ::menued::menued_done 1}
	bind $w <Up>    {::menued::select_adjacent_button adj_but_up}
	bind $w <Down>  {::menued::select_adjacent_button adj_but_down}
	bind $w <Right> {::menued::select_adjacent_button adj_but_right}
	bind $w <Left>  {::menued::select_adjacent_button adj_but_left}
	#for {set i 1} {$i <= 9} {incr i} {
	#	bind $w $i "::menued::button_changed $i"
	#}

	menu $w.mnu -tearoff $::config(tearoff_menus)

	$w.mnu add cascade -label " File " -menu $w.mnu.file
	menu $w.mnu.file -tearoff $::config(tearoff_menus)
	$w.mnu.file add command -label "Export current button" -command ::menued::export_GUI_values
	$w.mnu.file add command -label "Import current button" -command ::menued::import_GUI_values
	$w.mnu.file add separator
	$w.mnu.file add command -label "Export whole menu" -command ::menued::export_whole_menu
	$w.mnu.file add command -label "Import whole menu" -command ::menued::import_whole_menu
	$w.mnu.file add separator
	$w.mnu.file add command -label "Export color schemes" -command ::menued::export_color_schemes
	$w.mnu.file add command -label "Import color schemes" -command ::menued::import_color_schemes

	$w.mnu add cascade -label " Edit " -menu $w.mnu.edit
	menu $w.mnu.edit -tearoff $::config(tearoff_menus)
	$w.mnu.edit add command -label "Copy current button" -command {::menued::export_GUI_values true}
	$w.mnu.edit add command -label "Copy button position only" -command {::menued::export_GUI_values true true}
	$w.mnu.edit add command -label "Paste current button" -command {::menued::import_GUI_values true}
	$w.mnu.edit add separator
	$w.mnu.edit add command -label "Copy whole menu" -command {::menued::export_whole_menu true}
	$w.mnu.edit add command -label "Paste whole menu" -command {::menued::import_whole_menu true}
	$w.mnu.edit add separator
	$w.mnu.edit add command -label "Copy color schemes" -command {::menued::export_color_schemes true}
	$w.mnu.edit add command -label "Paste color schemes" -command {::menued::import_color_schemes true}
	$w.mnu.edit add separator
	$w.mnu.edit add command -label "Copy command" -command {clipboard clear; clipboard append [string range $::menued::command 1 23]}
	$w.mnu.edit add command -label "Paste command" -command {set c [::ced::extract_hexcommand [clipboard get]]; if {$c == ""} bell else {set ::menued::command "\[$c\]\n[::disasm::disasm $c]; ::menued::validate_command $c .menued.f2.cmd"}}
	$w.mnu.edit add command -label "Copy as text" -command {clipboard clear; clipboard append [string map {\n { }} $::menued::command]}

	$w.mnu add cascade -label " Button " -menu $w.mnu.but
	menu $w.mnu.but -tearoff $::config(tearoff_menus)
	$w.mnu.but add command -label "Undo last changes" -command ::menued::undo_button_changes
	$w.mnu.but add separator
	$w.mnu.but add command -label "New button" -command ::menued::new_button
	$w.mnu.but add command -label "Delete button" -command ::menued::delete_button
	$w.mnu.but add separator
	$w.mnu.but add command -label "Increase button number" -command {::menued::renumber_button 1}
	$w.mnu.but add command -label "Decrease button number" -command {::menued::renumber_button -1}
	$w.mnu.but add separator
	$w.mnu.but add command -label "Align on same column" -command "::menued::allign_button column"
	$w.mnu.but add command -label "Align on same row" -command "::menued::allign_button row"

	$w.mnu add cascade -label " Hide Button " -menu $w.mnu.hide
		menu $w.mnu.hide -tearoff $::config(tearoff_menus)
		$w.mnu.hide add command -label "Jump to non-deleted button" -command "::menued::hide_button jump false"
		$w.mnu.hide add command -label "Replace command with NOP" -command "::menued::hide_button nop false"
		$w.mnu.hide add command -label "Leave original command" -command "::menued::hide_button orig false"
		$w.mnu.hide add separator
		$w.mnu.hide add command -label "Help!" -command "::menued::hide_button_help"

	$w.mnu add cascade -label " View " -menu $w.mnu.view
	menu $w.mnu.view -tearoff $::config(tearoff_menus)
	$w.mnu.view add command -label "Color schemes Editor" -command ::menued::colorschemesed
	$w.mnu.view add separator
	$w.mnu.view add checkbutton -label "Show Advanced Controls" -variable ::config(menued_advanced_controls) \
			-command ::menued::advanced_controls -onvalue true -offvalue false
	$w.mnu.view add separator
	$w.mnu.view add checkbutton -label "Show all buttons" -variable ::config(menued_show_all_buttons) \
			-command ::menued::draw_all_buttons -onvalue true -offvalue false
	$w.mnu.view add checkbutton -label "Show adjacent buttons links" -variable ::config(menued_show_links) \
			-command ::menued::draw_button_outline -onvalue true -offvalue false
	$w.mnu.view add separator
	$w.mnu.view add command -label "Current button edit color" \
			-command {::menued::choose_outline_color menued_outline_color}
	$w.mnu.view add command -label "Current button phantom color" \
			-command {::menued::choose_outline_color menued_phantom_color}
	$w.mnu.view add command -label "Other buttons color" \
			-command {::menued::choose_outline_color menued_other_buttons_color}

	# top frame: button editor
	frame $w.f2 -bd 2 -relief groove
	pack  $w.f2 -padx 4 -pady 2 -fill x
	frame $w.f2.sel
	pack  $w.f2.sel -fill x

	label $w.f2.sel.lbn -text " Edit button"
	pack  $w.f2.sel.lbn -side left -pady 6
	label $w.f2.sel.nbn -textvariable ::menued::button -width 3
	pack  $w.f2.sel.nbn -side left
	scale $w.f2.sel.sbn -from 1 -to 36 -variable ::menued::button -showvalue false \
			-command ::menued::button_changed
	pack  $w.f2.sel.sbn -side left -fill x -expand 1

	label $w.f2.sel.lgn -text "   Group"
	pack  $w.f2.sel.lgn -side left
	radiobutton $w.f2.sel.rgn1 -variable ::menued::gui_group -value 1 -text "1 " \
			-command "::menued::group_changed 1"
	pack  $w.f2.sel.rgn1 -side left -padx 4
	radiobutton $w.f2.sel.rgn2 -variable ::menued::gui_group -value 2 -text "2 " \
			-command "::menued::group_changed 2"
	pack  $w.f2.sel.rgn2 -side left -padx 4
	radiobutton $w.f2.sel.rgn3 -variable ::menued::gui_group -value 3 -text "3 " \
			-command "::menued::group_changed 3"
	pack  $w.f2.sel.rgn3 -side left -padx 4
	radiobutton $w.f2.sel.rgn4 -variable ::menued::gui_group -value All -text All \
			-command "::menued::group_changed All"
	pack  $w.f2.sel.rgn4 -side left -padx 4

	frame $w.f2.pos -bd 2 -relief groove
	pack  $w.f2.pos -padx 2 -fill x -pady 2

	label $w.f2.pos.lpx -text "X position:"
	grid  $w.f2.pos.lpx -row 0 -column 0 -sticky w
	label $w.f2.pos.npx -textvariable ::menued::posx -width 3
	grid  $w.f2.pos.npx -row 0 -column 1
	scale $w.f2.pos.spx -from 0 -to 718 -variable ::menued::posx -showvalue false \
			-command "::menued::button_outline x"
	grid  $w.f2.pos.spx -row 0 -column 2 -sticky ew

	label $w.f2.pos.lpw -text "Width:"
	grid  $w.f2.pos.lpw -row 1 -column 0 -sticky w
	label $w.f2.pos.npw -textvariable ::menued::posw -width 3
	grid  $w.f2.pos.npw -row 1 -column 1
	scale $w.f2.pos.spw -from 1 -to 719 -variable ::menued::posw -showvalue false \
			-command "::menued::button_outline w"
	grid  $w.f2.pos.spw -row 1 -column 2 -sticky ew

	label $w.f2.pos.lpy -text "Y position:"
	grid  $w.f2.pos.lpy -row 2 -column 0 -sticky w
	label $w.f2.pos.npy -textvariable ::menued::posy -width 3
	grid  $w.f2.pos.npy -row 2 -column 1
	scale $w.f2.pos.spy -from 0 -to 574 -variable ::menued::posy -showvalue false \
			-command "::menued::button_outline y"
	grid  $w.f2.pos.spy -row 2 -column 2 -sticky ew

	label $w.f2.pos.lph -text "Height:"
	grid  $w.f2.pos.lph -row 3 -column 0 -sticky w
	label $w.f2.pos.nph -textvariable ::menued::posh -width 3
	grid  $w.f2.pos.nph -row 3 -column 1
	scale $w.f2.pos.sph -from 1 -to 575 -variable ::menued::posh -showvalue false \
			-command "::menued::button_outline h"
	grid  $w.f2.pos.sph -row 3 -column 2 -sticky ew

	grid columnconfigure $w.f2.pos 2 -weight 1

	frame $w.f2.ab -bd 2 -relief groove
	pack  $w.f2.ab -padx 2 -fill x

	label $w.f2.ab.lab -text "Adjacent buttons :"
	scale $w.f2.ab.sup -from 1 -to 36 -variable ::menued::adj_but_up -showvalue false \
			-command {::menued::highlight_target_button up}
	if {$::config(tk) == "aqua"} {
		label $w.f2.ab.lup -textvariable ::menued::adj_but_up -width 2 -bd 1 -relief raised
		bind $w.f2.ab.lup <1> {::menued::highlight_target_button up $::menued::adj_but_up}
	} else {
		button $w.f2.ab.lup -textvariable ::menued::adj_but_up -width 2 \
				-command {::menued::highlight_target_button up $::menued::adj_but_up}
	}
	scale $w.f2.ab.slf -from 1 -to 36 -variable ::menued::adj_but_left -showvalue false \
			-command {::menued::highlight_target_button left}

	frame $w.f2.ab.frl
	if {$::config(tk) == "aqua"} {
		label $w.f2.ab.frl.llf -textvariable ::menued::adj_but_left -width 2 -bd 1 -relief raised
		bind  $w.f2.ab.frl.llf <1> {::menued::highlight_target_button left $::menued::adj_but_left}
		label $w.f2.ab.frl.lbb -textvariable ::menued::button -width 2 -bd 1 -relief raised
		bind  $w.f2.ab.frl.lbb <1> {	::menued::highlight_target_button up $::menued::adj_but_up; \
					::menued::highlight_target_button down $::menued::adj_but_down; \
					::menued::highlight_target_button right $::menued::adj_but_right; \
					::menued::highlight_target_button left $::menued::adj_but_left}
		label $w.f2.ab.frl.lrg -textvariable ::menued::adj_but_right -width 2 -bd 1 -relief raised
		bind  $w.f2.ab.frl.lrg <1> {::menued::highlight_target_button right $::menued::adj_but_right}
	} else {
		button $w.f2.ab.frl.llf -textvariable ::menued::adj_but_left -width 2 \
				-command {::menued::highlight_target_button left $::menued::adj_but_left}
		button $w.f2.ab.frl.lbb -textvariable ::menued::button -width 2 \
				-command {	::menued::highlight_target_button up $::menued::adj_but_up; \
							::menued::highlight_target_button down $::menued::adj_but_down; \
							::menued::highlight_target_button right $::menued::adj_but_right; \
							::menued::highlight_target_button left $::menued::adj_but_left}
		button $w.f2.ab.frl.lrg -textvariable ::menued::adj_but_right -width 2 \
				-command {::menued::highlight_target_button right $::menued::adj_but_right}
	}
	pack  $w.f2.ab.frl.llf -side left
	pack  $w.f2.ab.frl.lbb -side left -fill x -padx 2
	pack  $w.f2.ab.frl.lrg -side left

	scale $w.f2.ab.srg -from 1 -to 36 -variable ::menued::adj_but_right -showvalue false \
			-command {::menued::highlight_target_button right}
	if {$::config(tk) == "aqua"} {
		label $w.f2.ab.ldn -textvariable ::menued::adj_but_down -width 2 -bd 1 -relief raised
		bind  $w.f2.ab.ldn <1> {::menued::highlight_target_button down $::menued::adj_but_down}
	} else {
		button $w.f2.ab.ldn -textvariable ::menued::adj_but_down -width 2 \
				-command {::menued::highlight_target_button down $::menued::adj_but_down}
	}
	scale $w.f2.ab.sdn -from 1 -to 36 -variable ::menued::adj_but_down -showvalue false \
			-command {::menued::highlight_target_button down}
	$w.f2.ab.lup configure -bd 1 -padx 0 -pady 0
	$w.f2.ab.frl.llf configure -bd 1 -padx 0 -pady 0
	$w.f2.ab.frl.lbb configure -bd 1 -padx 0 -pady 0
	$w.f2.ab.frl.lrg configure -bd 1 -padx 0 -pady 0
	$w.f2.ab.ldn configure -bd 1 -padx 0 -pady 0

	grid  $w.f2.ab.lab -row 0 -column 0 -sticky w -columnspan 2
	grid  $w.f2.ab.sup -row 0 -column 2 -columnspan 3 -sticky ew
	grid  $w.f2.ab.lup -row 1 -column 3
	grid  $w.f2.ab.slf -row 2 -column 0 -columnspan 3 -sticky ew
	grid  $w.f2.ab.frl -row 2 -column 3
	grid  $w.f2.ab.srg -row 2 -column 4 -columnspan 3 -sticky ew
	grid  $w.f2.ab.ldn -row 3 -column 3
	grid  $w.f2.ab.sdn -row 4 -column 2 -columnspan 3 -sticky ew

	grid  columnconfigure $w.f2.ab 0 -uniform 1 -weight 1
	grid  columnconfigure $w.f2.ab 1 -uniform 1 -weight 1
	grid  columnconfigure $w.f2.ab 2 -uniform 1 -weight 1
	grid  columnconfigure $w.f2.ab 3 -uniform 1 -weight 1
	grid  columnconfigure $w.f2.ab 4 -uniform 1 -weight 1
	grid  columnconfigure $w.f2.ab 5 -uniform 1 -weight 1
	grid  columnconfigure $w.f2.ab 6 -uniform 1 -weight 1

	frame $w.f2.csf
	pack  $w.f2.csf -anchor w -padx 2 -pady 2 -fill x
	frame $w.f2.csf.f -bd 2 -relief groove
	pack  $w.f2.csf.f -side left -fill x -expand 1
	label $w.f2.csf.f.lab -text "Use color scheme"
	pack  $w.f2.csf.f.lab -side left
	radiobutton $w.f2.csf.f.cs0 -variable ::menued::color_scheme -value 0 -text "0 (default colors) "
	pack  $w.f2.csf.f.cs0 -side left -fill x -expand 1
	radiobutton $w.f2.csf.f.cs1 -variable ::menued::color_scheme -value 1 -text "1 "
	pack  $w.f2.csf.f.cs1 -side left -fill x -expand 1
	radiobutton $w.f2.csf.f.cs2 -variable ::menued::color_scheme -value 2 -text "2 "
	pack  $w.f2.csf.f.cs2 -side left -fill x -expand 1
	radiobutton $w.f2.csf.f.cs3 -variable ::menued::color_scheme -value 3 -text 3
	pack  $w.f2.csf.f.cs3 -side left -fill x -expand 1
	button $w.f2.csf.ecs -text "Edit Color Schemes" -command ::menued::colorschemesed
	if {$::config(tk) == "aqua"} {$w.f2.csf.ecs configure -text "Edit"}
	pack  $w.f2.csf.ecs -side right -padx 2 -fill both -expand 1

	if {$::config(tk) == "aqua"} {
		label $w.f2.cmd -bd 2 -relief raised -textvariable ::menued::command -height 2
		bind  $w.f2.cmd <1> ::menued::edit_command
	} else {
		button $w.f2.cmd -textvariable ::menued::command -height 2 \
				-command ::menued::edit_command
	}
	pack  $w.f2.cmd -anchor w -padx 4 -pady 2 -fill x

	checkbutton $w.f2.caa -variable ::menued::auto_activate -text "Auto Activate  (Execute the command directly when the button is highlighted)"
	pack  $w.f2.caa -anchor w

	# bottom frame: general, advanced controls params
	labelframe $w.f1 -bd 2 -relief groove -text "Advanced, global controls"
	if {$::config(menued_advanced_controls)} {
		pack  $w.f1 -padx 4 -pady 2 -fill x
	}

	frame $w.f1.lnsf -bd 2 -relief groove
	pack  $w.f1.lnsf -fill x -expand 1 -padx 2 -pady 2
	label $w.f1.lnsf.lns -text "Number of numerically selectable buttons  (Max 9) :"
	pack  $w.f1.lnsf.lns -side left
	label $w.f1.lnsf.nns -textvariable ::menued::numselectablebuts -width 2
	pack  $w.f1.lnsf.nns -side left
	scale $w.f1.lnsf.sns -from 0 -to 9 -variable ::menued::numselectablebuts -showvalue false
	pack  $w.f1.lnsf.sns -side left -fill x -expand 1 -padx 1 -pady 1


	frame $w.f1.fs -bd 2 -relief groove
	pack  $w.f1.fs -fill x -expand 1 -padx 2
	label $w.f1.fs.lsb -text "When the buttons are displayed, force the selection of"
	pack  $w.f1.fs.lsb -side left
	menubutton $w.f1.fs.ssb -menu $w.f1.fs.ssb.m -textvariable ::menued::force_select_but_text -height 1 -pady 2
	pack  $w.f1.fs.ssb -side left -fill x -expand 1 -padx 2


	frame $w.f1.fa -bd 2 -relief groove
	pack  $w.f1.fa -fill x -expand 1 -padx 2 -pady 2
	frame $w.f1.fa.f1
	pack  $w.f1.fa.f1 -fill x -expand 1
	label $w.f1.fa.f1.lab -text "In non-still menu only, force the auto-activation of"
	pack  $w.f1.fa.f1.lab -side left
	menubutton $w.f1.fa.f1.sab -menu $w.f1.fa.f1.sab.m -textvariable ::menued::force_action_but_text -height 1 -pady 2
	pack  $w.f1.fa.f1.sab -side left -fill x -expand 1 -padx 2

	frame  $w.f1.fa.f2
	pack   $w.f1.fa.f2 -fill x -expand 1
	label  $w.f1.fa.f2.selec -text "after"
	pack   $w.f1.fa.f2.selec -side left
	entry  $w.f1.fa.f2.selev -width 7 -textvariable ::menued::selendtime \
			-validate all -validatecommand "::menued::validate_selendtime $w {%d} {%P}"
	pack   $w.f1.fa.f2.selev -side left
	label  $w.f1.fa.f2.seles -text "seconds."
	pack   $w.f1.fa.f2.seles -side left
	if {$::menued::maxselendtimesecs == -1} {
		label  $w.f1.fa.f2.msg -bd 2 -padx 2 -text "( Unavailable in still menus!  Should be \"no button\" and -1. )"
	} else {
		label  $w.f1.fa.f2.msg -bd 2 -padx 2 -text "( MAX $::menued::maxselendtimesecs seconds!  -1 = do not force action button )"
	}
	pack   $w.f1.fa.f2.msg -side left -padx 2

    warning_action_but_in_still $w


	# OK/Cancel buttons
	frame  $w.fb
	pack   $w.fb -fill x -padx 2 -pady 4
	checkbutton $w.fb.ac -text "Advanced" -variable ::config(menued_advanced_controls) \
			-command ::menued::advanced_controls -onvalue true -offvalue false
	pack   $w.fb.ac -side left -padx 2
	checkbutton $w.fb.dl -text "Show adjacent buttons" -variable ::config(menued_show_links) \
			-command ::menued::draw_button_outline -onvalue true -offvalue false
	pack   $w.fb.dl -side left -padx 2
	button $w.fb.cancel -text "Cancel" -width 13 -command {set ::menued::menued_done 0}
	pack   $w.fb.cancel -side right -padx 2
	button $w.fb.ok -text "OK" -width 13 -command {set ::menued::menued_done 1}
	pack   $w.fb.ok -side right -padx 2

	init_menued_gui $w $::menued::numgroups $::menued::numbuts

	update
	wm geometry $w $::config(menueditor_win_geometry)
	set h [winfo reqheight $w]
	if {$::config(tk) == "x11"} {incr h 32}
	wm minsize $w [winfo reqwidth $w] $h
	wm maxsize $w [winfo reqwidth $w] $h
	focus $w
	grab set $w
	wm deiconify $w
}

proc warning_action_but_in_still {w {t ""}} {
	if {[winfo exists $w.f1.fa.f2.msg]} {
		if {$t == ""} {set t $::menued::selendtime}
		if {$::menued::maxselendtimesecs == -1 && \
				($::menued::force_action_but != 0 || $t != -1)} {
		    $w.f1.fa.f2.msg configure -background pink -relief groove
		} else {
		    $w.f1.fa.f2.msg configure -background $::config(systembuttonface) -relief flat
		}
	}
	return
}

proc validate_selendtime {w d p args} {
    set rc [::ced::value_entry_validate -1.0 3600.0 $d $p $args]
	warning_action_but_in_still $w $p
	return $rc
}

proc choose_outline_color {type} {
	set c [tk_chooseColor -initialcolor $::config($type) -parent .menued -title "Choose color"]
	if {$c != ""} {
		set ::config($type) $c
		::menued::draw_all_buttons
		::menued::draw_button_outline
		return $c
	}
	return $::config($type)
}

proc advanced_controls {} {
	set w .menued
	if {$::config(menued_advanced_controls)} {
		pack $w.f1 -padx 4 -pady 4 -fill x -after $w.f2
	} else {
		pack forget $w.f1
	}
	update
	set h [winfo reqheight $w]
	if {$::config(tk) == "x11"} {incr h 32}
	wm minsize $w [winfo reqwidth $w] $h
	wm maxsize $w [winfo reqwidth $w] $h
	wm geometry $w [winfo reqwidth $w]x$h
	#wm withdraw $w
	#wm deiconify $w
	update idletasks
}

proc colorschemesed {} {
	if {! [init_colors]} {
		return false
	}

	set ::menued::colshm_done 1
	set utiltitle "Color schemes Editor"
	set w .colshmed

	# retrieve the PGC color table
	if {[lindex $::editor_cmds_from 0] == "menu"} {
		set pgcheader $::menus([lindex $::editor_cmds_from 1],header)
	} else {
		set pgcheader $::pgcs([lindex $::editor_cmds_from 1],header)
	}
	for {set c 0} {$c < 16} {incr c} {
		set idx [expr {(0xA4 + ($c*4))}]
		set ::menued::rgbcolor($c) [::pgced::yc2rgb [string range $pgcheader $idx [expr {$idx + 3}]]]
	}
	unset pgcheader

	### GUI
	if {[winfo exists $w]} {destroy $w}
	toplevel $w -class Dialog
	wm withdraw $w
	wm title $w $utiltitle
	if {$::config(tk) != "aqua"} {wm transient $w .}
	if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
	wm protocol $w WM_DELETE_WINDOW {set ::menued::colshm_done 0}
	bind $w <Escape> {set ::menued::colshm_done 0}
	bind $w <Return> {set ::menued::colshm_done 1}

	set color_height [expr [font metrics $::config(global_font) -ascent] + 4]
	if {[expr $color_height % 2] == 1} {incr color_height}
	set color_half [expr $color_height / 2]

	# color schemes
	frame $w.f1
	pack  $w.f1 -padx 4 -pady 8 -fill x

	frame $w.f1.sel
	pack  $w.f1.sel -fill x
	label $w.f1.sel.lcs -text "Edit color scheme"
	pack  $w.f1.sel.lcs -side left
	radiobutton $w.f1.sel.cs1 -variable ::menued::edit_color_scheme -value 1 -text "1  " \
			-command {::menued::change_color_scheme 1}
	pack  $w.f1.sel.cs1 -side left
	radiobutton $w.f1.sel.cs2 -variable ::menued::edit_color_scheme -value 2 -text "2  " \
			-command {::menued::change_color_scheme 2}
	pack  $w.f1.sel.cs2 -side left
	radiobutton $w.f1.sel.cs3 -variable ::menued::edit_color_scheme -value 3 -text "3" \
			-command {::menued::change_color_scheme 3}
	pack  $w.f1.sel.cs3 -side left

	# color schemes
	frame $w.f3 -bd 2 -relief groove
	pack  $w.f3 -padx 4 -fill x

	label $w.f3.lt11 -text "Color for Selected button:"
	pack  $w.f3.lt11 -pady 0 -anchor w

	frame $w.f3.f
	pack  $w.f3.f -fill x

	label $w.f3.f.lt12 -text "Color index in PGC lookup table" -width 25 -anchor w
	grid  $w.f3.f.lt12 -row 1 -column 1 -sticky w -columnspan 3
	label $w.f3.f.lt13 -text "   "
	grid  $w.f3.f.lt13 -row 1 -column 4
	label $w.f3.f.lt14 -text "Contrast (Opacity)" -width 25 -anchor w
	grid  $w.f3.f.lt14 -row 1 -column 5 -sticky w -columnspan 2

	label $w.f3.f.lb1 -text "Background"
	grid  $w.f3.f.lb1 -row 2 -column 0 -sticky w
	scale $w.f3.f.sbc1 -from 0 -to 15 -variable ::menued::sel_color_3 -showvalue false \
			-command "::menued::display_pgc_color $w.f3.f.bbc1 sel_opacity_3"
	grid  $w.f3.f.sbc1 -row 2 -column 1 -sticky ew
	label $w.f3.f.lbc1 -width 2 -textvariable ::menued::sel_color_3
	grid  $w.f3.f.lbc1 -row 2 -column 2 -sticky e
	frame $w.f3.f.bbc1  -bd 1 -relief sunken
	grid  $w.f3.f.bbc1 -row 2 -column 3 -sticky ew
	frame $w.f3.f.bbc1.b -pady 0 -width $color_height -height $color_height
	pack  $w.f3.f.bbc1.b
	scale $w.f3.f.sbo1 -from 0 -to 15 -variable ::menued::sel_opacity_3 -showvalue false \
			-command "::menued::display_opacity $w.f3.f.bbc1o sel_color_3"
	grid  $w.f3.f.sbo1 -row 2 -column 5 -sticky ew
	label $w.f3.f.lbo1 -width 2 -textvariable ::menued::sel_opacity_3
	grid  $w.f3.f.lbo1 -row 2 -column 6 -sticky e
	frame $w.f3.f.bbc1o -bd 1 -relief sunken
	grid  $w.f3.f.bbc1o -row 2 -column 7 -sticky ew
	frame $w.f3.f.bbc1o.b1 -pady 0 -width $color_half -height $color_height
	pack  $w.f3.f.bbc1o.b1 -side left
	frame $w.f3.f.bbc1o.b2 -pady 0 -width $color_half -height $color_height
	pack  $w.f3.f.bbc1o.b2 -side left

	label $w.f3.f.lp1 -text "Pattern"
	grid  $w.f3.f.lp1 -row 3 -column 0 -sticky w
	scale $w.f3.f.spc1 -from 0 -to 15 -variable ::menued::sel_color_2 -showvalue false \
			-command "::menued::display_pgc_color $w.f3.f.bpc1 sel_opacity_2"
	grid  $w.f3.f.spc1 -row 3 -column 1 -sticky ew
	label $w.f3.f.lpc1 -width 2 -textvariable ::menued::sel_color_2
	grid  $w.f3.f.lpc1 -row 3 -column 2 -sticky e
	frame $w.f3.f.bpc1  -bd 1 -relief sunken
	grid  $w.f3.f.bpc1 -row 3 -column 3 -sticky ew
	frame $w.f3.f.bpc1.b -pady 0 -width $color_height -height $color_height
	pack  $w.f3.f.bpc1.b
	scale $w.f3.f.spo1 -from 0 -to 15 -variable ::menued::sel_opacity_2 -showvalue false \
			-command "::menued::display_opacity $w.f3.f.bpc1o sel_color_2"
	grid  $w.f3.f.spo1 -row 3 -column 5 -sticky ew
	label $w.f3.f.lpo1 -width 2 -textvariable ::menued::sel_opacity_2
	grid  $w.f3.f.lpo1 -row 3 -column 6 -sticky e
	frame $w.f3.f.bpc1o -bd 1 -relief sunken
	grid  $w.f3.f.bpc1o -row 3 -column 7 -sticky ew
	frame $w.f3.f.bpc1o.b1 -pady 0 -width $color_half -height $color_height
	pack  $w.f3.f.bpc1o.b1 -side left
	frame $w.f3.f.bpc1o.b2 -pady 0 -width $color_half -height $color_height
	pack  $w.f3.f.bpc1o.b2 -side left

	label $w.f3.f.l11 -text "Emphasis 1"
	grid  $w.f3.f.l11 -row 4 -column 0 -sticky w
	scale $w.f3.f.s1c1 -from 0 -to 15 -variable ::menued::sel_color_1 -showvalue false \
			-command "::menued::display_pgc_color $w.f3.f.b1c1 sel_opacity_1"
	grid  $w.f3.f.s1c1 -row 4 -column 1 -sticky ew
	label $w.f3.f.l1c1 -width 2 -textvariable ::menued::sel_color_1
	grid  $w.f3.f.l1c1 -row 4 -column 2 -sticky e
	frame $w.f3.f.b1c1  -bd 1 -relief sunken
	grid  $w.f3.f.b1c1 -row 4 -column 3 -sticky ew
	frame $w.f3.f.b1c1.b -pady 0 -width $color_height -height $color_height
	pack  $w.f3.f.b1c1.b
	scale $w.f3.f.s1o1 -from 0 -to 15 -variable ::menued::sel_opacity_1 -showvalue false \
			-command "::menued::display_opacity $w.f3.f.b1c1o sel_color_1"
	grid  $w.f3.f.s1o1 -row 4 -column 5 -sticky ew
	label $w.f3.f.l1o1 -width 2 -textvariable ::menued::sel_opacity_1
	grid  $w.f3.f.l1o1 -row 4 -column 6 -sticky e
	frame $w.f3.f.b1c1o -bd 1 -relief sunken
	grid  $w.f3.f.b1c1o -row 4 -column 7 -sticky ew
	frame $w.f3.f.b1c1o.b1 -pady 0 -width $color_half -height $color_height
	pack  $w.f3.f.b1c1o.b1 -side left
	frame $w.f3.f.b1c1o.b2 -pady 0 -width $color_half -height $color_height
	pack  $w.f3.f.b1c1o.b2 -side left

	label $w.f3.f.l21 -text "Emphasis 2"
	grid  $w.f3.f.l21 -row 5 -column 0 -sticky w
	scale $w.f3.f.s2c1 -from 0 -to 15 -variable ::menued::sel_color_0 -showvalue false \
			-command "::menued::display_pgc_color $w.f3.f.b2c1 sel_opacity_0"
	grid  $w.f3.f.s2c1 -row 5 -column 1 -sticky ew
	label $w.f3.f.l2c1 -width 2 -textvariable ::menued::sel_color_0
	grid  $w.f3.f.l2c1 -row 5 -column 2 -sticky e
	frame $w.f3.f.b2c1  -bd 1 -relief sunken
	grid  $w.f3.f.b2c1 -row 5 -column 3 -sticky ew
	frame $w.f3.f.b2c1.b -pady 0 -width $color_height -height $color_height
	pack  $w.f3.f.b2c1.b
	scale $w.f3.f.s2o1 -from 0 -to 15 -variable ::menued::sel_opacity_0 -showvalue false \
			-command "::menued::display_opacity $w.f3.f.b2c1o sel_color_0"
	grid  $w.f3.f.s2o1 -row 5 -column 5 -sticky ew
	label $w.f3.f.l2o1 -width 2 -textvariable ::menued::sel_opacity_0
	grid  $w.f3.f.l2o1 -row 5 -column 6 -sticky e
	frame $w.f3.f.b2c1o -bd 1 -relief sunken
	grid  $w.f3.f.b2c1o -row 5 -column 7 -sticky ew
	frame $w.f3.f.b2c1o.b1 -pady 0 -width $color_half -height $color_height
	pack  $w.f3.f.b2c1o.b1 -side left
	frame $w.f3.f.b2c1o.b2 -pady 0 -width $color_half -height $color_height
	pack  $w.f3.f.b2c1o.b2 -side left

	frame $w.f3.f.pad -width 4
	grid  $w.f3.f.pad -row 0 -column 8

	grid  columnconfigure $w.f3.f 1 -weight 1 -uniform 1
	grid  columnconfigure $w.f3.f 5 -weight 1 -uniform 1


	frame $w.f4 -bd 2 -relief groove
	pack  $w.f4 -padx 4 -pady 4 -fill x

	label $w.f4.lt11 -text "Color for Activated button:"
	pack  $w.f4.lt11 -pady 0 -anchor w

	frame $w.f4.f
	pack  $w.f4.f -fill x

	label $w.f4.f.lt12 -text "Color index in PGC lookup table" -width 25 -anchor w
	grid  $w.f4.f.lt12 -row 6 -column 1 -sticky w -columnspan 3
	label $w.f4.f.lt13 -text "   "
	grid  $w.f4.f.lt13 -row 6 -column 4
	label $w.f4.f.lt14 -text "Contrast (Opacity)" -width 25 -anchor w
	grid  $w.f4.f.lt14 -row 6 -column 5 -sticky w -columnspan 2

	label $w.f4.f.lb2 -text "Background"
	grid  $w.f4.f.lb2 -row 7 -column 0 -sticky w
	scale $w.f4.f.sbc2 -from 0 -to 15 -variable ::menued::act_color_3 -showvalue false \
			-command "::menued::display_pgc_color $w.f4.f.bbc2 act_opacity_3"
	grid  $w.f4.f.sbc2 -row 7 -column 1 -sticky ew
	label $w.f4.f.lbc2 -width 2 -textvariable ::menued::act_color_3
	grid  $w.f4.f.lbc2 -row 7 -column 2 -sticky e
	frame $w.f4.f.bbc2  -bd 1 -relief sunken
	grid  $w.f4.f.bbc2 -row 7 -column 3 -sticky ew
	frame $w.f4.f.bbc2.b -pady 0 -width $color_height -height $color_height
	pack  $w.f4.f.bbc2.b
	scale $w.f4.f.sbo2 -from 0 -to 15 -variable ::menued::act_opacity_3 -showvalue false \
			-command "::menued::display_opacity $w.f4.f.bbc2o act_color_3"
	grid  $w.f4.f.sbo2 -row 7 -column 5 -sticky ew
	label $w.f4.f.lbo2 -width 2 -textvariable ::menued::act_opacity_3
	grid  $w.f4.f.lbo2 -row 7 -column 6 -sticky e
	frame $w.f4.f.bbc2o -bd 1 -relief sunken
	grid  $w.f4.f.bbc2o -row 7 -column 7 -sticky ew
	frame $w.f4.f.bbc2o.b1 -pady 0 -width $color_half -height $color_height
	pack  $w.f4.f.bbc2o.b1 -side left
	frame $w.f4.f.bbc2o.b2 -pady 0 -width $color_half -height $color_height
	pack  $w.f4.f.bbc2o.b2 -side left

	label $w.f4.f.lp2 -text "Pattern"
	grid  $w.f4.f.lp2 -row 8 -column 0 -sticky w
	scale $w.f4.f.spc2 -from 0 -to 15 -variable ::menued::act_color_2 -showvalue false \
			-command "::menued::display_pgc_color $w.f4.f.bpc2 act_opacity_2"
	grid  $w.f4.f.spc2 -row 8 -column 1 -sticky ew
	label $w.f4.f.lpc2 -width 2 -textvariable ::menued::act_color_2
	grid  $w.f4.f.lpc2 -row 8 -column 2 -sticky e
	frame $w.f4.f.bpc2  -bd 1 -relief sunken
	grid  $w.f4.f.bpc2 -row 8 -column 3 -sticky ew
	frame $w.f4.f.bpc2.b -pady 0 -width $color_height -height $color_height
	pack  $w.f4.f.bpc2.b
	scale $w.f4.f.spo2 -from 0 -to 15 -variable ::menued::act_opacity_2 -showvalue false \
			-command "::menued::display_opacity $w.f4.f.bpc2o act_color_2"
	grid  $w.f4.f.spo2 -row 8 -column 5 -sticky ew
	label $w.f4.f.lpo2 -width 2 -textvariable ::menued::act_opacity_2
	grid  $w.f4.f.lpo2 -row 8 -column 6 -sticky e
	frame $w.f4.f.bpc2o -bd 1 -relief sunken
	grid  $w.f4.f.bpc2o -row 8 -column 7 -sticky ew
	frame $w.f4.f.bpc2o.b1 -pady 0 -width $color_half -height $color_height
	pack  $w.f4.f.bpc2o.b1 -side left
	frame $w.f4.f.bpc2o.b2 -pady 0 -width $color_half -height $color_height
	pack  $w.f4.f.bpc2o.b2 -side left

	label $w.f4.f.l12 -text "Emphasis 1"
	grid  $w.f4.f.l12 -row 9 -column 0 -sticky w
	scale $w.f4.f.s1c2 -from 0 -to 15 -variable ::menued::act_color_1 -showvalue false \
			-command "::menued::display_pgc_color $w.f4.f.b1c2 act_opacity_1"
	grid  $w.f4.f.s1c2 -row 9 -column 1 -sticky ew
	label $w.f4.f.l1c2 -width 2 -textvariable ::menued::act_color_1
	grid  $w.f4.f.l1c2 -row 9 -column 2 -sticky e
	frame $w.f4.f.b1c2  -bd 1 -relief sunken
	grid  $w.f4.f.b1c2 -row 9 -column 3 -sticky ew
	frame $w.f4.f.b1c2.b -pady 0 -width $color_height -height $color_height
	pack  $w.f4.f.b1c2.b
	scale $w.f4.f.s1o2 -from 0 -to 15 -variable ::menued::act_opacity_1 -showvalue false \
			-command "::menued::display_opacity $w.f4.f.b1c2o act_color_1"
	grid  $w.f4.f.s1o2 -row 9 -column 5 -sticky ew
	label $w.f4.f.l1o2 -width 2 -textvariable ::menued::act_opacity_1
	grid  $w.f4.f.l1o2 -row 9 -column 6 -sticky e
	frame $w.f4.f.b1c2o -bd 1 -relief sunken
	grid  $w.f4.f.b1c2o -row 9 -column 7 -sticky ew
	frame $w.f4.f.b1c2o.b1 -pady 0 -width $color_half -height $color_height
	pack  $w.f4.f.b1c2o.b1 -side left
	frame $w.f4.f.b1c2o.b2 -pady 0 -width $color_half -height $color_height
	pack  $w.f4.f.b1c2o.b2 -side left

	label $w.f4.f.l22 -text "Emphasis 2"
	grid  $w.f4.f.l22 -row 10 -column 0 -sticky w
	scale $w.f4.f.s2c2 -from 0 -to 15 -variable ::menued::act_color_0 -showvalue false \
			-command "::menued::display_pgc_color $w.f4.f.b2c2 act_opacity_0"
	grid  $w.f4.f.s2c2 -row 10 -column 1 -sticky ew
	label $w.f4.f.l2c2 -width 2 -textvariable ::menued::act_color_0
	grid  $w.f4.f.l2c2 -row 10 -column 2 -sticky e
	frame $w.f4.f.b2c2  -bd 1 -relief sunken
	grid  $w.f4.f.b2c2 -row 10 -column 3 -sticky ew
	frame $w.f4.f.b2c2.b -pady 0 -width $color_height -height $color_height
	pack  $w.f4.f.b2c2.b
	scale $w.f4.f.s2o2 -from 0 -to 15 -variable ::menued::act_opacity_0 -showvalue false \
			-command "::menued::display_opacity $w.f4.f.b2c2o act_color_0"
	grid  $w.f4.f.s2o2 -row 10 -column 5 -sticky ew
	label $w.f4.f.l2o2 -width 2 -textvariable ::menued::act_opacity_0
	grid  $w.f4.f.l2o2 -row 10 -column 6 -sticky e
	frame $w.f4.f.b2c2o -bd 1 -relief sunken
	grid  $w.f4.f.b2c2o -row 10 -column 7 -sticky ew
	frame $w.f4.f.b2c2o.b1 -pady 0 -width $color_half -height $color_height
	pack  $w.f4.f.b2c2o.b1 -side left
	frame $w.f4.f.b2c2o.b2 -pady 0 -width $color_half -height $color_height
	pack  $w.f4.f.b2c2o.b2 -side left

	frame $w.f4.f.pad -width 4
	grid  $w.f4.f.pad -row 0 -column 8

	grid  columnconfigure $w.f4.f 1 -weight 1 -uniform 1
	grid  columnconfigure $w.f4.f 5 -weight 1 -uniform 1

	# OK/Cancel buttons
	frame  $w.fb
	pack   $w.fb -fill x -padx 2 -pady 4
	button $w.fb.cancel -text "Cancel" -width 13 -command {set ::menued::colshm_done 0}
	pack   $w.fb.cancel -side right -padx 2
	button $w.fb.ok -text "OK" -width 13 -command {set ::menued::colshm_done 1}
	pack   $w.fb.ok -side right -padx 2

	update
	wm geometry $w $::config(colorschemes_win_geometry)
	wm minsize $w [winfo reqwidth $w] [winfo reqheight $w]
	wm maxsize $w [winfo reqwidth $w] [winfo reqheight $w]

	set scheme $::menued::color_scheme
	if {$scheme == 0} {set scheme 1}
	set ::menued::edit_color_scheme $scheme
	set ::menued::old_edit_color_scheme $scheme
	init_GUI_scheme_colors $scheme

	wm deiconify $w
	focus $w
	grab set $w

	set ::menued::colshm_done 0
	tkwait variable ::menued::colshm_done

	if {$::menued::colshm_done == 1} {
		set ::menued::color_schemes_edited true
		change_color_scheme -1
	}
	grab set .menued
	focus .menued
	set ::config(colorschemes_win_geometry) [wm geometry $w]
	destroy $w
	return $::menued::colshm_done
}


proc change_color_scheme {scheme} {
	if {$scheme != $::menued::old_edit_color_scheme} {
		foreach c  {sel_color_3 sel_opacity_3 sel_color_2 sel_opacity_2 \
					sel_color_1 sel_opacity_1 sel_color_0 sel_opacity_0 \
					act_color_3 act_opacity_3 act_color_2 act_opacity_2 \
					act_color_1 act_opacity_1 act_color_0 act_opacity_0} {
			set ::menued::color($::menued::old_edit_color_scheme,$c) [subst \$::menued::$c]
		}
		if {$scheme != -1} {
			set ::menued::old_edit_color_scheme $scheme
			init_GUI_scheme_colors $scheme
		}
	}
}

proc init_GUI_scheme_colors {scheme} {
	if {$scheme == 0} {set scheme 1}
	foreach c  {sel_color_3 sel_opacity_3 sel_color_2 sel_opacity_2 \
				sel_color_1 sel_opacity_1 sel_color_0 sel_opacity_0 \
				act_color_3 act_opacity_3 act_color_2 act_opacity_2 \
				act_color_1 act_opacity_1 act_color_0 act_opacity_0} {
		set ::menued::$c $::menued::color($scheme,$c)
	}
	if {[winfo exists .colshmed.f3.f.sbc1]} {
		foreach s {	.f3.f.sbc1 .f3.f.sbo1 .f3.f.spc1 .f3.f.spo1 \
					.f3.f.s1c1 .f3.f.s1o1 .f3.f.s2c1 .f3.f.s2o1 \
					.f4.f.sbc2 .f4.f.sbo2 .f4.f.spc2 .f4.f.spo2 \
					.f4.f.s1c2 .f4.f.s1o2 .f4.f.s2c2 .f4.f.s2o2} {
			set cmd "[.colshmed$s cget -command] \$[.colshmed$s cget -variable]"
			eval $cmd
		}
	}
}

proc display_pgc_color {colorframe opacity_var colorindex} {
	set c $::menued::rgbcolor($colorindex)
	$colorframe.b configure -background $c
	display_opacity_2 $c [subst \$::menued::$opacity_var] ${colorframe}o
}

proc display_opacity {opacityframe color_var opacity} {
	display_opacity_2 $::menued::rgbcolor([subst \$::menued::$color_var]) $opacity $opacityframe
}

proc display_opacity_2 {color opacity frame} {
	set o [expr {$opacity/15.0}]
	set bg [expr {1-$o}]
	set r [expr {round(((([x2d [string range $color 1 2]]/255.0)*$o)+$bg)*255)}]
	set g [expr {round(((([x2d [string range $color 3 4]]/255.0)*$o)+$bg)*255)}]
	set b [expr {round(((([x2d [string range $color 5 6]]/255.0)*$o)+$bg)*255)}]
	$frame.b1 configure -background [format "#%02X%02X%02X" $r $g $b]

	set r [expr {round([x2d [string range $color 1 2]]*$o)}]
	set g [expr {round([x2d [string range $color 3 4]]*$o)}]
	set b [expr {round([x2d [string range $color 5 6]]*$o)}]
	$frame.b2 configure -background [format "#%02X%02X%02X" $r $g $b]
}

# save a NavPack to an opened VOB file
proc save_nav_pack {filename fh sector navpack} {
	if {[catch {seek $fh [expr {$sector*2048}]} errstr]} {
		tk_messageBox -title "PgcEdit: Save NAV pack" -icon error -parent . -type ok \
				-message "Can't seek to \n\"$filename\"\nat sector $sector:\n$errstr"
		return 0
	}
	if {[catch {puts -nonewline $fh $navpack} errstr]} {
		tk_messageBox -title "PgcEdit: Save NAV pack" -icon error -parent . -type ok \
				-message "Error saving NAV pack to \n\"$filename\"\nat sector (LBA) $sector:\n$errstr"
		return 0
	}
	return 1
}

# write all edited buttons infos of the menu domain of the VTS.
proc write_buttons_menus {dir vts} {
	if {! [info exists ::buttons::butsinfo(menu,$vts,errorcode)] ||
		$::buttons::butsinfo(menu,$vts,errorcode) != 0 ||
		$::buttons::butsinfo(menu,$vts,numvobids) == 0 ||
		![info exists ::buttons::butsinfo(menu,$vts,edited)]} {
		return 0
	}
	unset -nocomplain ::menued::load_nav_pack_error_skip
	init_get_nav_pack_position_w $dir

	set poslist [get_nav_pack_position_w menu $vts 0 $dir]
	set vobfile [lindex $poslist 0]
	if {$vobfile == "" || [backup_vob $dir $vobfile]} {
		return 1
	}
	set fh [openbinfile $vobfile r+]
	if {$fh == ""} {
		return 1
	}
	for {set vid 1} {$vid <= $::buttons::butsinfo(menu,$vts,numvobids)} {incr vid} {
		if {![info exists ::buttons::butsinfo(menu,$vts,$vid,edited)]} {
			continue
		}
		for {set cid 1} {$cid <= $::buttons::butsinfo(menu,$vts,$vid,numcellids)} {incr cid} {
			if {![info exists ::buttons::butsinfo(menu,$vts,$vid,$cid,edited)]} {
				continue
			}
			# We have a modified menu cell.  Here we go!
			set lba $::buttons::butsinfo(menu,$vts,$vid,$cid,navpack)
			if {$vts == 0} {
				set txt "Writing menu buttons: VMGM, VOB ID $vid, Cell ID $cid, LBA $lba"
			} else {
				set txt "Writing menu buttons: VTSM $vts, VOB ID $vid, Cell ID $cid, LBA $lba"
			}
			show_msg $txt -1
			#puts $txt

			set origsector [load_nav_pack $vobfile $fh $lba]
			if {[string length $origsector] != 2048} {
				close $fh
				return 1
			}

			# fill in buttons definitions
			set newsector [fill_sector_with_buttons_infos $origsector menu $vts $vid $cid]

			# save new nav pack
			if {![save_nav_pack $vobfile $fh $lba $newsector]} {
				close $fh
				return 1
			}
			# extract the buttons data from the original sector for subsequent sectors
			set butsdata [string range $newsector 141 834]
			;# change highlight status to 2
			if {$::buttons::butsinfo(menu,$vts,$vid,$cid,numbuts) > 0} {
				set_int butsdata 0 2 2
			} else {
				set_int butsdata 0 0 2
			}
			unset newsector

			# write the buttons informations until the VOB ID or the Cell ID
			# or the original number of buttons change or the Highlight Status change
			set newlba [expr {$lba + [get_int origsector 0x040F 4] + 1}]
			set origstatus [get_int origsector 0x8D 2]
			set orignumbuts [get_int origsector 0x9E 1]
			unset origsector

			while {$newlba > $lba} {
				set lba $newlba
				if {[lindex [get_nav_pack_position_w menu $vts $lba $dir] 0] == ""} {
					break
				}
				set sector [load_nav_pack $vobfile $fh $lba]
				if {[string length $sector] != 2048} {
					close $fh
					return 1
				}
				set newvid [get_int sector 0x41F 2]
				set newcid [get_int sector 0x422 1]
				set newnumbuts [get_int sector 0x9E 1]
				if {$newvid!=$vid||$newcid!=$cid||$newnumbuts!=$orignumbuts} {
					break
				}
				set newstatus [get_int sector 0x8D 2]
				if {$origstatus>0&&$newstatus==0} {
					break
				}
				# fill in buttons definitions
				if {$::buttons::butsinfo(menu,$vts,$vid,$cid,numbuts) > 0} {
					if {$newstatus > 0} {
						set_int butsdata 0 $newstatus 2	;# replace original highlight status
					} else {
						set_int butsdata 0 2 2	;# new buttons: overwrite original highlight status
					}
				} else {
					set_int butsdata 0 0 2	;# or disable the buttons if the last button has been deleted
				}
				set sector [string range $sector 0 140]$butsdata[string range $sector 835 end]
				# save new nav pack
				if {![save_nav_pack $vobfile $fh $lba $sector]} {
					close $fh
					return 1
				}
				# get next sector address and loop back
				set newlba [expr {$lba + [get_int sector 0x040F 4] + 1}]
			}
			unset -nocomplain ::buttons::butsinfo(menu,$vts,$vid,$cid,edited)
		}
		unset -nocomplain ::buttons::butsinfo(menu,$vts,$vid,edited)
	}

	unset -nocomplain ::buttons::butsinfo(menu,$vts,edited)
	close $fh
	return 0
}

# write all edited BOVs infos of the VTST domain of the VTS.
proc write_buttons_titles {dir vts} {
	if {![info exists ::buttons::butsinfo(pgc,$vts,errorcode)] || \
		$::buttons::butsinfo(pgc,$vts,errorcode) != 0 ||
		$::buttons::butsinfo(pgc,$vts,numvobids) == 0 ||
		![info exists ::buttons::butsinfo(pgc,$vts,edited)]} {
		return 0
	}
	unset -nocomplain ::menued::load_nav_pack_error_skip
	set fh ""
	set oldvobfile "-"
	init_get_nav_pack_position_w $dir

	for {set vid 1} {$vid <= $::buttons::butsinfo(pgc,$vts,numvobids)} {incr vid} {
		if {![info exists ::buttons::butsinfo(pgc,$vts,$vid,edited)]} {
			continue
		}
		for {set cid 1} {$cid <= $::buttons::butsinfo(pgc,$vts,$vid,numcellids)} {incr cid} {
			if {![info exists ::buttons::butsinfo(pgc,$vts,$vid,$cid,edited)]} {
				continue
			}
			if {$vts == 0} {
				show_msg "Writing BOVs: VMGM, VOB ID $vid, Cell ID $cid" -1
			} else {
				show_msg "Writing BOVs: VTSM $vts, VOB ID $vid, Cell ID $cid" -1
			}
			# We have a modified menu cell.  Here we go!
			set lba $::buttons::butsinfo(pgc,$vts,$vid,$cid,navpack)

			set poslist [get_nav_pack_position_w pgc $vts $lba $dir]
			set vobfile [lindex $poslist 0]
			set nsector [lindex $poslist 1]
			if {$oldvobfile != $vobfile} {
				if {$fh != ""} {
					close $fh
					set fh ""
				}
				if {[backup_vob $dir $vobfile]} {
					return 1
				}
				set fh [openbinfile $vobfile r+]
				if {$fh == ""} {
					return 1
				}
				set oldvobfile $vobfile
			}
			set origsector [load_nav_pack $vobfile $fh $nsector]
			if {[string length $origsector] != 2048} {
				close $fh
				return 1
			}

			# fill in buttons definitions
			set newsector [fill_sector_with_buttons_infos $origsector pgc $vts $vid $cid]

			# save new nav pack
			if {![save_nav_pack $vobfile $fh $nsector $newsector]} {
				close $fh
				return 1
			}

			# extract the buttons data from the original sector for subsequent sectors
			set butsdata [string range $newsector 141 834]
			if {$::buttons::butsinfo(pgc,$vts,$vid,$cid,numbuts) > 0} {
				set_int butsdata 0 2 2
			} else {
				set_int butsdata 0 0 2
			}
			unset newsector

			# write the buttons informations until the VOB ID or the Cell ID
			# or the original number of buttons change or the Highlight Status change
			set newlba [expr {$lba + [get_int origsector 0x040F 4] + 1}]
			set origstatus [get_int origsector 0x8D 2]
			set orignumbuts [get_int origsector 0x9E 1]
			unset origsector

			while {$newlba > $lba} {
				set lba $newlba
				set poslist [get_nav_pack_position_w pgc $vts $lba $dir]
				set vobfile [lindex $poslist 0]
				set nsector [lindex $poslist 1]
				if {$vobfile == ""} {
					break
				}
				if {$oldvobfile != $vobfile} {
					if {$fh != ""} {
						close $fh
						set fh ""
					}
					if {[backup_vob $dir $vobfile]} {
						return 1
					}
					set fh [openbinfile $vobfile r+]
					if {$fh == ""} {
						return 1
					}
					set oldvobfile $vobfile
				}
				set sector [load_nav_pack $vobfile $fh $nsector]
				if {[string length $sector] != 2048} {
					close $fh
					return 1
				}
				set newvid [get_int sector 0x41F 2]
				set newcid [get_int sector 0x422 1]
				set newnumbuts [get_int sector 0x9E 1]
				if {$newvid!=$vid||$newcid!=$cid||$newnumbuts!=$orignumbuts} {
					break
				}
				set newstatus [get_int sector 0x8D 2]
				if {$origstatus>0&&$newstatus==0} {
					break
				}
				# fill in buttons definitions
				if {$::buttons::butsinfo(pgc,$vts,$vid,$cid,numbuts) > 0} {
					if {$newstatus > 0} {
						set_int butsdata 0 $newstatus 2	;# replace original highlight status
					} else {
						set_int butsdata 0 2 2	;# new buttons: overwrite original highlight status
					}
				} else {
					set_int butsdata 0 0 2	;# or disable the buttons if the last button has been deleted
				}
				set sector [string range $sector 0 140]$butsdata[string range $sector 835 end]
				# save new nav pack
				if {![save_nav_pack $vobfile $fh $nsector $sector]} {
					close $fh
					return 1
				}
				# get next sector address and loop back
				set newlba [expr {$lba + [get_int sector 0x040F 4] + 1}]
			}
			unset -nocomplain ::buttons::butsinfo(pgc,$vts,$vid,$cid,edited)
		}
		unset -nocomplain ::buttons::butsinfo(pgc,$vts,$vid,edited)
	}
	unset -nocomplain ::buttons::butsinfo(pgc,$vts,edited)
	if {$fh != ""} {
		close $fh
	}
	return 0
}

proc fill_sector_with_buttons_infos {buf type vts vid cid} {
	set numbuts $::buttons::butsinfo($type,$vts,$vid,$cid,numbuts)
	set numgroups $::buttons::butsinfo($type,$vts,$vid,$cid,numgroups)
	if {$numbuts > 0} {
		set_int buf 0x8D 1 2	;# Highlight status
	} else {
		set_int buf 0x8D 0 2
		set numgroups 0
	}
	set_int buf 0x9E $numbuts 1
	if {$::buttons::butsinfo($type,$vts,$vid,$cid,hlstarttime) <= 0} {
		set starttime [get_int buf 0x39 4]
		set ::buttons::butsinfo($type,$vts,$vid,$cid,hlstarttime) $starttime
		set_int buf 0x8F $starttime 4
		if {$::buttons::butsinfo($type,$vts,$vid,$cid,selendtime) >= 0} {
			incr ::buttons::butsinfo($type,$vts,$vid,$cid,selendtime) $starttime
		}
		set ::buttons::butsinfo($type,$vts,$vid,$cid,hlendtime) $::buttons::butsinfo($type,$vts,$vid,$cid,selendtime)
	}
	set_int buf 0x93 $::buttons::butsinfo($type,$vts,$vid,$cid,hlendtime) 4
	if {$::buttons::butsinfo($type,$vts,$vid,$cid,selendtime) == [expr {(0xFFFFFFFF)}]} {
		set time -1
	} else {
		set time $::buttons::butsinfo($type,$vts,$vid,$cid,selendtime)
	}
	set_int buf 0x97 $time 4
	set_int buf 0x9F $::buttons::butsinfo($type,$vts,$vid,$cid,numselectablebuts) 1
	set_int buf 0xA1 $::buttons::butsinfo($type,$vts,$vid,$cid,forceselectbutn) 1
	set_int buf 0xA2 $::buttons::butsinfo($type,$vts,$vid,$cid,forceactionbutn) 1
	switch $numgroups {
		0 - 1 {set maxbutton 36}
		2 {set maxbutton 18}
		3 {set maxbutton 12}
	}
	set groupdescr [expr {($numgroups * 0x1000) + \
			($::buttons::butsinfo($type,$vts,$vid,$cid,1,grouptype) * 0x100)}]
	if {$numgroups >= 2} {
		set groupdescr [expr {$groupdescr + \
				($::buttons::butsinfo($type,$vts,$vid,$cid,2,grouptype) * 0x10)}]
	}
	if {$numgroups >= 3} {
		set groupdescr [expr {$groupdescr + \
				$::buttons::butsinfo($type,$vts,$vid,$cid,3,grouptype)}]
	}
	set_int buf 0x9B $groupdescr 2
	for {set group 1} {$group <= $numgroups} {incr group} {
		set idx [expr {(0xBB+(($maxbutton*($group-1))*18))}]
		for {set but 1} {$but <= $numbuts} {incr but} {
			set x1pos [lindex $::buttons::butsinfo($type,$vts,$vid,$cid,$group,$but,coords) 0]
			set y1pos [lindex $::buttons::butsinfo($type,$vts,$vid,$cid,$group,$but,coords) 1]
			set x2pos [lindex $::buttons::butsinfo($type,$vts,$vid,$cid,$group,$but,coords) 3]
			set y2pos [lindex $::buttons::butsinfo($type,$vts,$vid,$cid,$group,$but,coords) 4]

			set_int buf $idx [expr \
					{($::buttons::butsinfo($type,$vts,$vid,$cid,$group,$but,colorscheme) \
					*0x40)+($x1pos/0x10)}] 1
			incr idx
			set_int buf $idx [expr {(($x1pos&0x0F)*0x10)+($x2pos/0x100)}] 1
			incr idx
			set_int buf $idx [expr {$x2pos&0xFF}] 1
			incr idx

			set_int buf $idx [expr \
					{($::buttons::butsinfo($type,$vts,$vid,$cid,$group,$but,autoaction) \
					*0x40)+($y1pos/0x10)}] 1
			incr idx
			set_int buf $idx [expr {(($y1pos&0x0F)*0x10)+($y2pos/0x100)}] 1
			incr idx
			set_int buf $idx [expr {$y2pos&0xFF}] 1
			incr idx

			set udlr $::buttons::butsinfo($type,$vts,$vid,$cid,$group,$but,udlr)
			set_int buf $idx [lindex $udlr 0] 1
			incr idx
			set_int buf $idx [lindex $udlr 1] 1
			incr idx
			set_int buf $idx [lindex $udlr 2] 1
			incr idx
			set_int buf $idx [lindex $udlr 3] 1
			incr idx
			for {set b 0} {$b < 8} {incr b} {
				set_int buf $idx 0x[lindex \
						$::buttons::butsinfo($type,$vts,$vid,$cid,$group,$but,command) $b] 1
				incr idx
			}
		}
	}
	if {[info exists ::buttons::butsinfo($type,$vts,$vid,$cid,colortable)]} {
		set buf [string range $buf 0 0xA2]$::buttons::butsinfo($type,$vts,$vid,$cid,colortable)[string range $buf 0xBB end]
	}
	return $buf
}

proc write_all_buttons {} {
	if {! [info exists ::dvddir]} {
		bell
		return 1
	}

	unset -nocomplain ::backupVOBanswer
	set progress_max 0
	set progress_current 0
	if {[info exists ::buttons::butsinfo(menu,edited)]} {
		incr progress_max [expr $::menus(numvts)+1]
	}
	if {[info exists ::buttons::butsinfo(pgc,edited)]} {
		incr progress_max $::menus(numvts)
	}

	set error 0
	if {[info exists ::buttons::butsinfo(menu,edited)]} {
		for {set vts 0} {$vts <= $::menus(numvts)} {incr vts} {
			incr progress_current
			update_progress $progress_current $progress_max
			if {[write_buttons_menus $::dvddir $vts]} {
				incr error
				break
			}
		}
		unset -nocomplain ::buttons::butsinfo(menu,edited)
	}
	if {$error == 0 && [info exists ::buttons::butsinfo(pgc,edited)]} {
		for {set vts 1} {$vts <= $::pgcs(numvts)} {incr vts} {
			incr progress_current
			update_progress $progress_current $progress_max
			if {[write_buttons_titles $::dvddir $vts]} {
				incr error
				break
			}
		}
		unset -nocomplain ::buttons::butsinfo(pgc,edited)
	}
	if {$error > 0} {
		show_msg "Errors saving the menu buttons/BOVs !"
		tk_messageBox -title "PgcEdit: Save Buttons/BOV" -icon error -parent . -type ok \
				-message "Errors saving the menu buttons/BOVs !"
		return 1
	} else {
		show_msg "Menu buttons/BOVs saved OK."
		return 0
	}
}

proc backup_vob {dir vobfile} {
	set tailname [file tail $vobfile]
	set backupfile [file join $::backupdir $tailname]
	if {! [file exists $backupfile]} {
		set ::menued::backuped($tailname) 1
		set filesize [expr {round([file size $vobfile] / 1024.0 / 1024.0 / 10.24) / 100.0}]
		if {$filesize == 0.00} {set filesize 0.01}
		set txt "There are modified buttons in file \"[file tail $vobfile]\" ($filesize GB)."
		set txt "$txt\nThis VOB file will be updated now."
		set txt "$txt\nThere is still no backup of this file in the PgcEdit_backup folder."
		set txt "$txt\n\nDo you want to backup the original file now?"
		if {[info exists ::backupVOBanswer]} {
			set r $::backupVOBanswer
		} else {
			set r [tk_messageBox -title "Save Buttons" -icon question \
				-message $txt -parent . -type "Yes to All|Yes|No to All|No" -default yes]
			if {$r == "yes to all"} {
				set ::backupVOBanswer yes
				set r yes
			}
			if {$r == "no to all"} {
				set ::backupVOBanswer no
				set r no
			}
		}
		if {$r == "yes"} {
			if {![file exists $::backupdir]} {
				show_msg "Creating backup folder..."
				if {[makedir $::backupdir "PgcEdit: Save Buttons" .]} {
					tk_messageBox -title "Save Buttons" -icon error -parent . \
						-message "Cannot create backup folder \"PgcEdit_backup\".\n\nAborting Save Buttons process!"
					return 1
				}
			}
			show_msg "Backing up \"[file tail $vobfile]\" ($filesize GB).  Please wait..."
			#blocking_msg "Backing up VOB file..."
			set rc [CopyFile $vobfile $backupfile -force "PgcEdit: Save Buttons" .]
			#close_blocking_msg
			if {$rc} {
				tk_messageBox -title "Save Buttons" -icon error -parent . \
					-message "Error copying \"[file tail $vobfile]\" to backup folder.\n\nAborting Save Buttons process!"
				return 1
			}
		}
	}
	return 0
}



#############################################################################
## Procedure:  title_vob_sectors_used
## return the number of sectors used by the VOB files of a specific VTS set
proc title_vob_sectors_used {dvddir vtsnum} {
	if {$vtsnum == 0} {
		return 0
	} else {
		set numsecs 0
		set vobbasename [file join $dvddir VTS_[format %02i $vtsnum]_]
		for {set n 1} {$n < 10} {incr n} {
			set vobname ${vobbasename}$n.VOB
			if {[file exists $vobname]} {
				incr numsecs [expr {[file size $vobname] / 0X800}]
			} else {
				break
			}
		}
		return $numsecs
	}
}

# Given a pgc type, a vts number and an LBA number, return the VOB filename and LBA number within this file
proc get_nav_pack_position {type vts lba dir} {
	if {$vts == 0} {
		if {$type == "pgc"} {
			bell
			return [list "" "No VOB file for First-Play PGC"]
		}
		set vobfile  [file nativename [file join $dir "VIDEO_TS.VOB"]]
		set sector $lba
		if {! [file exists $vobfile]} {
			return [list "" "VIDEO_TS.VOB file not found"]
		}
	} else {
		if {$type == "menu"} {
			set vobfile  [file nativename [file join $dir "VTS_[format %02d $vts]_0.VOB"]]
			set sector $lba
			if {! [file exists $vobfile]} {
				return [list "" "[file tail $vobfile] file not found"]
			}
		} else {
			set sector $lba
			for {set v 1} {$v <= 9} {incr v} {
				set vobfile [file nativename [file join $dir "VTS_[format %02d $vts]_${v}.VOB"]]
				if {[file exists $vobfile]} {
					set vobsize [expr {[file size $vobfile] / 0X800}]
					if {$sector >= $vobsize} {
						set sector [expr {$sector - $vobsize}]
					} else {
						break
					}
				} else {
					return [list "" "[file tail $vobfile] file not found"]
				}
			}
		}
	}
	set vobsize [expr {[file size $vobfile] / 0X800}]
	if {$sector >= $vobsize} {
		return [list "" "[file tail $vobfile] file too short"]
	} else {
		return [list $vobfile $sector]
	}
}

# same routine as above, but used to write buttons.  (read routine is too slow)
proc init_get_nav_pack_position_w {dir} {
	unset -nocomplain ::menued::vobfilesizes
	set ::menued::vobfiles [glob -directory $dir -tails -nocomplain -types f *.VOB]
	foreach f $::menued::vobfiles {
		set ::menued::vobfilesizes($f) [expr {[file size [file join $dir $f]] / 0X800}]
	}
}
proc get_nav_pack_position_w {type vts lba dir} {
	if {$vts == 0} {
		if {$type == "pgc"} {
			bell
			return [list "" "No VOB file for First-Play PGC"]
		}
		set vobtail  "VIDEO_TS.VOB"
		set vobfile  [file nativename [file join $dir $vobtail]]
		set sector $lba
		if {[lsearch $::menued::vobfiles $vobtail] == -1} {
			return [list "" "$vobtail file not found"]
		}
	} else {
		if {$type == "menu"} {
			set vobtail  "VTS_[format %02d $vts]_0.VOB"
			set vobfile  [file nativename [file join $dir $vobtail]]
			set sector $lba
			if {[lsearch $::menued::vobfiles $vobtail] == -1} {
				return [list "" "$vobtail file not found"]
			}
		} else {
			set sector $lba
			for {set v 1} {$v <= 9} {incr v} {
				set vobtail "VTS_[format %02d $vts]_${v}.VOB"
				set vobfile [file nativename [file join $dir $vobtail]]
			if {[lsearch $::menued::vobfiles $vobtail] != -1} {
					set vobsize $::menued::vobfilesizes($vobtail)
					if {$sector >= $vobsize} {
						set sector [expr {$sector - $vobsize}]
					} else {
						break
					}
				} else {
					return [list "" "$vobtail file not found"]
				}
			}
		}
	}
	set vobsize $::menued::vobfilesizes($vobtail)
	if {$sector >= $vobsize} {
		return [list "" "$vobtail file too short"]
	} else {
		return [list $vobfile $sector]
	}
}

# open a binary file for read or write
proc openbinfile {filename mode} {
	# check if file is writable
	if {$mode == "r+"} {
		catch {file attributes $filename -readonly false}
		catch {file attributes $filename -permissions +W}
	}
	if {[catch {set fh [open $filename $mode]} errstr]} {
		tk_messageBox -title "PgcEdit: Open file error!" -icon error \
				-message "Error opening file\n\"$filename\"\n$errstr" \
				-parent . -type ok
		return ""
	}
	fconfigure $fh -translation binary
	return $fh
}

# load a NavPack from an opened VOB file
proc load_nav_pack {filename fh sector} {
	set ::menued::load_nav_pack_error false
	set navpack ""
	set blocksize [expr {(0x800)}]
	set errmsg "Error loading NAV pack from \n\"$filename\"\nat sector $sector (in file)"
	set startaddress [expr {$sector*$blocksize}]
	if {[catch {seek $fh $startaddress} errstr]} {
		if {! [info exists ::skip_dialog_during_session(load_nav_pack_1)]} {
			set r [tk_messageBox -title "PgcEdit: Load NAV pack" -icon error -parent . \
					-type "OK to All|OK" -default ok \
					-message "Can't seek to \n\"$filename\"\nat sector $sector:\n$errstr"]
			if {$r == "ok to all"} {
				set ::skip_dialog_during_session(load_nav_pack_1) true
			}
		}
		return ""
	}
	if {[catch {set navpack [read $fh $blocksize]} errstr]} {
		if {! [info exists ::skip_dialog_during_session(load_nav_pack_2)]} {
			if {[string first "invalid argument" $errstr] > -1} {
				set errstr "Can't read file data.\nThe VOB file *may* be CSS encrypted."
			}
			set r [tk_messageBox -title "PgcEdit: Load NAV pack" -icon error -parent . \
					-type "OK to All|OK" -default ok \
					-message "$errmsg:\n$errstr"]
			if {$r == "ok to all"} {
				set ::skip_dialog_during_session(load_nav_pack_2) true
			}
		}
		set ::menued::load_nav_pack_error true
		set navpack ""
	} elseif {[string length $navpack]<$blocksize} {
		if {! [info exists ::skip_dialog_during_session(load_nav_pack_3)]} {
			set r [tk_messageBox -title "PgcEdit: Load NAV pack" -icon error -parent . \
					-type "OK to All|OK" -default ok \
					-message "$errmsg:\nNot enough data retrieved ([string length $navpack] bytes)."]
			if {$r == "ok to all"} {
				set ::skip_dialog_during_session(load_nav_pack_3) true
			}
		}
		set ::menued::load_nav_pack_error true
		set navpack ""
	}
	if {$navpack!=""&&([get_int navpack 0x26 4]!=447||[get_int navpack 0x400 4]!=447||[get_int navpack 0 4]!=442)} {
		if {! [info exists ::menued::load_nav_pack_error_skip] && ! [info exists ::skip_dialog_during_session(load_nav_pack_4)]} {
			if {[get_int navpack 0x26 4]==0&&[get_int navpack 0x400 4]==0&&[get_int navpack 0 4]==0} {
				set msg "$errmsg:\nMay be a ARccOS protection sector skipped by a DVD ripper."
			} else {
				set msg "$errmsg:\nNav Pack identifier not valid.\nThe VOB file *may* be CSS encoded:\nYou cannot edit a CSS encoded disc!\nOr the VOB doesn't match the IFOs:\nTry to do a mock strip with IfoEdit."
			}
			set r [tk_messageBox -title "PgcEdit: Load NAV pack" -icon error -parent . \
					-type "Yes to All domains|Yes|No" -default yes \
					-message "$msg\n\nSkip remaining 'Bad Nav Pack' error messages\nfor the current domain?"]
			if {$r == "yes"} {
				set ::menued::load_nav_pack_error_skip true
			}
			if {$r == "yes to all domains"} {
				set ::skip_dialog_during_session(load_nav_pack_4) true
			}
		}
		set ::menued::load_nav_pack_error true
		set navpack ""
	}
	return $navpack
}

proc get_buttons_from_navpack {type vts navpack_upvar {set 1}} {
	if {$set == 1} {
		set set ""
	} else {
		set set ",$set"
	}
	upvar $navpack_upvar navpack
	set vobid [get_int navpack 0x41F 2]
	set cellid [get_int navpack 0x422 1]
	set numbuts [get_int navpack 0x9E 1]
	set ::buttons::butsinfo($type,$vts,$vobid,$cellid$set,navpack) [get_int navpack 0x2D 4]
	set ::buttons::butsinfo($type,$vts,$vobid,$cellid$set,numbuts) $numbuts
	set grouping [get_int navpack 0x9B 2]
	set numgroups [expr {$grouping/0x1000}]
	if {$numgroups == 0} {
		set numbuts 0
	}
	if {$numbuts>0} {
		set ::buttons::butsinfo($type,$vts,$vobid,$cellid$set,hlstarttime) [get_int navpack 0x8F 4]
		set ::buttons::butsinfo($type,$vts,$vobid,$cellid$set,hlendtime)   [get_int navpack 0x93 4]
		set ::buttons::butsinfo($type,$vts,$vobid,$cellid$set,selendtime)  [get_int navpack 0x97 4]
		set ::buttons::butsinfo($type,$vts,$vobid,$cellid$set,numselectablebuts) [get_int navpack 0x9F 1]
		set ::buttons::butsinfo($type,$vts,$vobid,$cellid$set,forceselectbutn) [get_int navpack 0xA1 1]
		set ::buttons::butsinfo($type,$vts,$vobid,$cellid$set,forceactionbutn) [get_int navpack 0xA2 1]
		set ::buttons::butsinfo($type,$vts,$vobid,$cellid$set,numgroups) $numgroups
		for {set g 1} {$g<=$numgroups} {incr g} {
			switch $g {
				1 { set ::buttons::butsinfo($type,$vts,$vobid,$cellid$set,$g,grouptype) [expr {($grouping & 0x0F00) / 0x100}]
					set start(1) [expr {(0xBB)}]
				}
				2 { set ::buttons::butsinfo($type,$vts,$vobid,$cellid$set,$g,grouptype) [expr {($grouping & 0x00F0) / 0x10}]
					set start(2) [expr {(0x1FF)}]
				}
				3 { set ::buttons::butsinfo($type,$vts,$vobid,$cellid$set,$g,grouptype) [expr { $grouping & 0x000F}]
					set start(2) [expr {(0x193)}]
					set start(3) [expr {(0x26B)}]
				}
			}
		}
		for {set g 1} {$g<=$numgroups} {incr g} {
			for {set b 1} {$b<=$numbuts} {incr b} {
				set idx [expr {$start($g)+(($b-1)*18)}]
				set ::buttons::butsinfo($type,$vts,$vobid,$cellid$set,$g,$b,colorscheme) [expr {([get_int navpack $idx 1]&0xC0)/0x40}]
				set ::buttons::butsinfo($type,$vts,$vobid,$cellid$set,$g,$b,autoaction) [expr {([get_int navpack [expr {$idx+3}] 1]&0x40)/0x40}]
				set ::buttons::butsinfo($type,$vts,$vobid,$cellid$set,$g,$b,coords)	[format "%d %d - %d %d" \
						[expr {([get_int navpack $idx 2]&0x3FF0)/16}] [expr {([get_int navpack [expr {$idx+3}] 2]&0x3FF0)/16}] \
						[expr {[get_int navpack [expr {$idx+1}] 2]&0x03FF}] [expr {[get_int navpack [expr {$idx+4}] 2]&0x03FF}]]
				set ::buttons::butsinfo($type,$vts,$vobid,$cellid$set,$g,$b,udlr) [list [get_int navpack [expr {$idx+6}] 1] \
						[get_int navpack [expr {$idx+7}] 1] [get_int navpack [expr {$idx+8}] 1] [get_int navpack [expr {$idx+9}] 1]]
				set ::buttons::butsinfo($type,$vts,$vobid,$cellid$set,$g,$b,command) [format "%02x %02x %02x %02x %02x %02x %02x %02x" \
						[get_int navpack [expr {$idx+10}] 1] [get_int navpack [expr {$idx+11}] 1] [get_int navpack [expr {$idx+12}] 1] [get_int navpack [expr {$idx+13}] 1] \
						[get_int navpack [expr {$idx+14}] 1] [get_int navpack [expr {$idx+15}] 1] [get_int navpack [expr {$idx+16}] 1] [get_int navpack [expr {$idx+17}] 1]]
			}
		}
	}
	return $numbuts
}

# get all buttons infos from a specific domain in a specific VTS.
proc get_domain_buttons_infos_menus {dir vts} {
	tolog "Scanning menu VOB of VTS $vts..."

	unset -nocomplain ::menued::load_nav_pack_error_skip
	set ::buttons::butsinfo(menu,$vts,numvobids) 0
	set ::buttons::butsinfo(menu,$vts,errorcode) 0	;# TODO: supprimer
	set ::buttons::butsinfo(menu,$vts,multiplesets) false
	set totalbuts 0

	if {$vts > 0} {set tmp VTSI_MAT} else {set tmp VMGM_MAT}
	if {[get_int ::raw_sectors($vts,$tmp) 0x154 2]==0} {
		set ::buttons::butsinfo(menu,$vts,errorcode) 0
		set ::buttons::butsinfo(menu,$vts,scanned) 2
		set ::buttons::butsinfo(menu,$vts,errormsg) "No Subpic stream in IFO"
		set ::buttons::butsinfo(menu,$vts,numvobids) 0
		set ::buttons::butsinfo(menu,$vts,totalbuttons) 0
		tolog "Note: No Subpic stream in IFO"
	}

	if {$vts == 0} {
		set tablename VMGM_C_ADT
		set vobutablename VMGM_VOBU_ADMAP
	} else {
		set tablename VTSM_C_ADT
		set vobutablename VTSM_VOBU_ADMAP
	}
	if {$::raw_sectors($vts,$tablename) == ""} {
		set ::buttons::butsinfo(menu,$vts,errorcode) 2
		set ::buttons::butsinfo(menu,$vts,scanned) 2
		set ::buttons::butsinfo(menu,$vts,errormsg) "No Cells in menu domain"
		set ::buttons::butsinfo(menu,$vts,numvobids) 0
		set ::buttons::butsinfo(menu,$vts,totalbuttons) 0
		tolog "No Cells in menu domain"
		return 0
	}

	set numids [expr {([get_int ::raw_sectors($vts,$tablename) 4 4]-7)/12}]
	if {$numids == 0} {
		set ::raw_sectors($vts,$tablename) ""
		set ::buttons::butsinfo(menu,$vts,errorcode) -1
		set ::buttons::butsinfo(menu,$vts,scanned) 2
		set ::buttons::butsinfo(menu,$vts,errormsg) "No Cells in menu domain"
		set ::buttons::butsinfo(menu,$vts,numvobids) 0
		set ::buttons::butsinfo(menu,$vts,totalbuttons) 0
		tolog "No Cells in menu domain"
		return 0
	}

	set lba [get_int ::raw_sectors($vts,$tablename) 12 4]

	set poslist [get_nav_pack_position menu $vts $lba $dir]
	set vobfile [lindex $poslist 0]
	if {$vobfile == ""} {
		set ::buttons::butsinfo(menu,$vts,errorcode) -1
		set ::buttons::butsinfo(menu,$vts,scanned) 2
		set ::buttons::butsinfo(menu,$vts,errormsg) [lindex $poslist 1]
		set ::buttons::butsinfo(menu,$vts,numvobids) 0
		set ::buttons::butsinfo(menu,$vts,totalbuttons) 0
		tolog [lindex $poslist 1]
		return 0
	}

	set fh [openbinfile $vobfile r]
	if {$fh == ""} {
		set ::buttons::butsinfo(menu,$vts,errorcode) -1
		set ::buttons::butsinfo(menu,$vts,scanned) 2
		set ::buttons::butsinfo(menu,$vts,errormsg) "Can't open [file tail $vobfile]"
		set ::buttons::butsinfo(menu,$vts,numvobids) 0
		set ::buttons::butsinfo(menu,$vts,totalbuttons) 0
		tolog "Can't open [file tail $vobfile]"
		bell
		return 0
	}

	set navpack [load_nav_pack $vobfile $fh $lba]
	if {$navpack!=""} {
		set ::buttons::butsinfo(menu,$vts,errorcode) 0
		set ::buttons::butsinfo(menu,$vts,errormsg) "OK"
	} else {
		set ::buttons::butsinfo(menu,$vts,errorcode) -1
		set ::buttons::butsinfo(menu,$vts,errormsg) "First Nav Pack in [file tail $vobfile] is not of required type"
		set ::buttons::butsinfo(menu,$vts,numvobids) 0
		tolog "First Nav Pack in [file tail $vobfile] is not of required type"
		close $fh
		return 0
	}

	for {set idx 12} {$idx<=[expr {$numids*12}]} {incr idx 12} {
		show_msg "Scanning [file tail $vobfile] for menu buttons: cell [expr {$idx/12}]..." -1
		set lba [get_int ::raw_sectors($vts,$tablename) $idx 4]
		set endlba [get_int ::raw_sectors($vts,$tablename) [expr {$idx+4}] 4]
		set navpack [load_nav_pack $vobfile $fh $lba]
		if {$navpack == "" && $::menued::load_nav_pack_error} break

		while {$navpack!=""} {
			set vobid	[get_int navpack 0x41F 2]
			set cellid	[get_int navpack 0x422 1]
			set hl_status [get_int navpack 0x8D 2]
			set numbuts	[get_int navpack 0x9E 1]
			set grouping [get_int navpack 0x9B 2]
			set numgroups [expr {$grouping/0x1000}]
			if {$numgroups == 0} {
				set numbuts 0
			}
			if {$vobid>$::buttons::butsinfo(menu,$vts,numvobids)} {
				set ::buttons::butsinfo(menu,$vts,numvobids) $vobid
				set ::buttons::butsinfo(menu,$vts,$vobid,numcellids) $cellid
			}
			if {![info exists ::buttons::butsinfo(menu,$vts,$vobid,numcellids)] || \
					$cellid>$::buttons::butsinfo(menu,$vts,$vobid,numcellids)} {
				set ::buttons::butsinfo(menu,$vts,$vobid,numcellids) $cellid
			}
			if {$hl_status > 0 && $hl_status != 2} {
				if {[info exists ::buttons::butsinfo(menu,$vts,$vobid,$cellid,numbuts)] && \
						$::buttons::butsinfo(menu,$vts,$vobid,$cellid,numbuts)>0} {
					tolog "WARNING: multiple sets of menu buttons in VTS $vts's menu domain, V/CID $vobid/$cellid, at LBA $lba: highlight status = $hl_status"
					set ::buttons::butsinfo(menu,multiplesets) true
					set ::buttons::butsinfo(menu,$vts,multiplesets) true
					if {! [info exists ::buttons::butsinfo(menu,$vts,$vobid,$cellid,numsets)]} {
						set ::buttons::butsinfo(menu,$vts,$vobid,$cellid,numsets) 2
					} else {
						incr ::buttons::butsinfo(menu,$vts,$vobid,$cellid,numsets)
					}
					set set $::buttons::butsinfo(menu,$vts,$vobid,$cellid,numsets)
					set ::buttons::butsinfo(menu,$vts,$vobid,$cellid,$set,hl_status) $hl_status
					set ::buttons::butsinfo(menu,$vts,$vobid,$cellid,$set,lba) $lba
#					get_buttons_from_navpack menu $vts navpack 2	;# not yet used
				}
			}
			if {![info exists ::buttons::butsinfo(menu,$vts,$vobid,$cellid,numbuts)] || \
						($numbuts > 0 && $::buttons::butsinfo(menu,$vts,$vobid,$cellid,numbuts)==0)} {
				set ::buttons::butsinfo(menu,$vts,$vobid,$cellid,numsets) 1
				set b [get_buttons_from_navpack menu $vts navpack]
				incr totalbuts $b
				tolog "Found $b buttons in V/CID $vobid/$cellid, at LBA $lba."

				# Break if last VOBU's buttons definitions are the same as the first one with buttons,
				# to speedup the parsing.
				# Note: this is potentially a bug, but should work in *almost* all cases.
				# If the break occur when it should not, there will be no warning for multiple sets.
				if {$b>0} {
					# find last LBA of current cell
					set endsec [get_int ::raw_sectors($vts,$tablename) [expr {$idx+4}] 4]
					if {$endsec == [expr 0xffffffff]} {
						break	;# safety check
					}
					incr endsec
					set tidx [string first [d2c $endsec 4] $::raw_sectors($vts,$vobutablename)]
					while {([expr {$tidx%4}] != 0 && $tidx != -1) || $tidx == 0} {
						incr tidx
						set tidx [string first [d2c $endsec 4] $::raw_sectors($vts,$vobutablename) $tidx]
					}
					if {$tidx == -1} {
						set tidx [expr {[string length $::raw_sectors($vts,$vobutablename)] - 4}]
					} else {
						incr tidx -4
					}
					set lastlba [get_int ::raw_sectors($vts,$vobutablename) $tidx 4]
					# only one nav pack in this cell -> break immediately
					if {$lastlba==$lba} {
						break
					}
					# load last nav pack of the current cell
					set lastnavpack [load_nav_pack $vobfile $fh $lastlba]
					# break if last buttons definitions are equal
				 	if {[string range $navpack 143 834] == [string range $lastnavpack 143 834]} {
						break
					}
				}
			}
			set oldlba $lba
			set lba [expr {$lba+[get_int navpack 0x40F 4]+1}]
			if {$lba<=$oldlba} {
				break
			}
			if {$lba<=$endlba} {
				set navpack [load_nav_pack $vobfile $fh $lba]
				if {$navpack==""} break
			} else {
				break
			}
		}
	}
	close $fh

	# fill holes in VOB and Cell IDs
	for {set v 1} {$v<=$::buttons::butsinfo(menu,$vts,numvobids)} {incr v} {
		if {![info exists ::buttons::butsinfo(menu,$vts,$v,numcellids)]} {
			set ::buttons::butsinfo(menu,$vts,$v,numcellids) 0
		} else {
			for {set c 1} {$c<$::buttons::butsinfo(menu,$vts,$v,numcellids)} {incr c} {
				if {![info exists ::buttons::butsinfo(menu,$vts,$v,$c,navpack)]} {
					set ::buttons::butsinfo(menu,$vts,$v,$c,navpack) -1
					set ::buttons::butsinfo(menu,$vts,$v,$c,numbuts) 0
				}
			}
		}
	}
	set ::buttons::butsinfo(menu,$vts,totalbuttons) $totalbuts
	unset -nocomplain ::menued::load_nav_pack_error_skip
	return $totalbuts
}

# get all buttons infos from a specific domain in a specific VTS.
proc get_domain_buttons_infos_titles {dir vts lbas_to_skip} {
	unset -nocomplain ::menued::load_nav_pack_error_skip
	set aborted false

	foreach a [array names ::buttons::butsinfo pgc,$vts,*] {
		unset ::buttons::butsinfo($a)
	}

	set ::buttons::butsinfo(pgc,$vts,multiplesets) false
	set ::buttons::butsinfo(pgc,$vts,numvobids) 0
	set ::buttons::butsinfo(pgc,$vts,errorcode) 0	;# TODO: supprimer
	set totalbuts 0

	if {$::raw_sectors($vts,VTS_C_ADT) == ""} {
		set ::buttons::butsinfo(pgc,$vts,errorcode) -1
		set ::buttons::butsinfo(pgc,$vts,scanned) 2
		set ::buttons::butsinfo(pgc,$vts,errormsg) "ERROR: No Cells in VTST domain!"
		set ::buttons::butsinfo(pgc,$vts,numvobids) 0
		set ::buttons::butsinfo(pgc,$vts,totalbuttons) 0
		return 0
	}

	set numids [expr {([get_int ::raw_sectors($vts,VTS_C_ADT) 4 4]-7)/12}]
	if {$numids == 0} {
		set ::raw_sectors($vts,VTS_C_ADT) ""
		set ::buttons::butsinfo(pgc,$vts,errorcode) -1
		set ::buttons::butsinfo(pgc,$vts,scanned) 2
		set ::buttons::butsinfo(pgc,$vts,errormsg) "ERROR: No Cells in VTST domain!"
		set ::buttons::butsinfo(pgc,$vts,numvobids) 0
		set ::buttons::butsinfo(pgc,$vts,totalbuttons) 0
		return 0
	}
	set lba [get_int ::raw_sectors($vts,VTS_C_ADT) 12 4]

	set poslist [get_nav_pack_position pgc $vts $lba $dir]
	set vobfile [lindex $poslist 0]
	set sector  [lindex $poslist 1]
	set oldvobfile $vobfile
	if {$vobfile == ""} {
		set ::buttons::butsinfo(pgc,$vts,errorcode) -1
		set ::buttons::butsinfo(pgc,$vts,scanned) 2
		set ::buttons::butsinfo(pgc,$vts,errormsg) $sector
		set ::buttons::butsinfo(pgc,$vts,numvobids) 0
		set ::buttons::butsinfo(pgc,$vts,totalbuttons) 0
		return 0
	}

	set fh [openbinfile $vobfile r]
	if {$fh == ""} {
		set ::buttons::butsinfo(pgc,$vts,errorcode) -1
		set ::buttons::butsinfo(pgc,$vts,scanned) 2
		set ::buttons::butsinfo(pgc,$vts,errormsg) "Can't open [file tail $vobfile]"
		set ::buttons::butsinfo(pgc,$vts,numvobids) 0
		set ::buttons::butsinfo(pgc,$vts,totalbuttons) 0
		bell
		return 0
	}

	if {[winfo exists .scan.f.f.c.f1.c$vts]} {
		.scan.f.f.c.f1.c$vts configure -text "VTS $vts: Scanning [file tail $vobfile]..."
	}

	set navpack [load_nav_pack $vobfile $fh $sector]
	if {$navpack!=""} {
		set ::buttons::butsinfo(pgc,$vts,errorcode) 0
		set ::buttons::butsinfo(pgc,$vts,errormsg) "OK"
	} else {
		set ::buttons::butsinfo(pgc,$vts,errorcode) -1
		set ::buttons::butsinfo(pgc,$vts,errormsg) "First Nav Pack in [file tail $vobfile] is not of required type"
		set ::buttons::butsinfo(pgc,$vts,numvobids) 0
	}

	for {set idx 12} {$idx<=[expr {$numids*12}]} {incr idx 12} {
		# skip multi-angles VOBUs, except first one.
		set vobid    [get_int ::raw_sectors($vts,VTS_C_ADT) [expr {$idx-4}] 2]
		set cellid   [get_int ::raw_sectors($vts,VTS_C_ADT) [expr {$idx-2}] 1]
		if {[info exists ::buttons::butsinfo(pgc,$vts,$vobid,$cellid,numbuts)]&&
				$lbas_to_skip>1} {
			continue
		}

		set check 1
		set currentlbastoskip $lbas_to_skip
		set startlba [get_int ::raw_sectors($vts,VTS_C_ADT) $idx 4]
		set endlba   [get_int ::raw_sectors($vts,VTS_C_ADT) [expr {$idx+4}] 4]
		set lba $startlba

		after 1 {set ::menued::pause 1}
		tkwait variable ::menued::pause
		if {$::menued::scan_abort} {
			set aborted 1
			break
		}
		if {$::menued::scan_pause} {
			.scan.fb.pause configure -text "Resume" -command ".scan.fb.pause configure -text Pause; after 50 {set ::menued::scan_pause 0}"
			tkwait variable ::menued::scan_pause
			.scan.fb.pause configure -text "Pause" -command "set ::menued::scan_pause 1"
		}
		show_msg "Scanning [file tail $vobfile] for BOVs: cell [expr {$idx/12}] (V/CID: $vobid/$cellid, starting LBA: $startlba)..." -1

		set startvobuidx [string first [d2c $lba 4] $::raw_sectors($vts,VTS_VOBU_ADMAP) 4]
		while {[expr $startvobuidx % 4] != 0 && $startvobuidx != -1} {
			set startvobuidx [string first [d2c $lba 4] $::raw_sectors($vts,VTS_VOBU_ADMAP) [expr {$startvobuidx+1}]]
		}
		if {$startvobuidx == -1} {
			set ::buttons::butsinfo(pgc,$vts,errorcode) -1
			set ::buttons::butsinfo(pgc,$vts,scanned) 2
			set ::buttons::butsinfo(pgc,$vts,errormsg) "Invalid IFO file: cannot find LBA $lba in VTS_VOBU_ADMAP"
			set ::buttons::butsinfo(pgc,$vts,numvobids) 0
			set ::buttons::butsinfo(pgc,$vts,totalbuttons) 0
			bell
			return 0
		}
		set endvobuidx [string first [d2c [expr {$endlba+1}] 4] $::raw_sectors($vts,VTS_VOBU_ADMAP) 4]
		while {$endvobuidx != -1 && [expr $endvobuidx % 4] != 0} {
			set endvobuidx [string first [d2c [expr {$endlba+1}] 4] $::raw_sectors($vts,VTS_VOBU_ADMAP) [expr {$endvobuidx+1}]]
		}
		if {$endvobuidx == -1} {
			set endvobuidx [expr {[string length $::raw_sectors($vts,VTS_VOBU_ADMAP)]-1}]
		} else {
			incr endvobuidx -1
		}
		set vobutable [string range $::raw_sectors($vts,VTS_VOBU_ADMAP) $startvobuidx $endvobuidx]
		set vobutablelen [string length $vobutable]
		set vobutablepos 0

		set poslist [get_nav_pack_position pgc $vts $lba $dir]
		set vobfile [lindex $poslist 0]
		set sector  [lindex $poslist 1]
		if {$vobfile==""} {break}
		if {$vobfile!=$oldvobfile} {
			catch {close $fh}
			set fh [openbinfile $vobfile r]
			if {$fh==""} break
			set oldvobfile $vobfile
			if {[winfo exists .scan.f.f.c.f1.c$vts]} {
				.scan.f.f.c.f1.c$vts configure -text "VTS $vts: Scanning [file tail $vobfile]..."
			}
			after 5 {set ::menued::pause 1}
			tkwait variable ::menued::pause
		}

		set navpack [load_nav_pack $vobfile $fh $sector]
		if {$navpack == "" && $::menued::load_nav_pack_error} break

		while {$navpack!=""} {
			set vobid	[get_int navpack 0x41F 2]
			set cellid	[get_int navpack 0x422 1]
			set hl_status [get_int navpack 0x8D 2]
			set numbuts	[get_int navpack 0x9E 1]
			set grouping [get_int navpack 0x9B 2]
			set numgroups [expr {$grouping/0x1000}]
			if {$numgroups == 0} {
				set numbuts 0
			}
			if {$vobid>$::buttons::butsinfo(pgc,$vts,numvobids)} {
				set ::buttons::butsinfo(pgc,$vts,numvobids) $vobid
				set ::buttons::butsinfo(pgc,$vts,$vobid,numcellids) $cellid
			}
			if {![info exists ::buttons::butsinfo(pgc,$vts,$vobid,numcellids)] || \
					$cellid>$::buttons::butsinfo(pgc,$vts,$vobid,numcellids)} {
				set ::buttons::butsinfo(pgc,$vts,$vobid,numcellids) $cellid
			}
			if {$hl_status > 0 && $hl_status != 2} {
				if {[info exists ::buttons::butsinfo(pgc,$vts,$vobid,$cellid,numbuts)] && \
						$::buttons::butsinfo(pgc,$vts,$vobid,$cellid,numbuts)>0} {
					tolog "WARNING: multiple sets of BOVs in VTS $vts's VTST domain, V/CID $vobid/$cellid, at LBA $lba: highlight status = $hl_status"
					set ::buttons::butsinfo(pgc,multiplesets) true
					set ::buttons::butsinfo(pgc,$vts,multiplesets) true
					if {! [info exists ::buttons::butsinfo(pgc,$vts,$vobid,$cellid,numsets)]} {
						set ::buttons::butsinfo(pgc,$vts,$vobid,$cellid,numsets) 2
					} else {
						incr ::buttons::butsinfo(pgc,$vts,$vobid,$cellid,numsets)
					}
					set set $::buttons::butsinfo(pgc,$vts,$vobid,$cellid,numsets)
					set ::buttons::butsinfo(pgc,$vts,$vobid,$cellid,$set,hl_status) $hl_status
					set ::buttons::butsinfo(pgc,$vts,$vobid,$cellid,$set,lba) $lba
#					get_buttons_from_navpack pgc $vts navpack 2	;# not yet used
				}
			}
			if {![info exists ::buttons::butsinfo(pgc,$vts,$vobid,$cellid,numbuts)]|| \
					$::buttons::butsinfo(pgc,$vts,$vobid,$cellid,numbuts)==0} {
				if {$numbuts>0} {
					if {$currentlbastoskip>1&&$lba>$startlba} {
						set currentlbastoskip 1
						set vobutablepos $lastvobuposchecked
					} else {
						set ::buttons::butsinfo(pgc,$vts,$vobid,$cellid,numsets) 1
						get_buttons_from_navpack pgc $vts navpack
						incr totalbuts $numbuts
						if {!$::config(check_multiple_BOVs_sets)} {
							set currentlbastoskip $lbas_to_skip
							break
						}
					}
				} elseif {![info exists ::buttons::butsinfo(pgc,$vts,$vobid,$cellid,numbuts)]} {
					set ::buttons::butsinfo(pgc,$vts,$vobid,$cellid,numsets) 1
					get_buttons_from_navpack pgc $vts navpack
					set currentlbastoskip $lbas_to_skip
				}
			}
			set lastvobuposchecked $vobutablepos
			set newvobutablepos [expr {$vobutablepos+(4*$currentlbastoskip)}]
			if {$newvobutablepos>$vobutablelen&&$vobutablelen>4} {
				set newvobutablepos [expr {$vobutablelen-4}]
				if {$newvobutablepos < 4} {set newvobutablepos 4}
			}
			if {$newvobutablepos==$vobutablepos||$newvobutablepos>=$vobutablelen} {
				break
			}
			set vobutablepos $newvobutablepos
			set lba [get_int vobutable $vobutablepos 4]
			set poslist [get_nav_pack_position pgc $vts $lba $dir]
			set vobfile [lindex $poslist 0]
			set sector  [lindex $poslist 1]
			if {$vobfile==""} break
			if {$vobfile!=$oldvobfile} {
				catch {close $fh}
				set fh [openbinfile $vobfile r]
				if {$fh==""} break
				set oldvobfile $vobfile
				if {[winfo exists .scan.f.f.c.f1.c$vts]} {
					.scan.f.f.c.f1.c$vts configure -text "VTS $vts: Scanning [file tail $vobfile]..."
				}
				after 5 {set ::menued::pause 1}
				tkwait variable ::menued::pause
			}
			set navpack [load_nav_pack $vobfile $fh $sector]
			if {$navpack==""} break
			incr check
			if {$check==50&&[winfo exists .scan.f.f.c.f1.c$vts]} {
				set check 1
				after 1 {set ::menued::pause 1}
				tkwait variable ::menued::pause
				if {$::menued::scan_abort} {
					set aborted 1
					break
				}
				if {$::menued::scan_pause} {
					.scan.fb.pause configure -text "Resume" -command ".scan.fb.pause configure -text Pause; after 50 {set ::menued::scan_pause 0}"
					tkwait variable ::menued::scan_pause
					.scan.fb.pause configure -text "Pause" -command "set ::menued::scan_pause 1"
				}
			}
		}
		if {$aborted} break
	}
	catch {close $fh}

	# fill holes in VOB and Cell IDs
	for {set v 1} {$v<=$::buttons::butsinfo(pgc,$vts,numvobids)} {incr v} {
		if {![info exists ::buttons::butsinfo(pgc,$vts,$v,numcellids)]} {
			set ::buttons::butsinfo(pgc,$vts,$v,numcellids) 0
		} else {
			for {set c 1} {$c<$::buttons::butsinfo(pgc,$vts,$v,numcellids)} {incr c} {
				if {![info exists ::buttons::butsinfo(pgc,$vts,$v,$c,navpack)]} {
					set ::buttons::butsinfo(pgc,$vts,$v,$c,navpack) -1
					set ::buttons::butsinfo(pgc,$vts,$v,$c,numbuts) 0
				}
			}
		}
	}
	set ::buttons::butsinfo(pgc,$vts,totalbuttons) $totalbuts

	if {[winfo exists .scan.f.f.c.f1.c$vts]} {
		if {$aborted} {set abortedtext " (Aborted at LBA $lba)"} else {set abortedtext ""}
		.scan.f.f.c.f1.c$vts configure -text "VTS $vts: Found $totalbuts buttons.$abortedtext"
		after 5 {set ::menued::pause 1}
		tkwait variable ::menued::pause
	}

	if {$aborted} {
		set ::buttons::butsinfo(pgc,$vts,scanned) 0
	} else {
		set ::buttons::butsinfo(pgc,$vts,scanned) 1
	}
	show_msg "" -1
	unset -nocomplain ::menued::load_nav_pack_error_skip
	return $totalbuts
}

proc get_buttons_all_menu_domains {dir} {
	set ::buttons::butsinfo(menu,multiplesets) false
	set totalnumbuts 0
	for {set vts 0} {$vts <= $::menus(numvts)} {incr vts} {
		update_progress [expr $vts+1] [expr $::menus(numvts)+1]
		set numbuts [get_domain_buttons_infos_menus $dir $vts]
		if {$numbuts > 0} {
			tolog "Found $numbuts buttons in menu domain of VTS $vts."
		}
		incr totalnumbuts $numbuts
		add_menu_buttons_marker $vts
	}
	show_msg "Found a total of $totalnumbuts menu buttons in menu domain." -1
	catch ::main::init_tree_menu_no_move
	return $totalnumbuts
}

proc get_buttons_all_title_domains {dir autostart {msg ""} {autoclose false}} {
	set processed false
	set ::buttons::butsinfo(pgc,multiplesets) false
	if {$::pgcs(numvts) < 1} {
		set ::menued::scan_done 1
		return
	}

	if {$autostart} {
		set old_msg ""
		catch {set old_msg [.mf.st.f.status itemcget $::progressbar_text -text]}
		if {$old_msg != ""} {set old_msg "$old_msg   "}
	} else {
		set old_msg ""
	}
	set totalbuttons 0

	set w .scan
	if {[winfo exists $w]} {destroy $w}
	toplevel $w -class Dialog
	wm withdraw $w
	wm title $w "Find BOVs (Buttons Over Video in Titles)"
	if {$::config(tk) != "aqua"} {wm transient $w .}
	if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
	wm protocol $w WM_DELETE_WINDOW {set ::menued::scan_done 0}
	bind $w <Escape> "set ::menued::scan_done 0"
	bind $w <Return> "set ::menued::scan_done 1"

	if {$msg != ""} {
		label $w.lab -text $msg
		pack  $w.lab -pady 4
	} else {
		frame $w.lab -height 4
		pack  $w.lab
	}

	frame $w.f
	pack  $w.f -padx 2 -expand 1 -fill both
	frame $w.f.f -bd 2 -relief groove
	pack  $w.f.f -padx 2 -expand 1 -fill both
	canvas $w.f.f.c
	pack   $w.f.f.c -side left -expand 1 -fill both
	frame  $w.f.f.c.f1
	$w.f.f.c create window 0 0 -window $w.f.f.c.f1 -anchor nw

	for {set vts 1} {$vts <= $::pgcs(numvts)} {incr vts} {
		set color black
		if {[info exists ::buttons::butsinfo(pgc,$vts,scanned)]} {
			switch $::buttons::butsinfo(pgc,$vts,scanned) {
				0 {
					set txt "Scan was aborted  ($::buttons::butsinfo(pgc,$vts,totalbuttons) buttons)"
					set ::menued::scanvts($vts) 1
					if {$::buttons::butsinfo(pgc,$vts,totalbuttons) > 0} {
						set color blue
					}
				}
				1 {
					set txt "Already scanned    ($::buttons::butsinfo(pgc,$vts,totalbuttons) buttons)"
					set ::menued::scanvts($vts) 0
					if {$::buttons::butsinfo(pgc,$vts,totalbuttons) > 0} {
						set color blue
					}
				}
				2 {
					set txt "Scan error!"
					set ::menued::scanvts($vts) 0
				}
			}
		} else {
			if {[get_int ::raw_sectors($vts,VTSI_MAT) 0x254 2]==0} {
				set txt "No Subpictures."
				set ::menued::scanvts($vts) 0
			} else {
				if {![file exists [file join $::dvddir "VTS_[format %02d $vts]_1.VOB"]]} {
					set txt "Missing VOB file!"
					set ::menued::scanvts($vts) 0
					set color red
				} else {
					set txt "Not yet scanned."
					set ::menued::scanvts($vts) 1
				}
			}
		}
		checkbutton $w.f.f.c.f1.c$vts -text "VTS $vts  ([expr {ceil([title_vob_sectors_used $dir $vts]/52428.8)/10.0}] GB.)  $txt" \
				-variable ::menued::scanvts($vts) -anchor w -pady 0 -bd $::config(min_bd) -foreground $color
		pack $w.f.f.c.f1.c$vts -fill x -anchor w -pady 0
	}

	update
	frame  $w.f.f.c.f1.lenpad -width 300
	pack   $w.f.f.c.f1.lenpad

	# add scrollbar if needed
	set h [winfo height $w.f.f.c.f1]
	#set maxheight [expr {[winfo screenheight .] - 200}]
	set maxheight 204
	if {$h > $maxheight} {
		set h $maxheight
		scrollbar $w.f.f.vs -command "$w.f.f.c yview" -orient vert
		pack   $w.f.f.vs -side left -fill y
		$w.f.f.c configure -yscrollcommand "$w.f.f.vs set"
	}
	$w.f.f.c configure -height $h

	$w.f.f.c configure -scrollregion "0 0 0 [winfo height $w.f.f.c.f1]" \
			-width [winfo width $w.f.f.c.f1]

	frame $w.f.a
	pack  $w.f.a -pady 2 -fill x
	button $w.f.a.i -text "Invert selection" -width 15 -pady 0 -command \
			{for {set vts 1} {$vts <= $::pgcs(numvts)} {incr vts} {set ::menued::scanvts($vts) [expr {!$::menued::scanvts($vts)}]}}
	pack   $w.f.a.i -side left -padx 2 -fill x -expand 1
	button $w.f.a.c -text "Select all"  -width 15 -pady 0 -command \
			{for {set vts 1} {$vts <= $::pgcs(numvts)} {incr vts} {set ::menued::scanvts($vts) 1}}
	pack   $w.f.a.c -side left -padx 2 -fill x -expand 1
	button $w.f.a.n -text "Select none" -width 15 -pady 0 -command \
			{for {set vts 1} {$vts <= $::pgcs(numvts)} {incr vts} {set ::menued::scanvts($vts) 0}}
	pack   $w.f.a.n -side left -padx 2 -fill x -expand 1

	frame $w.f2 -bd 2 -relief groove
	pack  $w.f2 -padx 4 -pady 2 -fill x
	frame $w.f2.f
	pack  $w.f2.f -fill x -expand 1
	label $w.f2.f.cb -text " Scan only one VOBU every"
	pack  $w.f2.f.cb -side left
	label $w.f2.f.slab -textvariable ::config(lbas_to_skip) -width 2
	pack  $w.f2.f.slab -side left
	label $w.f2.f.gblab -text "VOBUs ? "
	pack  $w.f2.f.gblab -side left
	scale $w.f2.scale -variable ::config(lbas_to_skip) -from 1 -to 30 -showvalue 0
	pack  $w.f2.scale -fill x -expand 1 -side top -padx 3
	label $w.f2.li -text " (The lower the better, but slower)" -anchor w
	pack  $w.f2.li -fill x -anchor w
	checkbutton $w.f2.ds -variable ::config(deep_BOV_scan) -onvalue true -offvalue false \
			-text "Deep-scan VTS with BOVs found during the first pass?  (Slower)"
	pack  $w.f2.ds -anchor w -pady 0
	checkbutton $w.f2.ms -variable ::config(check_multiple_BOVs_sets) -onvalue true -offvalue false \
			-text "Check for multiple sets of BOVs in the same cell?  (Slower)"
	pack  $w.f2.ms -anchor w -pady 0
	if {[info exists ::buttons::butsinfo(menu,multiplesets)] && $::buttons::butsinfo(menu,multiplesets)} {
		set ::config(check_multiple_BOVs_sets) true
		$w.f2.ms configure -foreground red
		::balloon::set_balloon $w.f2.ms "Recommended!\n(as there are multiple sets of menu buttons in the menu domains)"
	}

	checkbutton $w.cb -text "Automatically launch BOVs finder when a DVD is opened?" \
			-variable ::config(auto_launch_menubuttons_parser) -onvalue true -offvalue false
	pack  $w.cb -anchor w

	# buttons
	frame  $w.fb
	pack   $w.fb -padx 2 -pady 4 -anchor e
	button $w.fb.pause -text "Pause" -width 10 -command "set ::menued::scan_pause 1" -state disabled
	pack   $w.fb.pause -side left -padx 2
	button $w.fb.ok -text "Scan" -width 10 -command "set ::menued::scan_done 1; set ::menued::scan_pause 0"
	pack   $w.fb.ok -side left -padx 2
	button $w.fb.cancel -text "Cancel" -width 10 -command {set ::menued::scan_done 0}
	pack   $w.fb.cancel -side left -padx 2
	focus  $w.fb.ok
	if {$::config(platform) == "mac"} {
		pack [frame $w.fb.macpad] -side left -padx 6
	}

	update
	wm minsize $w [winfo reqwidth $w] [winfo reqheight $w]
#	wm maxsize $w [winfo reqwidth $w] [winfo reqheight $w]
	set wx [expr {[winfo rootx .]+(([winfo width  .]-[winfo reqwidth  $w])/2)}]
	set wy [expr {[winfo rooty .]+(([winfo height .]-[winfo reqheight $w])/2)-25}]
	if {$wx < 10} {set wx 10}
	if {$wy < 22} {set wy 22}
	wm geometry $w +$wx+$wy

	wm deiconify $w
	if {$::config(debug) == 0} {
		grab set $w
	}
	focus $w
	after 100 {set ::menued::pause 1}
	update idletasks
	tkwait variable ::menued::pause


	set startimmediately $autostart
	set ::menued::scan_done  1
	set ::menued::scan_pause 0
	while {$::menued::scan_done==1} {
		$w.fb.pause configure -state disabled
		if {$startimmediately} {
			set startimmediately false
		} else {
			tkwait variable ::menued::scan_done
		}
		$w.fb.pause configure -state normal

		set ::menued::scan_abort 0

		if {$::menued::scan_done} {
			set totalbuttons 0
			$w.fb.ok configure -text "Abort" -command "set ::menued::scan_abort 1; set ::menued::scan_pause 0; show_msg Aborted!"
			wm protocol $w WM_DELETE_WINDOW "set ::menued::scan_abort 1; set ::menued::scan_pause 0; show_msg Aborted!"
			$w.fb.cancel configure -state disabled
			set firstwithbuts 0
			$w.f.f.c yview moveto 0.0
			for {set vts 1} {$vts <= $::pgcs(numvts)} {incr vts} {
				update_progress $vts $::pgcs(numvts)
				if {$::menued::scanvts($vts)} {
					tolog "Scanning Title VOBs of VTS $vts for BOVs..."
					if {[expr {[winfo y $w.f.f.c.f1.c$vts]+[winfo height $w.f.f.c.f1.c$vts]}] > [winfo height $w.f.f.c]} {
						$w.f.f.c yview moveto [expr {(($vts-1)*1.0)/$::pgcs(numvts)}]
					}
					$w.f.f.c.f1.c$vts configure -background lightyellow -foreground black -command "set ::menued::scan_abort 1; after 100 {set ::menued::scan_done 1}"
					set numbuts [get_domain_buttons_infos_titles $dir $vts $::config(lbas_to_skip)]
					if {$firstwithbuts == 0 && $numbuts > 0} {
						set firstwithbuts $vts
					}
					if {[string first "Aborted" [$w.f.f.c.f1.c$vts cget -text]] == -1} {
						if {$numbuts>0 && $::config(lbas_to_skip)>1 && $::config(deep_BOV_scan)} {
							$w.f.f.c.f1.c$vts configure -background yellow -foreground blue
							add_title_buttons_marker $vts
							set numbuts [get_domain_buttons_infos_titles $dir $vts 1]
						}
						set ::menued::scanvts($vts) 0
					}
					$w.f.f.c.f1.c$vts configure -background $::config(systembuttonface) -command {}
					if {$numbuts > 0} {
						tolog "Found $numbuts BOVs in VTST domain of VTS $vts."
						$w.f.f.c.f1.c$vts configure -foreground blue
						incr totalbuttons $numbuts
						add_title_buttons_marker $vts
					}
				}
				update idletasks
				after 1 {set ::menued::pause 1}
				tkwait variable ::menued::pause
				if {$::menued::scan_abort} {
					break
				}
			}
			set processed true
			show_msg "${old_msg}Found a total of $totalbuttons BOVs."
			if {$firstwithbuts > 0} {
				$w.f.f.c yview moveto [expr {(($firstwithbuts-1)*1.0)/$::pgcs(numvts)}]
			} else {
				$w.f.f.c yview moveto 0.0
			}

			$w.fb.ok configure -state normal -text "Scan" -command "set ::menued::scan_done 1; set ::menued::scan_pause 0"
			wm protocol $w WM_DELETE_WINDOW {set ::menued::scan_done 0}
			$w.fb.cancel configure -state normal -text "Close"
			catch ::main::init_tree_menu_no_move
			if ($autoclose) {
				if {$processed} {
					set ::menued::scan_done 1
				} else {
					set ::menued::scan_done 0
				}
				destroy $w
				return
			}
		}
	}
	if {$processed} {
		set ::menued::scan_done 1
	} else {
		set ::menued::scan_done 0
	}
	destroy $w
	return $totalbuttons
}

#############################################################################
## Procedures:  add_menu/title_buttons_marker
## Get total number of buttons for all menu/title PGCs in the VTS.
## $::menus($vts,$lu,$pgc,numbuttons)	;# total number of buttons for this menu PGC.
## $::pgcs($vts,$pgc,numbuttons)		;# total number of buttons for this title PGC.
proc add_menu_buttons_marker {vts} {
	for {set lu 1} {$lu <= $::menus($vts,numlus)} {incr lu} {
		for {set pgc 1} {$pgc <= $::menus($vts,$lu,numpgcs)} {incr pgc} {
			set numcells $::menus($vts,$lu,$pgc,numcells)
			set numbuttons 0
			if {$numcells > 0} {
				set cellpostable $::menus($vts,$lu,$pgc,cellpostable)
				for {set cell 0} {$cell < $numcells} {incr cell} {
					set idx [expr {$cell * 4}]
					set vobid [get_int cellpostable $idx 2]
					incr idx 3
					set cellid [get_int cellpostable $idx 1]
					if {[info exists ::buttons::butsinfo(menu,$vts,$vobid,$cellid,numbuts)]} {
						incr numbuttons $::buttons::butsinfo(menu,$vts,$vobid,$cellid,numbuts)
					}
				}
			}
			set ::menus($vts,$lu,$pgc,numbuttons) $numbuttons
		}
	}
}
proc add_title_buttons_marker {vts} {
	for {set pgc 1} {$pgc <= $::pgcs($vts,numpgcs)} {incr pgc} {
		set numcells $::pgcs($vts,$pgc,numcells)
		set numbuttons 0
		if {$numcells > 0} {
			set cellpostable $::pgcs($vts,$pgc,cellpostable)
			for {set cell 0} {$cell < $numcells} {incr cell} {
				set idx [expr {$cell * 4}]
				set vobid [get_int cellpostable $idx 2]
				incr idx 3
				set cellid [get_int cellpostable $idx 1]
				if {[info exists ::buttons::butsinfo(pgc,$vts,$vobid,$cellid,numbuts)]} {
					incr numbuttons $::buttons::butsinfo(pgc,$vts,$vobid,$cellid,numbuts)
				}
			}
		}
		if {$numbuttons > 0} {
			set ::pgcs($vts,$pgc,numbuttons) $numbuttons
		} else {
			unset -nocomplain ::pgcs($vts,$pgc,numbuttons)
		}
	}
}

proc confirm_edit_several_sets {type vts vobid cellid {parentwin .menubuttons}} {
	if {[info exists ::buttons::butsinfo($type,$vts,$vobid,$cellid,edited)] && \
			$::buttons::butsinfo($type,$vts,$vobid,$cellid,edited)} {
		return true
	}
	if {[info exists ::buttons::butsinfo($type,$vts,$vobid,$cellid,numsets)] && \
			$::buttons::butsinfo($type,$vts,$vobid,$cellid,numsets) > 1} {
		set txt "The cell you want to edit has $::buttons::butsinfo($type,$vts,$vobid,$cellid,numsets) sets of buttons."
		set txt "$txt\nCurrently, PgcEdit supports only the first set of buttons."
		set txt "$txt\nIf you continue, you will overwrite the subsequent sets."
		set txt "$txt\nThe behaviour of the menu will no longer change over time."
		set txt "$txt\nThis can also lead to navigation problems, and you might"
		set txt "$txt\nlose easter egg buttons."
		set txt "$txt\n\nAre you sure you want to edit the menu of this cell?"
		if {! [winfo exists $parentwin]} {
			set parentwin .
		}
		set r [tk_messageBox -title "Menu Buttons/BOVs editor" -message $txt \
				-icon warning -type yesno -default no -parent $parentwin]
		if {$r == "no"} {
			return false
		}
	}
	return true
}

proc edit_menu_button {parentwin type vts vobid cellid group but} {
	if {$::menubuttons::command == ""} {bell; return false}
	if {$parentwin == ".menubuttons" && \
			! [confirm_edit_several_sets $type $vts $vobid $cellid $parentwin]} {
		return false
	}

	set ::hexcommand [string range $::menubuttons::command 1 23]

	set ::ced_return [::ced::cmdeditor $type $vts button]
	grab set $parentwin
	focus $parentwin
	if {$::ced_return == 1} {
		if {$::buttons::butsinfo($type,$vts,$vobid,$cellid,numgroups) > 1} {
			set equal true
			set cmd1 $::buttons::butsinfo($type,$vts,$vobid,$cellid,1,$but,command)
			for {set g 2} {$g <= $::buttons::butsinfo($type,$vts,$vobid,$cellid,numgroups)} {incr g} {
				if {$::buttons::butsinfo($type,$vts,$vobid,$cellid,$g,$but,command) != $cmd1} {
					set equal false
				}
			}
			if {$equal} {
				set r yes
			} else {
				set txt "Modify the command for the $::buttons::butsinfo($type,$vts,$vobid,$cellid,numgroups) groups?"
				set txt "$txt\nWarning: THE COMMANDS ARE DIFFERENT IN ALL GROUPS!"
				set r [tk_messageBox -title "Menu Button Editor" -icon warning \
						-message $txt -parent . -type yesnocancel -default no]
			}
			if {$r == "yes"} {
				set startgroup 1
				set endgroup $::buttons::butsinfo($type,$vts,$vobid,$cellid,numgroups)
			} elseif {$r == "no"} {
				set startgroup $group
				set endgroup   $group
			} else {
				return false
			}
		} else {
			set startgroup 1
			set endgroup   1
		}
		for {set g $startgroup} {$g <= $endgroup} {incr g} {
			#set ::buttons::butsinfo($type,$vts,$vobid,$cellid,$g,$but,autoaction) $::menubuttons::autoaction
			set ::buttons::butsinfo($type,$vts,$vobid,$cellid,$g,$but,command) $::hexcommand
			set ::buttons::curbutsinfo($vobid,$cellid,$g,$but,command) $::hexcommand
		}
		set ::menubuttons::command "\[$::hexcommand\]\n[::disasm::disasm $::hexcommand]"
		validate_command $::hexcommand .menubuttons.f2.labcmd

		wm title .menubuttons "Menu buttons  *"

		set ::buttons::butsinfo($type,edited) 1
		set ::buttons::butsinfo($type,$vts,edited) 1
		set ::buttons::butsinfo($type,$vts,$vobid,edited) 1
		set ::buttons::butsinfo($type,$vts,$vobid,$cellid,edited) 1
		::main::dvd_modified true true

		set index [lindex $::editor_cmds_from 1]
		::main::highlight_jump_cmd $type $index \[$::hexcommand\] 0 false true

		return true
	}
	return false
}

proc validate_command {hex w} {
	if {![winfo exists $w]} {return 1}
	if {$hex==""} {
		$w configure -background $::config(systembuttonface)
		return 1
	}
	set valid true
	set color pink
	if {[string range $hex 0 0]=="\["} {set hex [string range $hex 1 23]}
	if {[string first ILLEGAL [::disasm::disasm $hex]]>=0} {
		set valid false
	}
	if {$valid} {
		set type [lindex $::editor_cmds_from 0]
		set vts [lindex [split [lindex $::editor_cmds_from 1] ,] 0]
		set cmdname [::disasm::cmdname $hex]
		set valid [::ced::valid_command $cmdname $type $vts button]
		if {$valid} {
			set cmdname [::disasm::cmdname2 $hex]
			if {$cmdname!=""} {
				set valid [::ced::valid_command $cmdname $type $vts button]
			}
		}
		if {! $valid} {
			show_msg "Illegal button command for this domain!"
		} else {
			set color orange
			set target [::main::highlight_jump_cmd $type [lindex $::editor_cmds_from 1] \[$hex\] -1 false false]
			if {$target != ""} {
				set valid [lindex $target 4]
				if {!$valid} {
					show_msg "Target of button command not found!"
				}
			}
		}
	}
	if {!$valid} {
		$w configure -background $color
	} else {
		$w configure -background $::config(systembuttonface)
	}
	return $valid
}

proc fix_sethl_btn_pre_cmds {} {
	if {![info exists ::dvddir]} {
		bell
		return
	}
	set type [lindex $::editor_cmds_from 0]
	set indexes [split [lindex $::editor_cmds_from 1] ,]
	set vts [lindex $indexes 0]
	if {$type == "menu"} {
		set lu  [lindex $indexes 1]
		set pgc [lindex $indexes 2]
		set numcells $::menus($vts,$lu,$pgc,numcells)
		set cellpostable $::menus($vts,$lu,$pgc,cellpostable)
		set programmap   $::menus($vts,$lu,$pgc,programmap)
		set numbuttons   $::menus($vts,$lu,$pgc,numbuttons)
	} else {
		set pgc [lindex $indexes 1]
		if {$pgc == 0} {
			show_msg "There are no buttons in the First-Play PGC!"
			bell
			return
		}
		set numcells $::pgcs($vts,$pgc,numcells)
		set cellpostable $::pgcs($vts,$pgc,cellpostable)
		set programmap   $::pgcs($vts,$pgc,programmap)
		if [info exists ::pgcs($vts,$pgc,numbuttons)] {
			set numbuttons $::pgcs($vts,$pgc,numbuttons)
		} else {
			show_msg "This VTST domain has not been scanned for BOVs yet!"
			bell
			return
		}
	}
	if {$numcells==0} {
		show_msg "There are no buttons in a dummy PGC!"
		bell
		return
	}
	if {$numbuttons==0} {
		show_msg "No buttons detected in this PGC!"
		bell
		return
	}
	set txt "This function tries to detect the hidden buttons of"
	set txt "$txt\nthe current PGC, and modifies the pre-commands"
	set txt "$txt\nof the PGC to select a visible button by default,"
	set txt "$txt\ninstead of an hidden one."
	set txt "$txt\n\nYou should use this function only once per PGC,"
	set txt "$txt\nwhen all useless buttons have been hidden."
	set txt "$txt\n\nNote: In some cases, it is not possible to pre-select"
	set txt "$txt\na visible button automatically."
	set txt "$txt\n\nContinue ?"
	set r [tk_messageBox -title "Fix Hidden Buttons" -icon question -message $txt \
			-parent . -type yesno -default yes -skip "fix_sethl_btn_pre_cmds_usage"]
	if {$r == "no"} return

	show_msg "Fixing Hidden Buttons..."

	# build list of programs and chapters for each cell
	for {set idx 0} {$idx < [string length $programmap]} {incr idx} {
		set program([get_int programmap $idx 1]) [expr $idx+1]
	}
	set lastpg 0
	for {set cell 1} {$cell <= $numcells} {incr cell} {
		if {[info exists program($cell)]} {
			set lastpg $program($cell)
		} else {
			set program($cell) $lastpg
		}
	}
	if {$type == "pgc"} {
		foreach i $::pgcs($vts,$pgc,pttlist) {
			set chapter([lindex $i 1]) [lindex $i 0]	;# chapter(pg) = $ptt
		}
		set lastptt 0
		for {set idx 0} {$idx < [string length $programmap]} {incr idx} {
			set pg [expr $idx+1]
			if {[info exists chapter($pg)]} {
				set lastptt $chapter($pg)
			} else {
				set chapter($pg) $lastptt
			}
		}
	}

	# get pre-commands
	set pgcvar ::[lindex $::editor_cmds_from 0]s([lindex $::editor_cmds_from 1])
	set cmds [get_type_cmds pre $pgcvar]
	set postcmds [get_type_cmds post $pgcvar]
	set cellcmds [get_type_cmds cell $pgcvar]
	set firstcellprocessed false

	set lastpttwbutton  0
	set lastpgwbutton   0
	set lastcellwbutton 0
	set log ""
	set logicon info
	for {set cell 1} {$cell <= $numcells} {incr cell} {
		set cellidx [expr ($cell-1)*4]
		set vid [get_int cellpostable $cellidx 2]
		incr cellidx 3
		set cid [get_int cellpostable $cellidx 1]
		set numbuts   $::buttons::butsinfo($type,$vts,$vid,$cid,numbuts)
		if {$numbuts == 0} continue
		if {$type == "pgc"} {
			set curptt $chapter($program($cell))
		} else {
			set curptt $cell
		}
		set lastpttwobutton  [expr $lastpttwbutton+1]
		set lastpgwobutton   [expr $lastpgwbutton+1]
		set lastcellwobutton [expr $lastcellwbutton+1]
		set lastpttwbutton  $curptt
		set lastpgwbutton   $program($cell)
		set lastcellwbutton $cell

		set ishidden(0) false
		for {set b 1} {$b <= $numbuts} {incr b} {
			set ishidden($b) false
		}
		set numhidden 0
		set lasthidden 0
		for {set b 1} {$b <= $numbuts} {incr b} {
			if {$::buttons::butsinfo($type,$vts,$vid,$cid,1,$b,coords) == "1 1 - 2 2" || \
					$::buttons::butsinfo($type,$vts,$vid,$cid,1,$b,autoaction)} {
				set ishidden($b) true
				incr numhidden
				set lasthidden $b
			}
		}
		if {$numhidden == 0} {
			set firstcellprocessed true
			set log "$log\nCell $cell: No hidden buttons detected.  Nothing to do."
			continue
		}
		if {$numhidden == $numbuts} {
			set firstcellprocessed true
			set log "$log\nCell $cell: Warning: All buttons are hidden or auto-activating."
			set logicon warning
			continue
		}

		set ok true
		if {$::buttons::butsinfo($type,$vts,$vid,$cid,forceselectbutn) > $numbuts || \
			$::buttons::butsinfo($type,$vts,$vid,$cid,forceactionbutn) > $numbuts || \
			$ishidden($::buttons::butsinfo($type,$vts,$vid,$cid,forceselectbutn)) || \
			$ishidden($::buttons::butsinfo($type,$vts,$vid,$cid,forceactionbutn))} {

			set msg "Warning:  Cell $cell"
			if {$ishidden($::buttons::butsinfo($type,$vts,$vid,$cid,forceselectbutn))} {
				set msg "$msg\nThe selected button is forced to the hidden button $::buttons::butsinfo($type,$vts,$vid,$cid,forceselectbutn) in Cell $cell."
				set log "$log\nCell $cell: Warning: The selected button is forced to the hidden button $::buttons::butsinfo($type,$vts,$vid,$cid,forceselectbutn)."
				set logicon warning
			}
			if {$ishidden($::buttons::butsinfo($type,$vts,$vid,$cid,forceactionbutn))} {
				set msg "$msg\nThe action button is forced to the hidden button $::buttons::butsinfo($type,$vts,$vid,$cid,forceactionbutn) in Cell $cell."
				set log "$log\nCell $cell: Warning: The action button is forced to the hidden button $::buttons::butsinfo($type,$vts,$vid,$cid,forceselectbutn)."
				set logicon warning
			}
			set msg "$msg\n\nYou should manually edit the forced/action button(s) in the advanced section"
			set msg "$msg\nto use a non-hidden button instead."
			set msg "$msg\n\nThe default highlighted button for cell $cell will not be modified."
			set r [tk_messageBox -message $msg -parent . -title "Fix Hidden Buttons" \
					-icon question -type Continue|Cancel -default cancel]
			if {$r == "cancel"} return
			set ok false
		}
		if {$::buttons::butsinfo($type,$vts,$vid,$cid,forceselectbutn) <= $numbuts && \
				$::buttons::butsinfo($type,$vts,$vid,$cid,forceselectbutn) > 0} {
			set firstcellprocessed true
			if {$ok} {
				set log "$log\nCell $cell: A visible button is forced as the default highlighted button.  Nothing to do."
			}
			continue
		}

		# When the first cell with buttons is encountered,
		# adds default SetHL_BTN commands at the end of the PGC
		set gprmtouse -1
		if {!$firstcellprocessed} {
			set firstcellprocessed true

			if {![info exists ::menued::fix_sethl_btn_pre_cmds_gprm] ||
					([info exists ::menued::fix_sethl_btn_pre_cmds_gprm] && \
					$::menued::fix_sethl_btn_pre_cmds_gprm == -1)} {
				for {set gprm 0} {$gprm < 16} {incr gprm} {
					if {[info exists ::gprm_labels($gprm)] && $::gprm_labels($gprm)=="hidden HL_Btn control"} {
						set ::menued::fix_sethl_btn_pre_cmds_gprm $gprm
						set ::menued::fix_sethl_btn_pre_cmds_dvddir $::dvddir
						break
					}
				}
			}
			if {[info exists ::menued::fix_sethl_btn_pre_cmds_gprm] && \
					$::menued::fix_sethl_btn_pre_cmds_gprm != -1 && \
					$::menued::fix_sethl_btn_pre_cmds_dvddir == $::dvddir} {
				if {[info exists ::menued::fix_sethl_btn_pre_cmds_yestoall] && \
						$::menued::fix_sethl_btn_pre_cmds_yestoall} {
					set gprmtouse $::menued::fix_sethl_btn_pre_cmds_gprm
				} else {
					set msg "For best results, this function requires a free GPRM."
					set msg "$msg\n\nYou have already applied this function on this DVD."
					set msg "$msg\nThe last time, GPRM $::menued::fix_sethl_btn_pre_cmds_gprm was used."
					set msg "$msg\nNormally, you should use that GPRM again."
					set msg "$msg\n\nDo you want to use GPRM $::menued::fix_sethl_btn_pre_cmds_gprm again?"
					set r [tk_messageBox -message $msg -parent . -title "Fix Hidden Buttons" \
						-type "Yes to All|Yes|No|Cancel" -default yes -icon question]
					set ::menued::fix_sethl_btn_pre_cmds_yestoall false
					if {$r == "cancel"} return
					if {$r == "yes"} {
						set gprmtouse $::menued::fix_sethl_btn_pre_cmds_gprm
					}
					if {$r == "yes to all"} {
						set gprmtouse $::menued::fix_sethl_btn_pre_cmds_gprm
						set ::menued::fix_sethl_btn_pre_cmds_yestoall true
					}
				}
			}

			if {$gprmtouse == -1} {
				# search for a free GPRM
				set msg "For best results, this function requires a free GPRM."
				set msg "$msg\n\nPgcEdit can search for a free GPRM automatically, but you might prefer to select yourself a"
				set msg "$msg\nGPRM that is already used in the DVD, to keep the free GPRMs for your own use."
				set msg "$msg\nIf you have already applied this function to the same DVD, you should use the same GPRM."
				set msg "$msg\nYou can also continue without selecting a free GPRM, but most of the time it's not sufficient"
				set msg "$msg\nto pre-select a non-hidden button."

				set msg "$msg\n\nWARNING!"
				set msg "$msg\n\nIf you select a GPRM yourself, take care!  You must know what you are doing!"
				set msg "$msg\nThe content of the GPRM is modified in the pre-commands of the current PGC."
				set msg "$msg\nPgcEdit tries to restore its value, but that works only when the GPRM value is LESS THAN"
				set msg "$msg\n1024 when the PGC is executed."
				set msg "$msg\n\nTherefore, you should select either :"
				set msg "$msg\n\n- the GPRM that has been used during a previous call to this function on the same DVD;"
				set msg "$msg\n\n- a GPRM that is used only once    (searched by the automatic mode);"
				set msg "$msg\n\n- a free GPRM, that is never used  (searched by the automatic mode);"
				set msg "$msg\n\n- a GPRM that stores ONLY values less or equal to 1023  (for experienced users!)"
				set msg "$msg\n  Note: the GPRMs used by the Jump to PGC and the Import FP and Intro Clips functions"
				set msg "$msg\n  (labelled \"Jump2PGC_control\", \"FP_Clip_control\" and \"T*_IntroClip_control\") are safe;"
				set msg "$msg\n\n- or a GPRM that is not used in the current PGC, AND is used in the other PGCs only as a"
				set msg "$msg\n  temporary variable, always re-initialized just before it is used!  (for experienced users!)"
				set msg "$msg\n  Note: Use the trace mode to verify if a GRRM is suitable for this usage."

				set r [tk_messageBox -message $msg -parent . -title "Fix Hidden Buttons" \
						-type "Find a GPRM automatically|Select a GPRM myself|Continue without GPRM|Cancel" \
						-default "select a gprm myself" -icon warning \
						-skip fix_sethl_btn_pre_cmds_select_gprm]
				if {$r == "cancel"} return
				if {$r == "select a gprm myself"} {
					set msg "You haven't selected a GPRM."
					set gprmtouse [::utils::set_gprm_labels -1 . false true]
				} elseif {$r == "continue without gprm"} {
					set gprmtouse -2
				} else {
					set msg "PgcEdit cannot find a GPRM used only once or free."
					set gprmusage [::macros::find_unused_gprms true true]
					set gprmtouse [lsearch $gprmusage 1]
					if {$gprmtouse == -1} {
						set gprmtouse [lsearch $gprmusage 0]
					}
				}
				if {$gprmtouse == -1} {
					set msg "$msg\nThe function cannot be applied completely."

					set msg "$msg\n\nHowever, you can choose to continue anyway."
					set msg "$msg\nIn this case, only some link commands and"
					set msg "$msg\ndirect SetHL_BTN commands will be modified."
					set msg "$msg\nIt's not always sufficient to highlight the right"
					set msg "$msg\nbutton, but you can try."

					set msg "$msg\n\nYou can also free a GPRM manually and retry."

					set msg "$msg\n\nContinue without a free GPRM?"
					set r [tk_messageBox -message $msg -parent . \
							-title "Fix Hidden Buttons" -icon question \
							-type yesno -default yes]
					if {$r == "no"} return
				}
			}
			if {$gprmtouse >= 0} {
				set ::menued::fix_sethl_btn_pre_cmds_gprm $gprmtouse
				set ::menued::fix_sethl_btn_pre_cmds_dvddir $::dvddir
				# label the GPRM used for this function
				if {$::config(allow_auto_comments) && \
						(![info exists ::gprm_labels($gprmtouse)] || $::gprm_labels($gprmtouse)=="")} {
					for {set gprm 0} {$gprm < 16} {incr gprm} {
						if {[info exists ::gprm_labels($gprm)] && $::gprm_labels($gprm)=="hidden HL_Btn control"} {
							set ::gprm_labels($gprm) ""
						}
					}
					set ::gprm_labels($gprmtouse) "hidden HL_Btn control"
					set ::pgc_comments_modified true
				}

				set hexgprm [format %02X $gprmtouse]

				# add a NOP command at the end
				if {[llength $cmds]>0 && [lindex $cmds end]!={00 00 00 00 00 00 00 00}} {
					lappend cmds {00 00 00 00 00 00 00 00}	;# NOP
				}
				set gotocmd [llength $cmds]
				# replace Break commands by GoTos
				set idxs [lsearch -regexp -all $cmds {00 .2 .. .. .. .. .. ..}]	;# Break
				if {[llength $idxs] > 0} {
					foreach idx $idxs {
						set bcmd [lindex $cmds $idx]
						set gcmd [string range $bcmd 0 3]1[string range $bcmd 5 20][format %02X $gotocmd]
						lset cmds $idx $gcmd
					}
				}
				# replace Links to first cell and w/o HL_BTN by GoTos
				set idxs [lsearch -regexp -all $cmds {20 .[567] 00 0. .. .. 0. ..}]	;# LinkCN 1 and LinkPGN 1
				if {[llength $idxs] > 0} {
					foreach idx $idxs {
						set gcmd [lindex $cmds $idx]
						set but [expr 0x[lindex $gcmd 6] & 0xFC]
						if {$but == 0} {
							set doit false
							switch [string range [lindex $gcmd 1] 1 1] {
								5 {	;# LinkPTT
									if {[expr 0x[lindex $gcmd 6][lindex $gcmd 7] & 0x3FF] \
											== $chapter($program(1))} {
										set doit true
									}
								}
								6 {	;# LinkPGN
									if {[expr 0x[lindex $gcmd 7]] == $program(1)} {
										set doit true
									}
								}
								7 {	;# LinkCN
									if {[expr 0x[lindex $gcmd 7]] == 1} {
										set doit true
									}
								}
							}
							if {$doit} {
								lset gcmd 0 00
								lset gcmd 1 [string range [lindex $gcmd 1] 0 0]1
								lset gcmd 6 00
								lset gcmd 7 [format %02X $gotocmd]
								lset cmds $idx $gcmd
							}
						}
					}
				}

				# append LinkPGN 1 + SetHL_BTN commands
				lappend cmds "79 00 00 $hexgprm 03 FF 00 00"	;# Set gprm(free) &=(and) 1023
				lappend cmds "6A 00 00 $hexgprm 00 88 00 00"	;# Set gprm(free) |=(or) sprm(8:Highlighted button number)

				lappend cmds "79 C0 $hexgprm $hexgprm 03 FF [format %02X [expr ($numbuttons+1)*4]] 00"	;# if ( gprm(free) > last_button ) { set gprm(free) &= 0x03FF }
				set targetln [expr [llength $cmds]+$lasthidden+$numhidden+3]
				lappend cmds "00 F1 00 $hexgprm 04 00 00 [format %02X $targetln]"	;# if ( gprm(free) < 1024 ) { Goto line N }

				set targetln1 [expr [llength $cmds]+$lasthidden+1]
				set targetln  [expr $targetln1+1]
				for {set b 1} {$b <= $lasthidden} {incr b} {
					set butval [expr 1023+($b*1024)]
					if {$ishidden($b)} {
						;# if ( gprm(free) <= <hidden_button> ) then { Goto line <target> }
						lappend cmds "00 E1 00 $hexgprm [format %02X [expr $butval/0x100]] [format %02X [expr $butval&0xFF]] 00 [format %02X $targetln]"
						incr targetln
					} else {
						lappend cmds "00 E1 00 $hexgprm [format %02X [expr $butval/0x100]] [format %02X [expr $butval&0xFF]] 00 [format %02X $targetln1]"
					}
				}
				lappend cmds "79 06 00 $hexgprm 03 FF 00 [format %02X $program(1)]" ;# Set gprm(free) &=(and) 1023
				for {set b 1} {$b <= $lasthidden} {incr b} {
					if {$ishidden($b)} {
						;# Set gprm(free) &=(and) 1023 ; LinkPGN Program <target_PG>, button <non_hidden_button>
						lappend cmds "79 06 00 $hexgprm 03 FF [format %02X [expr [replacement_button $type $vts $vid $cid $b ishidden $numbuts]*4]] [format %02X $program(1)]"
					}
				}
				;# Set gprm(free) &=(and) 1023 ; LinkPGN Program <target_PG>, button <first_non_hidden_button>
				lappend cmds "79 06 00 $hexgprm 03 FF [format %02X [expr [replacement_button $type $vts $vid $cid 1 ishidden $numbuts]*4]] [format %02X $program(1)]"
			}
		}

		# Modify the links to the current cell + HL_BTN parameter
		set cmds [replace_commands $type $vts $vid $cid $cmds ishidden $numbuts \
				$curptt $program($cell) $cell $lastpttwobutton $lastpgwobutton $lastcellwobutton]
		set postcmds [replace_commands $type $vts $vid $cid $postcmds ishidden $numbuts \
				$curptt $program($cell) $cell $lastpttwobutton $lastpgwobutton $lastcellwobutton]
		set cellcmds [replace_commands $type $vts $vid $cid $cellcmds ishidden $numbuts \
				$curptt $program($cell) $cell $lastpttwobutton $lastpgwobutton $lastcellwobutton]
		for {set gr 1} {$gr <= $::buttons::butsinfo($type,$vts,$vid,$cid,numgroups)} {incr gr} {
			for {set bu 1} {$bu <= $::buttons::butsinfo($type,$vts,$vid,$cid,numbuts)} {incr bu} {
				set butcmd [lindex [replace_commands $type $vts $vid $cid \
						[list $::buttons::butsinfo($type,$vts,$vid,$cid,$gr,$bu,command)] \
						ishidden $numbuts $curptt $program($cell) $cell \
						$lastpttwobutton $lastpgwobutton $lastcellwobutton] 0]
				if {$butcmd != $::buttons::butsinfo($type,$vts,$vid,$cid,$gr,$bu,command)} {
					set ::buttons::butsinfo($type,$vts,$vid,$cid,$gr,$bu,command) $butcmd
					set ::buttons::butsinfo($type,$vts,$vid,$cid,edited) true
				}
			}
		}
	}

	# replace commands
	set_type_cmds pre  $pgcvar $cmds
	set_type_cmds post $pgcvar $postcmds
	set_type_cmds cell $pgcvar $cellcmds

	if {[info exists gprmtouse]} {
		::main::dvd_modified true false
		# refresh GUI
		set indexes [split [lindex $::editor_cmds_from 1] ","]
		if {[string first "menu" $pgcvar] != -1} {
			::main::display_pgc menu [lindex $indexes 0] [lindex $indexes 1] [lindex $indexes 2]
		} else {
			::main::display_pgc pgc [lindex $indexes 0] [lindex $indexes 1]
		}
		if {$gprmtouse >= 0} {
			show_msg "Fix Hidden Buttons applied successfully.  (Using GPRM $gprmtouse)"
		} else {
			show_msg "Fix Hidden Buttons Subset applied successfully.  (Using no GPRM)"
		}
	} else {
		set log "$log\n\nPGC:  Nothing to do for this PGC.  Pre-commands not modified."
		show_msg "Fix Hidden Buttons: Nothing to do for this PGC."
	}
	if {$log != ""} {
		if {$logicon == "warning"} {
			set log "Some cells cannot be successfully processed:\n$log"
		} else {
			set log "Some cells do not need to be processed:\n$log"
		}
		tk_messageBox -message $log -parent . -title "Fix Hidden Buttons" -icon $logicon
	}
	return
}
proc replace_commands {type vts vid cid cmds ishidden_upvar numbuts curptt curpg curcell lastpttwobutton lastpgwobutton lastcellwobutton} {
	upvar $ishidden_upvar ishidden

	# SetHL_BTN + LinkPTTN, LinkPGN or LinkCN
	set idxs [lsearch -regexp -all $cmds {56 0[5-7] .. .. .. .. .. ..}]
	if {[llength $idxs] > 0} {
		foreach idx $idxs {
			set cmd [lindex $cmds $idx]
			set but [expr 0x[lindex $cmd 4] / 4]
			if {$but == 0 || $but > $numbuts || !$ishidden($but)} continue
			set t [expr 0x[lindex $cmd 1]]
			set pcn [expr 0x[lindex $cmd 6][lindex $cmd 7] & 0x03FF]
			if {$t == 5 && ($pcn < $lastpttwobutton  || $pcn > $curptt)} continue
			if {$t == 6 && ($pcn < $lastpgwobutton   || $pcn > $curpg)}  continue
			if {$t == 7 && ($pcn < $lastcellwobutton || $pcn > $curcell)}   continue
			set targetbtn [replacement_button $type $vts $vid $cid $but ishidden $numbuts]
			if {$targetbtn > 0} {
				lset cmd 4 [format %02X [expr $targetbtn*4]]
				lset cmds $idx $cmd
			}
		}
	}
	# LinkCN and Set* + LinkCN
	set idxs [lsearch -regexp -all $cmds {20 .7 .. .. .. .. .. ..}]	;# LinkCN
	set idxs [concat $idxs [lsearch -regexp -all $cmds {[45][1-4] 07 .. .. .. .. .. ..}]] ;# SetSTN,SetNVTMR,SetGPRMMD,SetAMXMD + LinkCN
	set idxs [concat $idxs [lsearch -regexp -all $cmds {[67]1 07 .. .. .. .. .. ..}]] ;# Set + LinkCN
	if {[llength $idxs] > 0} {
		foreach idx $idxs {
			set cmd [lindex $cmds $idx]
			set but [expr 0x[lindex $cmd 6] / 4]
			if {$but == 0 || $but > $numbuts || !$ishidden($but)} continue
			set cn  [expr 0x[lindex $cmd 7]]
			if {$cn < $lastcellwobutton || $cn > $curcell} continue
			set targetbtn [replacement_button $type $vts $vid $cid $but ishidden $numbuts]
			if {$targetbtn > 0} {
				lset cmd 6 [format %02X [expr $targetbtn*4]]
				lset cmds $idx $cmd
			}
		}
	}
	# LinkPGN and Set* + LinkPGN
	set idxs [lsearch -regexp -all $cmds {20 .6 .. .. .. .. .. ..}]	;# LinkPGN
	set idxs [concat $idxs [lsearch -regexp -all $cmds {[45][1-4] 06 .. .. .. .. .. ..}]] ;# SetSTN,SetNVTMR,SetGPRMMD,SetAMXMD + LinkPGN
	set idxs [concat $idxs [lsearch -regexp -all $cmds {[67]1 06 .. .. .. .. .. ..}]] ;# Set + LinkPGN
	if {[llength $idxs] > 0} {
		foreach idx $idxs {
			set cmd [lindex $cmds $idx]
			set but [expr 0x[lindex $cmd 6] / 4]
			if {$but == 0 || $but > $numbuts || !$ishidden($but)} continue
			set pg  [expr 0x[lindex $cmd 7]]
			if {$pg < $lastpgwobutton || $pg > $curpg} continue
			set targetbtn [replacement_button $type $vts $vid $cid $but ishidden $numbuts]
			if {$targetbtn > 0} {
				lset cmd 6 [format %02X [expr $targetbtn*4]]
				lset cmds $idx $cmd
			}
		}
	}
	# LinkPTTN and Set* + LinkPTTN
	if {$type == "pgc"} {
		set idxs [lsearch -regexp -all $cmds {20 .5 .. .. .. .. .. ..}]	;# LinkPTTN
		set idxs [concat $idxs [lsearch -regexp -all $cmds {[45][1-4] 05 .. .. .. .. .. ..}]] ;# SetSTN,SetNVTMR,SetGPRMMD,SetAMXMD + LinkPTTN
		set idxs [concat $idxs [lsearch -regexp -all $cmds {[67]1 05 .. .. .. .. .. ..}]] ;# Set + LinkPTTN
		if {[llength $idxs] > 0} {
			foreach idx $idxs {
				set cmd [lindex $cmds $idx]
				set but [expr 0x[lindex $cmd 6] / 4]
				if {$but == 0 || $but > $numbuts || !$ishidden($but)} continue
				set ptt [expr 0x[lindex $cmd 6][lindex $cmd 7] & 0x03FF]
				if {$ptt < $lastpttwobutton || $ptt > $curptt} continue
				set targetbtn [replacement_button $type $vts $vid $cid $but ishidden $numbuts]
				if {$targetbtn > 0} {
					lset cmd 6 [format %02X [expr $targetbtn*4]]
					lset cmds $idx $cmd
				}
			}
		}
	}
	return $cmds
}
proc replacement_button {type vts vid cid b ishidden_upvar numbuts} {
	upvar $ishidden_upvar ishidden
	if {$b > $numbuts} {return 0}
	set targetbtn 0
	for {set b2 $b} {$b2 <= $numbuts} {incr b2} {
		if {!$ishidden($b2) && !$::buttons::butsinfo($type,$vts,$vid,$cid,1,$b2,autoaction) && \
				[lsearch $::buttons::butsinfo($type,$vts,$vid,$cid,1,$b,udlr) $b2]!=-1} {
			set targetbtn $b2
			break
		}
	}
	if {$targetbtn == 0} {
		for {set b2 $b} {$b2 > 0} {incr b2 -1} {
			if {!$ishidden($b2) && !$::buttons::butsinfo($type,$vts,$vid,$cid,1,$b2,autoaction) && \
					[lsearch $::buttons::butsinfo($type,$vts,$vid,$cid,1,$b,udlr) $b2]!=-1} {
				set targetbtn $b2
				break
			}
		}
	}
	if {$targetbtn == 0} {
		for {set b2 $b} {$b2 <= $numbuts} {incr b2} {
			if {!$ishidden($b2) && !$::buttons::butsinfo($type,$vts,$vid,$cid,1,$b2,autoaction)} {
				set targetbtn $b2
				break
			}
		}
	}
	if {$targetbtn == 0} {
		for {set b2 $b} {$b2 > 0} {incr b2 -1} {
			if {!$ishidden($b2) && !$::buttons::butsinfo($type,$vts,$vid,$cid,1,$b2,autoaction)} {
				set targetbtn $b2
				break
			}
		}
	}
	return $targetbtn
}

proc hide_button_help {} {
	set txt "When a button is useless because you have removed the material it is linked to,"
	set txt "${txt}\nit should be either deleted or hidden."
	set txt "${txt}\n\nSince the buttons are accessed by their button numbers, deleting for example"
	set txt "${txt}\nbutton 1 changes the numbers of the subsequent buttons.  The right button"
	set txt "${txt}\nmight not be selected by default any more, and a non-existent button might even"
	set txt "${txt}\nbe selected by default.  That can cause the player to crash!"
	set txt "${txt}\n\nIt is therefore recommended to hide the button instead of deleting it."
	set txt "${txt}\nWhen a button is hidden, you cannot select it any more.  However, that button"
	set txt "${txt}\ncan still be selected by default with the Force Select Button parameter (located"
	set txt "${txt}\nin the advanced controls of this menu editor) or by means of a SetHL_BTN"
	set txt "${txt}\ncommand executed before the menu is displayed."
	set txt "${txt}\n\nUsually, the best method to hide a button is \"Jump to non-deleted button\", as if,"
	set txt "${txt}\nper chance, that button is selected by default, activating that hidden button with"
	set txt "${txt}\nthe remote will select another, non-hidden button automatically."
	set txt "${txt}\n\nTo be sure that a non-hidden button is selected by default, you can force it with"
	set txt "${txt}\nthe \"force select button\" advanced control at the bottom of the menu editor."
	set txt "${txt}\nOf course, the same button will always be selected when the menu is opened."
	set txt "${txt}\n\nYou can also use the function \"Select a non-hidden button/BOV by default\" on"
	set txt "${txt}\nthe current PGC, when all useless buttons have been hidden."
	set txt "${txt}\nThis function modifies the pre-commands of the PGC and tries to select the best"
	set txt "${txt}\nreplacement button when an hidden button is selected by default."
	set txt "${txt}\nSee that function in the Menu menu of the main PgcEdit window for more info."
	tk_messageBox -title "Hide button: help" -message $txt -parent .menued
}

} ;# end namespace ::menued
